
	; Copyright (C) 1990,1993,1994 by Motorola, Inc.

	; implementation of the data structures and functions in signal.h

	opt	noidw
	section E709signal

	; C signal handling architecture:

	; 1) the sig, where sig is the int argument to signal, is the offset
	;    of the 2 word interrupt vector in the 56k hardware interrupt
	;    table. (e.g., 0x3e == illegal instruction.)

	; 2) when an interrupt is to be handled by the C signal facility:
	;       the HW vector is set to be a bsr to __c_sig_goto_dispatch 
	;       + sig. this is a table filled with the instruction bsr 
	;       __c_sig_dispatch. This enables __c_sig_dispatch to decode
	;       the signal number from the ssh (each HW vector is associated
	;       with a single slot in the __c_sig_goto_dispatch table).
	;
	; 3) __c_sig_dispatch will save all of the context assoicated with
	;    the CPU, (the resgiters that the C compiler treats as scratch),
	;    such as the a register, n registers, etc. it will then load
	;    the function address from the table __c_sig_handlers (indexed
	;    by the signal number), and bsr to it. if the signal handler
	;    returns, __c_sig_dispatch restores the CPU state that it saved,
	;    and does an rti.



	; void (* signal ( int sig, void ( * func ) ( int ))) ( int )
	;
	; a == sig
	; b == function pointer
	
	org	p:

Fsignal
	global	Fsignal

	move	b1,x0			; save function pointer.
	move	x:F__max_signal,b	; check to make sure that
	move	a1,r0
	cmpu	b,a			; the signal is in range.
	ble	<_continue

	move	#F__sig_err,a
	tst	a
	rts

_continue
	move	#>$0bf080,x1		; load the absolute bsr instruction.
	move	x1,p:(r0)+		; move the bsr into the interrupt table
	move	#F__c_sig_handlers,n0	; load the address of the C routine vec
	move	#F__c_sig_goto_dispatch,b	; add the goto table base addr.
	add	b,a			; to the signal number.
	move	a1,p:(r0)+n0		; jam address into the HW table
	move	x:-(r0),a		; retrieve the previous handler address
	tst	a	x0,x:(r0)	; install the new handler.
	rts


	; void raise ( int sig )
	;
	; the stack looks like this on entry:
	;
	; a == sig

Fraise
	global	Fraise

	move	x:F__max_signal,b	; check for a vaild signal.
	cmp	b,a	a,r0		; copy signal for invocation.
	ble	<_continue

	not	a
	nop
	move	a1,a
	rts

_continue
	move	#F__c_sig_goto_dispatch,n0
	nop
	move	(r0)+n0
	jsr	(r0)			; simulate an interrupt.
	clr	a
	rts

	; SIG_DFL:
	;
	; the default behavior is to ignore the signal, but keep track 
	; of how many signals have been dropped in __sig_drop_count.

	org	x:
F__sig_drop_count
	global	F__sig_drop_count

	dc	0		; the number of signals that have been
				; handled by SIG_DFL.

	org	p:

	; we ignore the single argument, sig.

F__sig_dfl
	global	F__sig_dfl

	move	x:F__sig_drop_count,a
	add	#1,a
	move	a,x:F__sig_drop_count
	rts

	; SIG_ERR:
	;
	; we simply goto abort.

F__sig_err
	global	F__sig_err

	bra	Fabort


	; SIG_IGN:
	;
	; just return.

F__sig_ign
	global	F__sig_ign

	rts

F__c_sig_dispatch
	global	F__c_sig_dispatch

	move	(r6)+		; we must bump stack by 1 to be safe.
	move	r3,x:(r6)+	; save r3.
	move	ssh,r3		; r3 has the signal number + 2
	move	n3,x:(r6)+	; save n3.
	move	#F__c_sig_goto_dispatch+2,n3
	nop
	move	(r3)-n3		; now r3 really has the sig.
	move	n2,x:(r6)+

	move	#F__c_sig_handlers,n3	; n3 points to the base of the 
					; signal handler table.

	move	n0,x:(r6)+	; save n0, load it with SIG_DFL
	move	#F__sig_dfl,n0
	move	r3,n2		; keep the signal # as an argument to the 
				; handler.
	move	(r3)+n3		; r3 points to the address of the C 
				; signal handler.
	move	x:(r3),n3	; n3 has the address of the signal handler.
	nop
	move	n0,x:(r3)	; install SIG_DFL to catch a second signal.
	move	n3,r3		; r3 has the address of our handler.

	move	ssl,x:(r6)+	; save the context of the interrupt.
	move	ssh,x:(r6)+
	move	r0,x:(r6)+
	move	r1,x:(r6)+
	move	n1,x:(r6)+
	move	r4,x:(r6)+
	move	n4,x:(r6)+
	move	r5,x:(r6)+
	move	n5,x:(r6)+
	move	n6,x:(r6)+
	move	x1,x:(r6)+
	move	x0,x:(r6)+
	move	b2,x:(r6)+
	move	b1,x:(r6)+
	move	b0,x:(r6)+
	move	a2,x:(r6)+
	move	a1,x:(r6)+
	move	a0,x:(r6)+

	move	n2,x:(r6)+
	jsr	(r3)		; call the handler.
	move	(r6)-

	move	(r6)-
		
	move	x:(r6)-,a0	; assuming that the handler returns,
				;  restore the context.
	move	x:(r6)-,a1
	move	x:(r6)-,a2
	move	x:(r6)-,b0
	move	x:(r6)-,b1
	move	x:(r6)-,b2
	move	x:(r6)-,x0
	move	x:(r6)-,x1
	move	x:(r6)-,n6
	move	x:(r6)-,n5
	move	x:(r6)-,r5
	move	x:(r6)-,n4
	move	x:(r6)-,r4
	move	x:(r6)-,n1
	move	x:(r6)-,r1
	move	x:(r6)-,r0
	move	x:(r6)-,ssh
	move	x:(r6)-,ssl
	move	x:(r6)-,n0
	move	x:(r6)-,n2
	move	x:(r6)-,n3
	move	x:(r6)-,r3
	rti			; return from the interrupt.

	org	p:

F__c_sig_goto_dispatch
	global  F__c_sig_goto_dispatch

	dup	$3e
	bsr	>F__c_sig_dispatch
	endm

	org	x:

F__c_sig_handlers
	global	F__c_sig_handlers

	dup	$3e
	dc	F__sig_dfl
	dc	0
	endm

	endsec
