/* Copyright (C) 1995, by Motorola, Inc. */

#include <math.h>
#include <errno.h>
#include "trans.h"

#define _POLY1(x, c)    ((c)[0] * (x) + (c)[1])
#define _POLY2(x, c)    (_POLY1((x), (c)) * (x) + (c)[2])
#define _POLY3(x, c)    (_POLY2((x), (c)) * (x) + (c)[3])
#define _POLY4(x, c)    (_POLY3((x), (c)) * (x) + (c)[4])
#define _POLY5(x, c)    (_POLY4((x), (c)) * (x) + (c)[5])

/* algorith and coefficients from cody and waite ( 1980 ). */

static double
asin_acos( double x, int acosflag)
{
   double y;
   int neg = 0, large = 0;
   double  sqrt();

   if ( x < 0 )
   {
      x = -x;
      neg++;
   }


   if ( x > 1 )
   {
      return ( 0.0 );
   }


   /* skip for efficiency and to prevent underflow */
   if ( x > X_EPS )
   {
      static double
	 p[] = { -0.69674573447350646411e0,
                  0.10152522233806463645e2,
                 -0.39688862997504877339e2,
                  0.57208227877891731407e2,
                 -0.27368494524164255994e2,
   	       },
	 q[] = { 1.0,
   		-0.23823859153670238830e2,
   		 0.15095270841030604719e3,
   		-0.38186303361750149284e3,
   		 0.41714430248260412556e3,
   		-0.16421096714498560795e3,
   	       };
   

   	if ( x <= 0.5 )
	{
   	   y = x * x;
	}

   	else
	{
   	   large++;
   	   y = 0.5 - 0.5 * x;
   	   x = -sqrt(y);
   	   x += x;
   	}

   	x += x * y * _POLY4(y, p)/_POLY5(y, q);
   }


   if ( acosflag )
   {
      if ( neg == 0 )
      {
         x = -x;
      }

      return ( large == 0 ? M_PI_2 + x : neg ? M_PI + x : x);
   }


   if ( large )
   {
      x += M_PI_2;
   }


   return ( neg ? -x : x );
}


double
asin( double x )
{
    if (( 1.0 < x ) || ( -1.0 > x ))
    {
	errno = EDOM;

	return 0.0;
    }
    return ( asin_acos(x, 0) );
}


double
acos( double x )
{
    if (( 1.0 < x ) || ( -1.0 > x ))
    {
	errno = EDOM;

	return 0.0;
    }
    return ( asin_acos(x, 1) );
}
