/* Copyright (C) 1995, by Motorola, Inc. */

/*
 *	atan returns the arctangent of its double-precision argument,
 *	in the range [-pi/2, pi/2].
 *	There are no error returns.
 *
 *	atan2(y, x) returns the arctangent of y/x,
 *	in the range [-pi, pi].
 *	atan2 discovers what quadrant the angle is in and calls atan.
 *	atan2 returns EDOM error and value 0 if both arguments are zero.
 *
 *	Coefficients are #5077 from Hart & Cheney (19.56D).
 */

#include <math.h>
#include "trans.h"

#define SQ2_M_1	0.41421356237309504880
#define SQ2_P_1	2.41421356237309504880
#define NEG	1
#define INV	2
#define MID	4


double
atan(x)
register double x;
{
    int type = 0;

    if ( x < 0 )
    {
	x = -x;
	type = NEG;
    }

    if ( x > SQ2_P_1 )
    {
	x = 1/x;
	type |= INV;
    }

    else if ( x > SQ2_M_1 )
    {
	x = (x - 1)/(x + 1);
	type |= MID;
    }


    if ( x < -X_EPS || x > X_EPS )
    {
	/* skip for efficiency and to prevent underflow */
	static double p[] = { 0.161536412982230228262e2,
                           0.26842548195503973794141e3,
                           0.11530293515404850115428136e4,
                           0.178040631643319697105464587e4,
                           0.89678597403663861959987488e3,
                           },
	q[] = { 1.0,
		    0.5895697050844462222791e2,
		    0.536265374031215315104235e3,
		    0.16667838148816337184521798e4,
		    0.207933497444540981287275926e4,
		    0.89678597403663861962481162e3,
		};

	double xsq = x * x;

	x *= _POLY4(xsq, p)/_POLY5(xsq, q);
    }


    if ( type & INV )
    {
	x = M_PI_2 - x;
    }

    else if ( type & MID )
    {
	x += M_PI_4;
    }

    return ( type & NEG ? -x : x );
}
