/* Copyright (C) 1990, 1995, Motorola, Inc. */

#include <stdarg.h>
#include <stdio.h>
#include <ctype.h>

/* fabulous MACRO definitions */

#define SMALL_BUFSIZ   32
#define CVT_BUFSIZ     80

#if defined( __DSP561C__ ) || defined( __DSP566C__ )

#define MSBLOW         0x8000
#define MSBHIGH        0x80000000
#define WORDSIZE       16
#define WORDON         0xffff
#define UPPERWORDON    0xffff8000

#elif defined( __DSP56K__ ) || defined( __DSP563C__ )

#define MSBLOW         0x800000
#define MSBHIGH        0x800000000000
#define WORDSIZE       24
#define WORDON         0xffffff
#define UPPERWORDON    0xffffff800000

#elif defined( __DSP96K__ )

#define MSBLOW         0x80000000
#define MSBHIGH        0x80000000
#define WORDSIZE       32
#define WORDON         0xffffffff
#define UPPERWORDON    0xffffffff

#endif

/* global variable declaration */
enum 
{
    NONE = 0, MINUS = 1, PLUS = 2, SPACE = 4, SHARP = 8, ZERO = 16
} flags;

static char* cvtptr;   /* pointer to the base of the cvt buffer */


/* forward function declarations */
static char* fill_decimal_word( int, int, char* );
static char* fill_decimal_2word( long, int, char* );
static char* fill_oh_word( unsigned int, int, int, char* );
static char* fill_o_long_word( unsigned long, int, char* );
static char* e_conv( double, int, int, int, char* );
static char* f_conv( double, int, int, int, char* );

int
__doprnt( FILE* f, const char* format, va_list ap )
{
    int chars_printed_so_far = 0;
    char *sbptr, small_buf[ SMALL_BUFSIZ ];
    char *cbptr, conversion_buffer[ 509 ];  /* ANSI defines 509 */

    enum sls
    {
	NEITHER = 0, SHORT = 1, LONG = 2
    } short_or_long_specifier = NEITHER; 

    /* always reset flags */
    flags = NONE;

    /* "format string" can be of the form:

        %[flags][width][.precision][size]conversion character

	where:
	  mandatory -
	   '%' == start character
	   conversion characters == 'd', 'i', 'o', 'u', 'x', 'X', 'f', 'e', 'E',
	                            'g', 'G', 'c', 's', 'p', 'n', '%'

	  optional -
	   flags == '-', '+', ' ', '#', '0'
	   width == decimal integer or '*'
	   precision == '.' followed by a decimal integer or '*'
	   size == 'h', 'l', 'L'
    */

    while ( '\0' != *format )
    {
	/* grab all characters up to the beginning of a conversion */
	if ( '%' != *format )
	{
	    char* base = (char*) format;
	    
	    while ( ('%' != *format) && ('\0' != *format) )
	    {
		format ++ ;
		chars_printed_so_far ++ ;
	    }

	    /* negative file numbers are fake, used by sprintf() */
	    if ( NON_FILE != f->_file )
	    {
		int nmemb = format - base;

		if ( nmemb != fwrite( base, sizeof( char ), nmemb, f ) )
		{
		    return( -1 );
		}
	    }
	    else
	    {
		int size;

		size = format - base;
		memcpy( f->_ptr, base, size );
		f->_ptr += size;
	    }
	}

	/* begin conversion */
	else
	{
	    int no_more_flags = 0;
	    int field_width = 0, width_used = 0;
	    int precision = 0, precision_was_specified = 1;
	    int conversion = 1;
	    int  radix = 0;
	    int  small_bufsiz;
	    int  padding;
	    char pad_char;
	    char ee = 'E';
	    double dval;

            /* always reset flags */
            flags = NONE;
            short_or_long_specifier = NEITHER;

	    /* first, scan for FLAGS. */
	    do
	    {
		switch ( *++format )
		{
		case '-':
		    flags |= MINUS;
		    break;
		    
		case '+':
		    flags |= PLUS;
		    break;
		    
		case ' ':
		    flags |= SPACE;
		    break;
		    
		case '#':
		    flags |= SHARP;
		    break;
		    
		case '0':
		    flags |= ZERO;
		    break;
		    
		default:
		    no_more_flags = 1;
		    continue;
		}
	    }
	    while ( ! no_more_flags );

	    /* "space" ignored when combined with "+" */
	    if ( (PLUS & flags) && (SPACE & flags) )
	    {
		flags &= ( ~SPACE );
	    }
	    
	    /* "0" ignored when combined with "-" */
	    if ( (MINUS & flags) && (ZERO & flags) )
	    {
		flags &= ( ~ZERO );
	    }
	    

	    /* second, scan for FIELD WIDTH. */
	    if ( '*' == *format )
	    {
		field_width = va_arg ( ap, int );
		format ++ ;

		/* */
		if ( field_width < 0 )
		{
		    field_width = -field_width;
		    flags |= MINUS;		    
		}
	    }
	    else
	    {
		while ( isdigit ( *format ))
		{
		    field_width *= 10;
		    field_width += (int) ( *format - '0' );
		    format ++ ;
		}
	    }

	    /* third, scan for PRECISION. */
	    if ( '.' == *format )
	    {
		/* when precision specified, '0' flag is ignored */
		flags &= ~ZERO;
		
		/* bump past '.' */
		format ++ ;
		
		if ( '*' == *format )
		{
		    precision = va_arg ( ap, int );
		    format ++ ;
		}
		else
		{
		    while ( isdigit ( *format ))
		    {
			precision *= 10;
			precision += (int) ( *format - '0' );
			format ++ ;
		    }
		}

		if ( precision < 0 )
		{
		    precision_was_specified = 0;
		    precision = 1;
		}
	    }
	    else
	    {
		precision_was_specified = 0;
		precision = 1;
	    }

	    /* fourth, scan for SIZE (h,l, or L) */
	    if ( ('l' == *format) || ('L' == *format) )
	    {
		short_or_long_specifier = LONG;
		format ++ ;
	    }
	    else if ( 'h' == *format )
	    {
		short_or_long_specifier = SHORT;
		format ++ ;
	    }

	    /* setup buffer pointers and "pad character" */
	    cbptr = conversion_buffer;
	    sbptr = small_buf + (SMALL_BUFSIZ - 1);

	    /* character for padding */
	    pad_char = ( flags & ZERO ) ? '0' : ' ';
	    
	    /* finally, switch on the CONVERSION CHAR. */
	    switch ( *format++ )
	    {
	    case 'u':     /* UNSIGNED DECIMAL */
		precision = - precision;
		
	    case 'd':
	    case 'i':
		/* short and int are the same size */
		if ( LONG != short_or_long_specifier )
		{
		    int value;
		    
		    value = va_arg ( ap, int );
		    sbptr = fill_decimal_word( value, precision, sbptr );

		    /* numbers width (with precision and sign) */
		    if ( ( 0 == precision ) && ( 0 == value ) )
		    {
			small_bufsiz = 0;
		    }
		    else
		    {
			small_bufsiz = SMALL_BUFSIZ - (sbptr - small_buf);
		    }
		}
		else
		{
		    long value;
		    
		    value = va_arg ( ap, long );

		    /* check for upper word contents */
		    if ( !(value & UPPERWORDON) )
		    {
			int word = (int) value;
			sbptr = fill_decimal_word( word, precision, sbptr );
		    }
		    else
		    {
			sbptr = fill_decimal_2word(value, precision, sbptr);
		    }

		    /* numbers width (with precision and sign) */
		    if ( ( 0 == precision ) && ( 0 == value ) )
		    {
			small_bufsiz = 0;
		    }
		    else
		    {
			small_bufsiz = SMALL_BUFSIZ - (sbptr - small_buf);
		    }
		}
		break;
		
		/* unsigned numbers */
		
		/* pointer conversion */
	    case 'p':
		flags |= SHARP;
		precision = 4;
		radix = (radix) ? radix : 16;

		/* octal */
	    case 'o':
		radix = (radix) ? radix : 8;
		
		/* short and int are the same size */
		if ( LONG != short_or_long_specifier )
		{
		    unsigned int value;
		    
		    value = va_arg ( ap, unsigned int );
		    sbptr = fill_oh_word( value, radix, precision, sbptr );
		}
		else
		{
		    /* doublewords can get tricky with octal. most machines
		       that really implement longs don't have a multiple of
		       three bits in each word. the easiest (slowest) way
		       is to just use long arithmetic and grind it out ... */

		    long value;
		    
		    value = va_arg ( ap, long );
		    sbptr = fill_o_long_word ( value, precision, sbptr );
		}

		/* numbers width (with precision and sign) */
		small_bufsiz = SMALL_BUFSIZ - (sbptr - small_buf);

		break;
		
		/* floating point numbers */
		/* hexadecimal conversion */
	    case 'x':
		radix = (radix) ? radix : 16;

	    case 'X':
		radix = (radix) ? radix : -16;

		/* short and int are the same size */
		if ( LONG != short_or_long_specifier )
		{
		    unsigned int value;
		    
		    value = va_arg ( ap, unsigned int );
		    sbptr = fill_oh_word( value, radix, precision, sbptr );
		}
		else
		{
		    long value;
		    unsigned int lword, uword;
		    
		    value = va_arg ( ap, long );

		    /* split the value */
		    lword = (unsigned int ) ( value & WORDON );
		    uword = (unsigned int ) ( (value >> WORDSIZE) & WORDON );

		    if ( uword )
		    {
	        	/* more comming so leave sbptr pointing to next available */
			sbptr = fill_oh_word( lword, radix, 0, sbptr ) - 1;
			/* WORSIZE / 4 characters guarenteed */
			precision -= (WORDSIZE / 4);
		    }
		    else
		    {
			uword = lword;
		    }
		    
		    sbptr = fill_oh_word( uword, radix, precision, sbptr );
		}

		/* numbers width (with precision and sign) */
		small_bufsiz = SMALL_BUFSIZ - (sbptr - small_buf);

		break;
		
		/* floating point numbers */
	    case 'f':
		dval = va_arg( ap, double );

		if ( !precision_was_specified )
		{
		    precision = 6;
		}
		
	        {
		    int   exponent, sign;
		    char* fcvt();

		    /* make sure that precision isn't to big */
		    precision = ( precision < CVT_BUFSIZ ) ?
			precision : CVT_BUFSIZ;

		    /* convert to ascii string */
		    cvtptr = fcvt( dval, precision, &exponent, &sign );

		    /* format ascii string */
		    sbptr = f_conv(dval, precision, exponent, sign, small_buf);
		}
		
		/* remember that we filled the buffer left to right */
		small_bufsiz = sbptr - small_buf;
		sbptr = small_buf;

		break;

	    case 'e':
		ee = 'e';
		
	    case 'E':
		dval = va_arg( ap, double );

		if ( !precision_was_specified )
		{
		    precision = 6;
		}

	        {
		    int e_precision, exponent, sign;
		    char* ecvt();
		    
		    e_precision = ( (precision + 1) < CVT_BUFSIZ ) ?
			precision + 1 : CVT_BUFSIZ;
    
		    /* convert to ascii string */
		    cvtptr = ecvt( dval, e_precision, &exponent, &sign );

		    if ( 0.0 != dval )
		    {
			exponent -- ;
		    }
		    

		    /* pass 'e' or 'E' in with buffer */
		    small_buf[ 0 ] = ee;
		    sbptr = e_conv(dval, precision, exponent, sign, small_buf);
		}
		
		/* remember that we filled the buffer left to right */
		small_bufsiz = sbptr - small_buf;
		sbptr = small_buf;
		
		break;

	    case 'g':
		ee = 'e';
		
	    case 'G':
		dval = va_arg( ap, double );
		
		if ( !precision_was_specified )
		{
		    precision = 6;
		}
		else if ( !precision )
		{
		    precision = 1;
		}
		
	        {
		    int exponent, sign;
		    int len, tz = 0;
		    char *fcvt(), *ecvt();
		    
		    /* convert to ascii string */
		    cvtptr = fcvt( dval, precision, &exponent, &sign );

		    /* if E format required, must re-convert */
		    if ( (exponent > precision) || (-4 > exponent) )
		    {
			small_buf[ 0 ] = ee;
			cvtptr = ecvt( dval, precision, &exponent, &sign );
			exponent -- ;

			/* one digit on left of '.' */
			precision --;

			if ( !(flags & SHARP) )
			{
			    /* eliminate trailing zeros */
			    while ( precision >= 0 )
			    {
				if ( cvtptr[ precision ] != '0' )
				    break;

				precision -- ;
			    }
			}
			
			sbptr = e_conv(dval, precision, exponent, sign, small_buf);
			}

		    /* F format */
		    else
		    {
			int max_dig;

			if ( !exponent )
			{
			    precision = 0;
			}
			else if ( exponent > 0 )
			{
			    /* */
			    max_dig = precision - 1;
			    precision -= exponent;

			    if ( !(flags & SHARP) )
			    {
				while ( precision )
				{
				    if ( cvtptr[ max_dig -- ] != '0' )
					break;
				    precision -- ;
				}
			    }
			}
			else
			{
			    /* '0' out on left side of '.' */
			    precision --;
			    
			    /* compute MAX digits past '.' */
			    max_dig = precision + exponent;

			    /* eliminate trailing zeros */
			    while ( cvtptr[ -- max_dig ] == '0' )
			    {
				precision --;
			    }
			}
			
			sbptr = f_conv(dval, precision, exponent, sign, small_buf);
		    }
		}
		
		/* remember that we filled the buffer left to right */
		small_bufsiz = sbptr - small_buf;
		sbptr = small_buf;
		
		break;
		
		/* character conversion */
	    case 'c':
		*sbptr = va_arg ( ap, unsigned char );
		small_bufsiz = 1;

		/* just in case, reset to ' ' */
		pad_char = ' ';

		break;
		
		/* string conversion */
	    case 's':
		sbptr = va_arg ( ap, char* );
		small_bufsiz = strlen( sbptr );

		/* check precision */
		if ( precision_was_specified && ( precision < small_bufsiz ) )
		{
		    int i;
		    
		    /* truncate small_buf */
		    for ( i = 0 ; i < precision ; i ++ )
		    {
			small_buf[ i ] = *(sbptr + i );
		    }

		    small_buf[i] = '\0';
		    sbptr = small_buf;
		    small_bufsiz = precision;
		}

		/* may need to truncate */
		if ( field_width && (field_width < small_bufsiz) )
		{
		    /* truncate */
		    small_bufsiz = field_width;
		}

		/* just in case, reset to ' ' */
		pad_char = ' ';

		break;
		
	    case 'n':
	        {
		    if ( LONG != short_or_long_specifier )
		    {
			*va_arg( ap, int* ) = chars_printed_so_far;
		    }
		    else
		    {
			*va_arg( ap, long* ) = (long) chars_printed_so_far;
		    }
		    
		    conversion = 0;
		}
		
		break;
		
	    case '%':
		*sbptr = '%';
		small_bufsiz = 1;

		break;
	    }

	    if ( !conversion )
	    {
		continue;
	    }
	    
	    /* calculate padding */
	    padding = field_width - small_bufsiz;

	    /* if pad_char is '0', check for prefix */
	    if ( padding && (pad_char == '0') )
	    {
		/* if a prefix exists, place it before the padding */
		if ( *sbptr == '-' || *sbptr == '+' )
		{
		    *cbptr ++ = *sbptr ++;
		    small_bufsiz --;
		}
		else if ( sbptr[1] == 'x' || sbptr[1] == 'X' )
		{
		    *cbptr ++ = *sbptr ++;
		    *cbptr ++ = *sbptr ++;
		    small_bufsiz -= 2;
		    padding += 2;
		}		    
	    }

	    /* if right justified, throw required padding out FIRST */
	    if ( !(flags & MINUS) && (0 < padding) )
	    {
		/* throw out required padding */
		do
		{
		    *cbptr++ = pad_char;
		    padding -- ;
		} while ( 0 < padding );
	    }

	    /* toss the number buffer into the conversion buffer */
	    memcpy( cbptr, sbptr, small_bufsiz );
	    cbptr += small_bufsiz;

	    /* if left justified AND padding required, throw out the padding */
	    if ( (MINUS & flags) && (0 < padding) )
	    {
		/* throw out required padding */
		do
		{
		    *cbptr++ = pad_char;
		    padding -- ;
		} while ( 0 < padding );
	    }

	    /* write the conversion to the buffer */
	    small_bufsiz = cbptr - conversion_buffer; /* re-use small_bufsiz */
	    chars_printed_so_far += small_bufsiz;

	    /* negative file numbers are fake, used by sprintf() */
	    if ( NON_FILE != f->_file )
	    {
		if ( small_bufsiz !=
		    fwrite(conversion_buffer, sizeof(char), small_bufsiz, f ) )
		{
		    return( -1 );
		}
	    }
	    else
	    {
		memcpy( f->_ptr, conversion_buffer, small_bufsiz );
		f->_ptr += small_bufsiz;
	    }
	}
    }

    return chars_printed_so_far;
}


/*
 * FILL_DECIMAL_WORD( ):
 *
 * function return value:
 *     The beginning of the number buffer.
 *
 * Special Note:
 *     We define fill_decimal_word( ) and fill_decimal_2word( ) for performance
 * reasons (double word arithmatic is very slow).
 */
static char*
fill_decimal_word( int value, int precision, char* buf )
{
    char* orig_buf = buf;
    int tval, negative = 0;
    
    /* "buf" points to the LAST location in the buffer */

    /* check special case, value == 0 and precision == 0 */
    if ( ( 0 == value ) && ( 0 == precision ) )
    {
	*buf = '\0';
	return buf;
    }
    

    /* check for unsigned value. By convention, this is when the precision
       is negative */
    if ( 0 > precision )
    {
	/* fixup precision value */
	precision = -precision;

	/* calculate first digit, if MSB of word is set */
	if ( MSBLOW & value )
	{
	    unsigned  uval = (unsigned) value;

	    value = (int) (uval / 10L);
	    *buf-- = (uval - (value * 10)) + '0';
	}
    }
    /* signed negative value */
    else if ( 0 > value )
    {
	negative = 1;
	value = -value;
    }

    while ( value > 9 )
    {
	tval = value;
	value /= 10;
	*buf-- = (tval - (value * 10)) + '0';
    }
    
    *buf-- = value + '0';

    /* check for precision "padding" */
    tval = orig_buf - buf;
    if ( precision > tval )
    {
	do
	{
	    *buf-- = '0';
	} while ( --precision > tval );
    }

    /* is a sign required? */
    if ( negative || (flags & PLUS) )
    {
	*buf-- = negative ? '-' : '+';
    }

    /* space required? */
    else if ( flags & SPACE )
    {
	*buf-- = ' ';
    }
    
    return ++buf;
}


/*
 * FILL_DECIMAL_2WORD( ):
 *     By the time this is called, we know that there is something in the
 * upper word.
 *
 * function return value:
 *     The beginning of the number buffer.
 *
 * Special Note:
 *     We define fill_decimal_word( ) and fill_decimal_2word( ) for performance
 * reasons (double word arithmatic is very slow).
 */
static char*
fill_decimal_2word( long value, int precision, char* buf )
{
    int negative = 0, tval;
    long tlval;
    char *orig_buf = buf;
    unsigned long absvalue;
    
    /* once value fits into 1 word, call fill_decimal_word() */
    /* if unsigned long, perform unsigned long divide ONCE - as it is VERY slow. */

    /* check for unsigned value. By convention, this is when the precision
       is negative */
    if ( 0 > precision )
    {
	/* fixup precision value */
	precision = -precision;
        
        absvalue = value;

	if ( MSBHIGH & value )
	{
	    unsigned  long ulval = (unsigned long) value;

	    absvalue = (long) (ulval / 10L);
	    *buf-- = (ulval - (absvalue * 10)) + '0';
	}
    }
    /* signed negative value */
    else if ( 0L > value )
    {
	negative = 1;
	absvalue = -value;
    }
    else
    {
        absvalue = value;
    }

    /* only do long divides as long as necessary */
    while ( absvalue & UPPERWORDON )
    {
	tlval = absvalue;
	absvalue /= 10;
	*buf-- = (tlval - (absvalue * 10)) + '0';
    }

    /* do the remainder with word arithmetic */
    tval = orig_buf - buf;
    if ( precision > tval )
    {
	precision -= tval;
    }
    
    buf = fill_decimal_word
	((int) ( negative ) ? - absvalue : absvalue, precision, buf );
}


/*
 * Fill_OH_WORD( ):
 *     Fill buffer with Octal or Hexadecimal number.
 *
 * function return value:
 *     The beginning of the number buffer.
 */

static char* lower_case = "0123456789abcdef";
static char* upper_case = "0123456789ABCDEF";

static char*
fill_oh_word( unsigned int value, int radix, int precision, char* buf )
{
    char *character = lower_case, *orig_buf = buf;
    char ex = 'x';
    int shiftval, padding;

    /* when precision is 0, the word passed is the lower word of a 2 word
       value. Therefore value == 0 is significant. */
    if ( !precision && !value )
    {
	int i = WORDSIZE / 4;
	while ( i-- != 0 )
	{
	    *buf-- = '0';
	}
	
	return ++buf;
    }
    else if ( !precision )
    {
	/* guarentees that significant zeros are printed */
	precision = WORDSIZE / 4;
    }

    /* by convention, a negative radix means UPPER case hex characters */
    if ( 0 > radix )
    {
	radix = -radix;
	character = upper_case;
	ex = 'X';
    }

    /* number of bits to shift off */
    shiftval = (radix == 8) ? 3 : 4;

    /* turn radix into a radix mask */
    radix -- ;

    do
    {
	*buf-- = character[ (value & radix) ];

	/* shift of character off */
	value >>= shiftval;
    } while ( value != 0 );

    /* check for precision padding */
    padding = precision - (orig_buf - buf);
    while ( 0 < padding )
    {
	*buf-- = '0';
	padding -- ;
    }
    
    /* alternate form? */
    if ( flags & SHARP )
    {
	/* remember, radix was transformed into the radix mask. */
	if ( 15 == radix )
	{
	    *buf-- = ex;
	}

	*buf-- = '0';
    }

    /* make buff point at first character */
    return ++buf;
}

static char*
fill_o_long_word( unsigned long value, int precision, char* buf )
{
    char *character = lower_case, *orig_buf = buf;
    char ex = 'x';
    int padding;

    do
    {
	*buf-- = character[ (value & 7) ];

	/* shift of character off */
	value >>= 3;
    } while ( value != 0 );

    /* check for precision padding */
    padding = precision - (orig_buf - buf);
    while ( 0 < padding )
    {
	*buf-- = '0';
	padding -- ;
    }
    
    /* alternate form? */
    if ( flags & SHARP )
    {
	/* remember, radix was transformed into the radix mask. */
	*buf-- = '0';
    }

    /* make buff point at first character */
    return ++buf;
}

static char*		
e_conv( double dval, int precision, int exponent, int sign, char* buf )
{
    int tval;
    char *eptr, exp[ 4 ];
    char ee;
    
    /* ee was passed in buf[0] */
    ee = buf[ 0 ];

    /* toss out sign */
    if ( (0.0 > dval) || (flags & PLUS) )
    {
	*buf ++ = (sign) ? '-' : '+';
    }

    /* space required? */
    else if ( flags & SPACE )
    {
	*buf++ = ' ';
    }
		    
    /* first digit */
    *buf ++ = *cvtptr ++;

    /* throw out '.' if 'posed to */
    if ( !(!precision && !(flags & SHARP)) )
    {
	*buf ++ = '.';
    }

    /* throw out the remainder of the number, NOT greater than precision */
    while ( (*cvtptr != '\0') && precision )
    {
	*buf ++ = *cvtptr ++;
	precision --;
    }

    /* trailing zeros? */
    while ( precision )
    {
	*buf ++ = '0';
	precision -- ;
    }
    
    /* output 'e' */
    *buf ++ = ee;

    /* exponent sign */
    if ( 0 <= exponent )
    {
	*buf ++ = '+';
    }
    else
    {
	*buf ++ = '-';
	exponent = - exponent;
    }

    /* set up exponent pointer */
    exp[ 3 ] = '\0';
    eptr = exp + 3;

    while ( exponent > 9 )
    {
	tval = exponent;
	exponent /= 10;
	*(--eptr) = (tval - (exponent * 10)) + '0';
    }
    *--eptr = exponent + '0';

    /* make sure that at least 2 decimal digits are printed */
    if ( eptr == (exp + 2) )
    {
	*--eptr = '0';
    }

    /* add the exponent to the small buffer */
    while ( *eptr != '\0' )
    {
	*buf ++ = *eptr ++ ;
    }

    return buf;
}


static char*		
f_conv( double dval, int precision, int exponent, int sign, char* buf )
{
    int tval;

    /* toss out sign */
    if ( (0.0 > dval) || (flags & PLUS) )
    {
	*buf ++ = (sign) ? '-' : '+';
    }

    /* space required? */
    else if ( flags & SPACE )
    {
	*buf++ = ' ';
    }

    /* everything before the '.' */
    if ( 0 < exponent )
    {
	do
	{
	    *buf ++ = *cvtptr ++;
	} while ( -- exponent );
    }
    else
    {
	*buf ++ = '0';
    }

    /* throw out '.' if 'posed to */
    if ( !(!precision && !(flags & SHARP)) )
    {
	*buf ++ = '.';
    }
    else
    {
	return buf;
    }

    /* throw out the remainder of the number */
    precision += exponent;

    /* leading zeros on the right side of '.' */
    while ( 0 > exponent++ )
    {
	*buf ++ = '0';
    }

    while (( precision -- ) > 0 )
    {
	*buf ++ = *cvtptr ++;
    }
    
    return buf;
}
