/* Copyright (C) 1995, Motorola, Inc. */

/*
 *	exp returns the exponential function of its double-precision argument.
 *	Returns ERANGE error and value 0 if argument too small,
 *	   value DBL_MAX if argument too large.
 *	Algorithm and coefficients from Cody and Waite (1980).
 *	Calls ldexp.
 */

#include <float.h>
#include <math.h>
#include "trans.h"
#include <errno.h>

#define _REDUCE(TYPE, X, XN, C1, C2)    { \
	double x1 = (double)(TYPE)X, x2 = X - x1; \
	X = x1 - (XN) * (C1); X += x2; X -= (XN) * (C2); }

double
exp(x)
register double x;
{
    static double p[] = 
    {
	0.31555192765684646356e-4,
	0.75753180159422776666e-2,
	0.25000000000000000000e0,
    }, q[] =
    {
	0.75104028399870046114e-6,
	0.63121894374398503557e-3,
	0.56817302698551221787e-1,
	0.50000000000000000000e0,
    };
    register double y, x_orig;
    register int n;

    x_orig = x;

    if (x < 0)
    {
	x = -x;
    }
    
    /* use first term of Taylor series expansion */
    if (x < X_EPS)
    {
	return (1.0 + x_orig );
    }

    if ( x_orig <= LN_MINDOUBLE )
    {
	if ( x_orig == LN_MINDOUBLE ) /* protect against roundoff */
	    return ( DBL_MIN ); /* causing ldexp to underflow */

	errno = ERANGE;
	return ( 0.0 );
    }

    if ( x_orig >= LN_MAXDOUBLE )
    {
	if ( x_orig == LN_MAXDOUBLE ) /* protect against roundoff */
	    return ( DBL_MAX ); /* causing ldexp to overflow */

	errno = ERANGE;
	return ( HUGE_VAL );
    }

    n = (int)(x * M_LOG2E + 0.5);
    y = (double)n;

    _REDUCE(int, x, y, 0.693359375, -2.1219444005469058277e-4);

    if ( x_orig < 0 )
    {
	x = -x;
	n = -n;
    }

    y = x * x;
    x *= _POLY2(y, p);
    return (ldexp(0.5 + x/(_POLY3(y, q) - x), n + 1 ));
}
