/* Copyright (C) 1995, Motorola, Inc. */

#include <stdio.h>

/* externally declared global variables */
extern FILE* __open_files[];

/*
 * FCLOSE():
 *    The goal here is to flush the stream pointed to by "stream" and
 *    close the associated file. Any UNREAD data is discarded and all
 *    UNWRITTEN data is delivered to the HOST to be written. The stream
 *    is disassociated from the file. NOTE: If the buffer was automatically
 *    allocated, it is deallocated.
 *
 * function return value:
 *    The fclose function returns zero if the stream is successfully closed,
 *    or EOF if any errors were detected.
 *
 * ERRNO:
 *    unused.
 */

int
fclose ( FILE *stream )
{
    int i;
    extern char *__temp_names[];
    
    if ( NULL == stream )
    {
	return EOF;
    }

    /* no need to flush non-buffered streams or READ streams */
    if ( !( stream->_flag & _IONBF ) || (stream->_flag & _IOREAD) )
    {
	if ( EOF == fflush( stream ) )
	{
	    return EOF;
	}
    }

    /* close the stream */
    if ( 0 != __close( stream->_file ) )
    {
	return EOF;
    }

    if ( NULL != __temp_names[stream->_file] )
    {
	char * name = __temp_names[stream->_file];
	
	__unlink ( name );
	free ( name );
	__temp_names[stream->_file] = NULL;
    }
	
    /* deallocate allocated buffers */
    if ( stream->_flag & _IOABUF )
    {
	free( stream->_buffer - 1 ); /* don't forget the ungetc char! */
	stream->_buffer = NULL;      /* no buffer allocated */
    }

    /* search for the stream to reset */
    for ( i = 0 ; i < FOPEN_MAX + 3 ; i ++ )
    {
	if ( __open_files[ i ] == stream )
	{
	    __open_files[ i ] = NULL;
	}
    }

    /* mark stream as closed. */
    stream->_file = -1;
    
    return 0;
}
