/* Copyright (C) 1995, Motorola, Inc. */

#include <stdio.h>

/*
 * FGETC():
 *    The goal here is to
 *
 *    Return value:
 *
 *    ERRNO:
 *        unused.
 */

int
fgetc( FILE *stream )
{
    int c;

    /* check stream for "readness" */
    if ( !( stream->_flag & _IOREAD ) && !( stream->_flag & _IORW ) )
    {
        /* error, BAD stream */
	stream->_flag |= _IOERR;
	return( EOF );
    }
    
    /* check to see if the stream's buffer requires a fillin'. A buffer is on 
       empty when one of the following is true.

       1. READ only stream (not update) and _cnt <= 0.
       2. UPDATE stream:
             o   READ/WRITE but not identified as a READ

       NOTE: DO NOT "read" from sscanf/sprintf buffer!
    */
    if ( ( ( ( stream->_flag & _IOREAD ) && ( stream->_cnt <= 0 ) ) ||
	   ( ( stream->_flag & _IORW ) && ! ( stream->_flag & _IOREAD ) ) ) &&
	   ( stream->_file != NON_FILE ) )
    {
	stream->_cnt = __read(stream->_file, stream->_buffer, stream->_bufsiz);

	if ( -1 == stream->_cnt )
	{
	    /* __read( ) error, NO elements read yet. */
	    stream->_flag |= _IOERR;
	    return( EOF );
	}	    

	stream->_ptr = stream->_buffer;

	/* mark stream as having input (read) data in buffer */
	stream->_flag |= _IOREAD;

	/* just in case it is an update stream, faster than checking */
	stream->_flag &= ~_IOWRITE;
    }

    /* end of file reached */
    if ( 0 == stream->_cnt )
    {
	stream->_flag |= _IOEOF;
	return( EOF );
    }

    /* */
    c = *stream->_ptr ++;
    stream->_cnt -- ;
    
    return c;
}
