/* Copyright (C) 1995, Motorola, Inc. */

#include <stdio.h>

/*
 * FGETS():
 *    The goal here is to
 *
 *    function return value:
 *
 *    ERRNO:
 *        unused.
 */

char*
fgets( char* s, int n, FILE* stream )
{
    char *retval = s;

    /* check for "readness" */
    if ( !( stream->_flag & _IOREAD ) && !( stream->_flag & _IORW ) )
    {
	/* error, BAD stream */
	stream->_flag |= _IOERR;
	return NULL;
    }
    
    /* check to see if the stream's buffer requires a fillin'. A buffer is on 
       empty when one of the following is true.

       1. READ only stream (not update) and _cnt <= 0.
       2. UPDATE stream:
             o   READ/WRITE but not identified as a READ

       NOTE: DO NOT "read" from sscanf/sprintf buffer!
    */
    if ( ( ( ( stream->_flag & _IOREAD ) && ( stream->_cnt <= 0 ) ) ||
           ( ( stream->_flag & _IORW ) && ! ( stream->_flag & _IOREAD ) ) ) &&
           ( stream->_file != NON_FILE ) )
    {
	stream->_cnt = __read(stream->_file, stream->_buffer, stream->_bufsiz);
	
	if ( -1 == stream->_cnt )
	{
	    /* __read( ) error, NO elements read yet. */
            stream->_flag |= _IOERR;
            return NULL;
	}	    

	stream->_ptr = stream->_buffer;
	
	/* mark stream as having input (read) data in buffer */
	stream->_flag |= _IOREAD;

	/* just in case it is an update stream, faster than checking */
        stream->_flag &= ~_IOWRITE;
    }
    
    /* it is possible that several __read() calls may need to fill large
       buffers. It is possible that "n" elements will NOT be read. */

    /* ANSI states: "reads at most one less that the number of characters
       specified by "n"" */
    n -- ;

    while ( 0 < n )
    {
	/* transfer characters from stream buffer to string */
	while ( (stream->_cnt != 0) && n )
	{
	    /* ANSI states: "No additional characters are read after a new-line
	       character (which is retained)" */
	    if ( (*s = *stream->_ptr++) == '\n' )
	    {
		n = 0;
	    }
	    else
	    {
		n -- ;
	    }

	    s++;
	    stream->_cnt --;
	}

	/* if more characters are requires, and stream buffer empty, refill
	   stream buffer. */
	if ( stream->_flag & _IOEOF )
	{
	    break;
	}

	if ( n )
	{
	    stream->_cnt = __read(stream->_file, stream->_buffer, stream->_bufsiz);
	
	    if ( -1 == stream->_cnt )
	    {
		/* __read( ) error, NO elements read yet. */
		stream->_flag |= _IOERR;
		return NULL;
	    }	    

	    /* end of file reached */
	    else if ( !stream->_cnt )
	    {
		stream->_flag |= _IOEOF;
		return NULL;
	    }

	    stream->_ptr = stream->_buffer;
	}
    }

    /* don't mess with the buffer when EOF detected */
    if ( !( stream->_flag & _IOEOF ) )
    {
	/* terminate string */
	*s = '\0';
    }
    
    return retval;
}
