/* Copyright (C) 1995, Motorola, Inc. */

#include <stdio.h>

/* externally declared global variables */
extern FILE* __open_files[];


/*
 * FFLUSH():
 *    According to the ANSI definition, this function causes any unwritten
 *    data for "stream" to be delivered to the HOST environment to be
 *    written to "the" file. "stream" must point to an output stream -OR-
 *    an update stream (in which the most recent operation was NOT an input).
 *
 *    When fflush() is called with stream == NULL, the function performs
 *    this flushing action on ALL streams for which the above behavior is
 *    defined.
 *
 * function return value:
 *    The fflush function returns zero if the stream is successfully flushed,
 *    or EOF if any write errors were detected.
 *
 * ERRNO:
 *    unused.
 */

int
fflush ( FILE* stream )
{
    /* if stream is NULL, flush ALL appropriate buffers */
    if ( NULL == stream )
    {
	int i = 0;
	
	while ( i < ( FOPEN_MAX + 3 ) )
	{
	    if ( __open_files[ i ] != NULL )
	    {
		fflush( __open_files[ i ] );
	    }
	    
	    i ++ ;
	}

        /* successful return */
        return 0; 
    }
    
    /* DO not flush a non buffered stream and streams without buffers */
    if ( ( stream->_flag & _IONBF ) || ( NULL == stream->_buffer ) )
    {
	return 0;
    }
    
    /* is it a write stream -OR- update stream last used as a write stream */
    if ( stream->_flag & _IOWRITE  )
    {
	int count = stream->_ptr - stream->_buffer;
	
	/* write out buffer -- if not empty */
	if ( 0 < count )
	{
	    if ( __write( stream->_file, stream->_buffer, count ) != count )
	    {
		/* write error has occured */
		stream->_flag |= _IOERR;
		return EOF;
	    }
	    
	    /* clear the count */
	    stream->_cnt = stream->_bufsiz;
	    stream->_ptr = stream->_buffer;
	}
    }

    /* ANSI says that fflush( ) on a "read" buffer is undefined, so do
       just reset the read buffer. */
    else
    {
	stream->_cnt = 0;
	stream->_ptr = stream->_buffer;
    }
    
    /* successful return */
    return 0;
}
