/* Copyright (C) 1995, Motorola, Inc. */

#include <stdio.h>

/*
 * FPUTC():
 *    The goal here is to write 'c' to the output stream.
 *
 *    Return value:
 *        returns the character written. If a write error occurs, the
 *    streams _flag is set with _IOERR and EOF is returned.
 *
 *    ERRNO:
 *        unused.
 */

int
fputc( int c, FILE *stream )
{
    extern int __already_registered_p;
    void __stdio_cleanup ( );
    
    /* ensure that all buffers will eventually be flushed. */
    if ( ! __already_registered_p )
    {
	atexit ( __stdio_cleanup );
	__already_registered_p = 1;
    }

    /* check for all of the basic error cases */
    if ( !( stream->_flag & _IOWRITE ) && !( stream->_flag & _IORW ) )
    {
	stream->_flag |= _IOERR;
	return  EOF;
    }
    
    /* check for EOF error */
    if ( stream->_flag & _IOEOF )
    {
	/* input cool on update buffers */
	if ( ! ( stream->_flag & _IORW ) )
	{
	    stream->_flag |= _IOERR;
	    return( EOF );
	}
	else
	{
	    stream->_flag &= ~_IOEOF;
	}
    }
    
    /* */
    if ( ( stream->_flag & _IORW ) && !( stream->_flag & _IOWRITE ) )
    {
	/* fix up for "update" buffers. faster to just do it then to check */
	stream->_flag |= _IOWRITE;
	stream->_flag &= ~_IOREAD;

	/* assume that the ANSI required file positioning took place */
	stream->_cnt = 	stream->_bufsiz;
    }

    /* if stream is unbuffered, just blow it out. */
    if ( stream->_flag & _IONBF )
    {
	char klien_buf[2];
	
	klien_buf[0] = (char) c;
	
	if ( __write( stream->_file, klien_buf, 1 ) != 1 )
	{
	    /* write error has occured */
	    stream->_flag |= _IOERR;
	    return( EOF );
	}
	
	else
	{
	    return c;
	}
    }
    
    /* check to see if we must flush the buffer */
    if ( 0 >= stream->_cnt )
    {
	if ( EOF == fflush( stream ) )
	{
	    stream->_flag |= _IOERR;
	    return( EOF );
	}
    }

    /* add 'c' to buffer */
    stream->_cnt -- ;
    *stream->_ptr ++ = c;

    /* flush if line buffered (if c == '\n' ) */
    if ( ( stream->_flag & _IOLBF ) && ( c == '\n' ) )
    {
	if ( EOF == fflush( stream ) )
	{
	    stream->_flag |= _IOERR;
	    return EOF;
	}
    }

    return c;
}
