/* Copyright (C) 1995, Motorola, Inc. */

#include <stdio.h>

/*
 * FREAD():
 *    The goal here is to
 *
 *    function return value:
 *
 *    ERRNO:
 *        unused.
 */

size_t
fread( void* ptr, size_t size, size_t nmemb, FILE* stream )
{
    size_t l_nmemb, retval;
    
    /* ANSI defined checks + check for "readness" */
    if ( 0 == size || 0 == nmemb ||
	( !( stream->_flag & _IOREAD ) && !( stream->_flag & _IORW ) ) )
    {
	/* error, NO elements read yet. */
	return 0;
    }
    
    /* check to see if the stream's buffer requires a fillin'. A buffer is on 
       empty when one of the following is true.

       1. READ only stream (not update) and _cnt <= 0.
       2. UPDATE stream:
             o   READ/WRITE but not identified as a READ

       NOTE: DO NOT "read" from sscanf/sprintf buffer!
    */
    if ( ( ( ( stream->_flag & _IOREAD ) && ( stream->_cnt <= 0 ) ) ||
           ( ( stream->_flag & _IORW ) && ! ( stream->_flag & _IOREAD ) ) ) &&
           ( stream->_file != NON_FILE ) )
    {
	stream->_cnt = __read(stream->_file, stream->_buffer, stream->_bufsiz);
	
	if ( -1 == stream->_cnt )
	{
	    /* __read( ) error, NO elements read yet. */
	    stream->_flag |= _IOERR;
	    return 0;
	}	    

	stream->_ptr = stream->_buffer;
	
	/* mark stream as having input (read) data in buffer */
	stream->_flag |= _IOREAD;

        /* just in case it is an update stream, faster than checking */
        stream->_flag &= ~_IOWRITE;
    }
    
    /* how many elements to read */
    l_nmemb = nmemb * size;
    
    /* it is possible that several calls to __read may need to be called
       to fill large buffers. It is possible that "l_nmemb" elements will
       NOT be read. */
    while ( 1 )
    {
	/* if requested number of elements are currently in the buffer */
	if ( l_nmemb <= stream->_cnt )
	{
	    memcpy( ptr, stream->_ptr, l_nmemb );
	    stream->_cnt -= l_nmemb;
	    stream->_ptr += l_nmemb;

	    /* ALL requested elements were read */
	    retval = nmemb;
	    
	    break;
	}
	else
	{
	    /* copy remaining buffer */
	    memcpy( ptr, stream->_ptr, stream->_cnt );
	    ptr += stream->_cnt;
	    l_nmemb -= stream->_cnt;

	    /* check to see if entire file has been read */
	    if ( stream->_flag & _IOEOF )
	    {
		/* all done, the entire "fread" request was not done. */
		stream->_cnt = 0;
		stream->_ptr = stream->_buffer;

		retval = nmemb - (l_nmemb / size);

		break;
	    }

	    /* refill */
	    stream->_cnt = __read(stream->_file, stream->_buffer, stream->_bufsiz);
	    
	    if ( -1 == stream->_cnt )
	    {
		/* read error, the entire "fread" request was not done. */

		/* set retval */
		retval = nmemb - (l_nmemb / size);

		break;
	    }

	    /* end of file reached */
	    else if ( stream->_cnt != stream->_bufsiz )
	    {
		stream->_flag |= _IOEOF;
	    }

	    stream->_ptr = stream->_buffer;
	}
    }
	
    return retval;
}
