/* Copyright (C) 1995, Motorola, Inc. */

#include <stdio.h>

/* external function declarations */
extern long __lseek( int, long, int);

/*
 * FSEEK( ):
 *
 * function return value:
 *
 * ERRNO:
 *    unused.
 */

int
fseek( FILE* stream, long offset, int whence )
{
    /* assume success */
    stream->_flag &= ~_IOEOF;

    /* make sure that the file's position is correct. */

    /* when stream last used as a write stream, just flush the buffer. */
    if ( stream->_flag & _IOWRITE )
    {
	/* flush buffer */
	if ( EOF == fflush( stream ) )
	{
	    /* fflush error occured */
	    return( -1 );
	}

	if ( __lseek( stream->_file, offset, whence ) < 0 )
	{
	    /* __lseek( ) error has occured */
	    return( -1 );
	}	    

	stream->_cnt = 0;
	
	/* clean up "update" streams */
	if ( stream->_flag & _IORW )
	{
	    stream->_ptr = stream->_buffer;
	    stream->_flag &= ~_IOWRITE;
	}

	/* successful fseek( ) */
	return( 0 );
    }
    
    /* the fseek is relative to the current position.
       when stream is last used as a read stream, file position is
       ahead of actual location and will need to be adjusted backwards. */

    /* clean up "update" streams */
    if ( stream->_flag & _IORW )
    {
	stream->_flag &= ~_IOREAD;
    }

    if ( whence == SEEK_CUR )
    {
	long current_fpos, new_fpos;

	if ( ( current_fpos = __lseek( stream->_file, 0L, __L_INCR ) ) < 0 )
	{
	    /* __lseek( ) error */
	    return( -1 );
	}
	
	/* calculate new absolute position */
	new_fpos = (current_fpos - stream->_cnt) + offset;

	if ( __lseek( stream->_file, new_fpos, __L_SET ) < 0 )
	{
	    /* __lseek( ) error */
	    return( -1 );
	}

	/* clean up stream, as if it were to be used as a READ stream next. */
	stream->_cnt = 0;
	stream->_ptr = stream->_buffer;

	/* successful fseek( ) */
	return( 0 );
    }
    
    /* seek from beginning or end */
    else
    {
	int lseek_val;
	
	if ( SEEK_SET == whence )
	{
	    if (( lseek_val = __lseek(stream->_file, offset, __L_SET)) < 0 )
	    {
		/* __lseek( ) error */
		return( -1 );
	    }
	}
	else if (( lseek_val = __lseek(stream->_file, offset, __L_XTND)) < 0 )
	{
	    /* __lseek( ) error */
	    return( -1 );
	}
	

	/* clean up stream, as if it were to be used as a READ stream next. */
	stream->_cnt = 0;
	stream->_ptr = stream->_buffer;

	/* successful fseek( ) */
	return( 0 );
    }
}
