/* Copyright (C) 1995, Motorola, Inc. */

/*
 *	log returns the natural logarithm of its double-precision argument.
 *	log10 returns the base-10 logarithm of its double-precision argument.
 *	Returns EDOM error and value -HUGE if argument <= 0.
 *	Algorithm and coefficients from Cody and Waite (1980).
 *	Calls frexp.
 */

#include <float.h>
#include <math.h>
#include "trans.h"
#include <errno.h>


#define C1	0.693359375
#define C2	-2.121944400546905827679e-4

double
log(x)
double x;
{
	static double p[] = {
		-0.78956112887491257267e0,
		 0.16383943563021534222e2,
		-0.64124943423745581147e2,
	}, q[] = {
		 1.0,
		-0.35667977739034646171e2,
		 0.31203222091924532844e3,
		-0.76949932108494879777e3,
	};
	double y;
	int n;

	if ( x < 0 )
	{
	    errno = EDOM;
	    
	    return - HUGE_VAL;
	}
	if ( 0 == x )
	{
	    errno = ERANGE;
	    
	    return - HUGE_VAL;
	}

	y = 1.0;
	x = frexp(x, &n);

	if (x < M_SQRT1_2)
	{
		n--;
		y = 0.5;
	}

	x = (x - y)/(x + y);
	x += x;
	y = x * x;
	x += x * y * _POLY2(y, p)/_POLY3(y, q);
	y = (double)n;
	x += y * C2;

	return ( x + y * C1 );
}


double
log10(x)
double x;
{
	if ( 0 < x )
	{
	    return log(x) * M_LOG10E;
	}
	if ( 0 == x )
	{
	    errno = ERANGE;
	}
	else
	{
	    errno = EDOM;
	}
	
	
	return - HUGE_VAL;
}


