/* Copyright (C) 1995, Motorola, Inc. */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#if defined( __DSP56K__ ) || defined( __DSP561C__ ) || defined( __DSP566C__ )
#define	MAX_M_BUFF	0x7fffL
#elif defined( __DSP563C__ )
#define	MAX_M_BUFF	0x7fffffL
#else
#define	MAX_M_BUFF	0x7fffffffL
#endif

/* fabulous macro definitions */
#define	NO		1
#define	YES		0

/* external function declarations */
extern char* sbrk( int );

/* m_buff - the structure which is used as the header for each heap buffer */

struct m_buff
{
  unsigned int used:1,  /* buffer in use - used YES and NO as values only */
#if defined( __DSP56K__ ) || defined( __DSP561C__ ) || defined( __DSP566C__ )
               size:15; /* the size of the buffer not including the header */
#elif defined( __DSP563C__ )
               size:23; /* the size of the buffer not including the header */
#else
               size:31; /* the size of the buffer not including the header */
#endif

  struct m_buff* lnk;   /* link to next contiguous buffer. this member could
			   be eliminated at the expense of speed and 
			   complexity. */
};

static struct m_buff* head = NULL;	/* head of the buffer list */

void*
malloc( size_t nbytes )
{
  struct m_buff* p = head, *best_fit = NULL;
  struct m_buff* new_buff;
  size_t waste = MAX_M_BUFF;

  if ( 0 == nbytes )
  {
      return NULL;
  }
  
  /* First: attempt to allocate the buffer from the existing pool with a
     minimum of wasted space. */

  while ( NULL != p )
  {
    if ( NO == p->used && nbytes <= p->size && waste > ( p->size - nbytes ) )
      waste = p->size - nbytes, best_fit = p;

    p = p->lnk;  
  }
  
  if ( NULL != best_fit )
  {
    best_fit->used = YES;
    return ((char*) best_fit ) + sizeof( struct m_buff );
  }

  /* Second: couldn't find a buffer that would fit. attempt to merge two
     existing buffers to make do. */

  p = head;
  while ( NULL != p )
  {
    if ( NO == p->used )
    {
      struct m_buff* look_ahd = p->lnk;
      size_t tot_room = p->size;

      while ( NULL != look_ahd && NO == look_ahd->used && tot_room < nbytes )
      {
        tot_room += ( look_ahd->size + sizeof( struct m_buff ));
        look_ahd = look_ahd->lnk;
      }

      if ( tot_room >= nbytes )
      {
        p->lnk = look_ahd;
        p->used = YES;
        p->size = tot_room;
        return ((char*) p ) + sizeof( struct m_buff );
      }
    }
    p = p->lnk;
  }

  /* Third: couldn't coalesce any set of consecutive buffers to form a buffer
     of the required size. we need to allocate one (via sbrk). */

  if ( (struct m_buff*) -1 == 
       ( new_buff = (struct m_buff*) sbrk( nbytes + sizeof( struct m_buff ))))
  {
    errno = ENOMEM;
    return NULL;
  }

  new_buff->lnk = head;
  new_buff->used = YES;
  new_buff->size = nbytes;
  head = new_buff;
  return ((char*) new_buff ) + sizeof( struct m_buff );
}

void*
realloc( void* oldp, size_t nbytes )
{
  if ( 0 == nbytes )
  {
      return NULL;
  }
  
  if ( NULL == oldp )
    return malloc( nbytes );

  {
    struct m_buff* old_buff = 
                            (struct m_buff*)(oldp - sizeof( struct m_buff ));

    char* newp;

    /* First: the actual buffer may be bigger than the user originally 
       requested - check to see if the new request can fit in the same 
       buffer. */

    if ( old_buff->size >= nbytes )
      return oldp;

    /* Second: get the new size buffer with malloc(), copy the old buffer 
       onto it, and free the old buffer. */

    if ( NULL == ( newp = malloc( nbytes )))
      return NULL;

    (void) memcpy( newp, oldp, old_buff->size );
    
    /* Free the old buffer */

    old_buff->used = NO;
    return newp;
  }
}

void
free( void* oldp )
{
    if ( NULL != oldp )
    {
	((struct m_buff*) (oldp - sizeof(struct m_buff)))->used = NO;
    }
}

#if 0
void
heap_dump ( char *message )
{
    struct m_buff *look = head;
    
    fprintf ( stderr, "%s\n", message );
    
    while ( NULL != look )
    {
	fprintf ( stderr, "[ %s, %d, %d ]\n", 
		 (( look->used == NO ) ? "NO" : "YES"), look->size, 
		 look->lnk );

	look = look->lnk;
    }
}
#endif
