/* Copyright (C) 1995, by Motorola, Inc. */

#include <math.h>
#include "trans.h"

static double
sin_cos( double x, int cosflag)
{
   double y;
   int neg = 0;


   if ( x < 0 )
   {
      x = -x;
      neg++;
   }


   /* bypass range reduction if |x| < (pi/2) and taking sin */	
   if ( cosflag || x >= M_PI_2 )
   {
      y = x;

      if ( cosflag )
      {
         neg = 0;
         y += M_PI_2;
      }

      if ( y > X_TLOSS )
      {
         return ( 0.0 );
      }

      y = y * M_1_PI + 0.5;

      /* reduce using integer arithmetic if possible */
      if ( x <= MAXLONG )
      {
         long n = (long) y;

         y = (double) n;

         if ( cosflag ){
            y -= 0.5;
	    }

	 /*
         _REDUCE(long, x, y, 3.1416015625, -8.908910206761537356617e-6);
	 */
	 {
	    double x1 = ( double ) ( long ) x, x2 = x - x1;
	    x = x1 - y * 3.1416015625;
	    x += x2;
	    x -= y * -8.908910206761537356617e-6;
	 }

         neg ^= (int)n % 2;
      }

      else
      {
         double dn;

         x = ( modf(y, &dn) - 0.5 ) * M_PI;

         if ( modf(0.5 * dn, &dn) )
	 {
            neg ^= 1;
	 }
      }


      if ( x < 0 )
      {
         x = -x;
         neg ^= 1;
      }
   }


   /* skip for efficiency and to prevent underflow */
   if ( x > X_EPS )
   {
      static double p[] = { 0.27204790957888846175e-14,
                          -0.76429178068910467734e-12,
                           0.16058936490371589114e-9,
                          -0.25052106798274584544e-7,
                           0.27557319210152756119e-5,
                          -0.19841269841201840457e-3,
                           0.83333333333331650314e-2,
                          -0.16666666666666665052e0,
                         };
      y = x * x;

      x += x * y * _POLY7(y, p);

      /* inhibit roundoff out of range */
      if ( x > 1.0 )
      {
         x = 1.0;
      }
   }

   return ( neg ? -x : x );
}


double
sin( double x )
{
   return ( sin_cos(x, 0) );
}



double
cos( double x )
{
   return ( x ? sin_cos(x, 1) : 1.0 );
}
