/* Copyright (C) 1995, by Motorola, Inc. */

/*
 *	sqrt returns the square root of its double-precision argument,
 *	using Newton's method.
 *	Returns EDOM error and value 0 if argument negative.
 *	Calls frexp and ldexp.
 */

#include <errno.h>
#include <math.h>

#define ITERATIONS	4

double
sqrt( double x)
{
	double y;
	int iexp; /* can't be in register because of frexp() below */
	int i = ITERATIONS;

	if ( x <= 0)
	{
		if ( !x )
		{
			return (x); /* sqrt(0) == 0 */
		}
		errno = EDOM;
		
		return ( 0.0 );
	}

	y = frexp(x, &iexp); /* 0.5 <= y < 1 */

	/* iexp is odd */
	if ( iexp % 2 )
	{
		--iexp;
		y += y; /* 1 <= y < 2 */
	}

	y = ldexp(y + 1.0, iexp/2 - 1); /* first guess for sqrt */

	do {
		y = 0.5 * (y + x/y);
	} while (--i > 0);

	return (y);
}
