/* Copyright (C) 1995, by Motorola, Inc. */

#include <limits.h>
#include <errno.h>
#include <ctype.h>

#define DIGIT(x)	(isdigit(x) ? (x) - '0' : \
			islower(x) ? (x) + 10 - 'a' : (x) + 10 - 'A')

#define MAX_BASE	('z' - 'a' + 11)

#undef isalnum
#undef isalpha
#undef iscntrl
#undef isdigit
#undef isgraph
#undef islower
#undef isprint
#undef ispunct
#undef isspace
#undef isupper
#undef isxdigit
#undef tolower
#undef toupper

#undef strtoul

unsigned long
strtoul( const char* nptr, char** endptr, int base )
{
   unsigned long val, stride;
   int  c;
   int  xx;

   if ( (char**) 0 != endptr )
   {
      /* in case no number is formed */
      *endptr = (char*) nptr;
   }


   /* base is invalid -- should be a fatal error */
   if ( base < 0 || base > MAX_BASE )
   {
      return ( 0L );
   }


   if ( ! isalnum( c = *nptr ) )
   {
      while ( isspace( c ) )
      {
         c = *++nptr;
      }
      if ( '+' == c )
      {
	  c = *++nptr;
      }
   }


   if ( base == 0 )
   {
      if (c != '0')
      {
         base = 10;
      }

      else if (nptr[1] == 'x' || nptr[1] == 'X')
      {
         base = 16;
      }

      else
      {
         base = 8;
      }
   }


/*
   for any base > 10, the digits incrementally following
   9 are assumed to be "abc...z" or "ABC...Z"
*/
   if ( ! isalnum( c ) || ( xx = DIGIT( c ) ) >= base )
   {
      /* no number formed */
      return ( 0L );
   }


   /* skip over leading "0x" or "0X" */
   if ( base == 16 && c == '0' && isxdigit(nptr[2]) &&
        ( nptr[1] == 'x' || nptr[1] == 'X') )
   {
      c = *(nptr += 2);
   }


   stride = ULONG_MAX / base;

   for ( val = DIGIT(c); isalnum(c = *++nptr) && ( xx = DIGIT(c) ) < base; )
   {
       if (( stride < val ) || (( ULONG_MAX - ( base * val )) < xx ))
       {
	   errno = ERANGE;
	   
	   return ULONG_MAX;
       }
       val = base * val + xx;
   }

   if ( (char **) 0 != endptr )
   {
      *endptr = (char*) nptr;
   }

   return val;
}
