/* Copyright (C) 1990, 1995, by Motorola, Inc. */

#include <math.h>
#include <stdlib.h>
#include <errno.h>
#include <float.h>

#undef strtod

double
strtod ( const char* nptr, char** endptr )
{
    double whole = 0.0, fraction = 0.0;
    int exp = 0, exp_is_negative = 0;
    int is_negative = 0, really_got_something = 0;
    const char* point = nptr;
    
    while ( isspace ( *point ))
    {
	++ point;
    }
    
    switch ( *point )
    {
    case '-':
	is_negative = 1;

    case '+':
	++ point;
    }
    
    if ( isdigit ( *point ))
    {
	really_got_something = 1;
	
	do
	{
	    whole = whole * 10.0 + ((double) ( *( point ++ ) - '0' ));
	}
	while ( isdigit ( *point ));
    }
    
    if ( *point == '.' )
    {
	double divisor = 1.0;
	
	++ point;

	if ( isdigit ( *point ))
	{
	    really_got_something = 1;
	    
	    do
	    {
		fraction = fraction * 10.0 + 
		    ((double) ( *( point ++ ) - '0' ));
		divisor = divisor * 10.0;
	    }
	    while ( isdigit ( *point ));
	    
	    fraction /= divisor;
	}
    }
    
    if (( *point == 'e' ) || ( *point == 'E' ))
    {
	switch ( *( ++ point ))
	{
	case '-':
	    exp_is_negative = 1;
	    
	case '+':
	    ++ point;
	}
	
	while ( isdigit ( *point ))
	{
	    exp = exp * 10 + ( *( point ++ ) - '0' );

	    if ( exp > DBL_MAX_10_EXP )
	    {
		errno = ERANGE;
		
		return (( exp_is_negative ) ? 0.0 : 
			(( is_negative ) ? - HUGE_VAL : HUGE_VAL ));
	    }
	}
	
	if ( exp_is_negative )
	{
	    exp = - exp;
	}
    }
    
    if ( ! really_got_something )
    {
	errno = EBADFORMAT;

	if ( NULL != endptr )
	{
	    *endptr = (char*) nptr;
	}
	return 0.0L;
    }
    
    fraction += whole;
    
    while ( 1.0L < whole )
    {
	whole /= 10.0L;	
	fraction /= 10.0L;
	++ exp;
    }
    
    if (( DBL_MIN_10_EXP > exp ) || ( DBL_MAX_10_EXP < exp ))
    {
	errno = ERANGE;

	if ( NULL != endptr )
	{
	    *endptr = (char*) point;
	}
	return ( exp_is_negative ) ? 0.0L :
	    (( is_negative ) ? - HUGE_VAL : HUGE_VAL );
    }
    
    if ( NULL != endptr )
    {
	*endptr = (char*) point;
    }
    
    return ( is_negative ) ?
	- fraction * pow ( 10.0L, exp ) : fraction * pow ( 10.0L, exp );
}
