/* Copyright (C) 1995, by Motorola, Inc. */

#include <limits.h>
#include <errno.h>
#include <ctype.h>

#undef isalnum
#undef isdigit
#undef isxdigit
#undef islower
#undef isspace
#define DIGIT(x)	(isdigit(x) ? (x) - '0' : \
			islower(x) ? (x) + 10 - 'a' : (x) + 10 - 'A')

#define MAX_BASE	('z' - 'a' + 11)

long
strtol( const char* nptr, char** endptr, int base )
{
   long val, stride;
   int  c;
   int  xx, neg = 0;

   if ( (char**) 0 != endptr )
   {
      /* in case no number is formed */
      *endptr = (char*) nptr;
   }


   /* base is invalid -- should be a fatal error */
   if ( base < 0 || base > MAX_BASE )
   {
      return ( 0L );
   }


   if ( ! isalnum( c = *nptr ) )
   {
      while ( isspace( c ) )
      {
         c = *++nptr;
      }

      switch ( c )
      {
         case '-':
            neg = 1;
	    /* fall-through */

         case '+':
            c = *++nptr;
	    break;
      }
   }


   if ( base == 0 )
   {
      if (c != '0')
      {
         base = 10;
      }
      else if (nptr[1] == 'x' || nptr[1] == 'X')
      {
         base = 16;
      }
      else
      {
         base = 8;
      }
   }


/*
   for any base > 10, the digits incrementally following
   9 are assumed to be "abc...z" or "ABC...Z"
*/
   if ( ! isalnum( c ) || ( xx = DIGIT( c ) ) >= base )
   {
      /* no number formed */
      return ( 0L );
   }


   /* skip over leading "0x" or "0X" */
   if ( base == 16 && c == '0' && isxdigit(nptr[2]) &&
        ( nptr[1] == 'x' || nptr[1] == 'X') )
   {
      c = *(nptr += 2);
   }


   stride = LONG_MIN / base;
   
   for ( val = -DIGIT(c); isalnum(c = *++nptr) && ( xx = DIGIT(c) ) < base; )
   {
       if (( stride <= val ) && ((( LONG_MIN - ( base * val )) + xx ) <= 0 ))
       {
	   val = base * val - xx;
       }
       else
       {
	   errno = ERANGE;

	   return ( neg ? LONG_MIN : LONG_MAX );
       }
   }


   if ( (char **) 0 != endptr )
   {
      *endptr = (char*) nptr;
   }

   if ( neg )
   {
       return val;
   }

   if ( LONG_MIN == val )
   {
       errno = ERANGE;
       
       return LONG_MAX;
   }
   return ( - val );
}
