/*******************************************************************************
 * C H A M E L E O N    S. D. K.                                               *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/include/chameleon/chameleon.h                    $
 *     $Date:: 14/05/02 16:53                                                  $
 * $Revision:: 4                                                               $
 *-----------------------------------------------------------------------------*
 * This file is part of the Chameleon Software Development Kit                 *
 *                                                                             *
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#ifndef __CHAMELEON_H__
#define __CHAMELEON_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "dsp.h"
#include "flash.h"
#include "info.h"
#include "panel.h"

#ifndef CHAMELEON_MODEL
  #error "Undefined Chameleon model"
#endif

#if CHAMELEON_MODEL == 01
  #include "panel01.h"
#else
  #error "Unknown Chameleon model"
#endif

#ifdef DEBUG
  #include <stdio.h>
  #include <assert.h>

  #define TRACE			printf
  #define ASSERT(f)		assert(f)
  #define DEBUG_ONLY(f)		(f)
  #define UNUSED(x)
#else
  #ifndef __trace
    static __inline void __trace(const char *format, ...) { ; };
  #endif

  #define TRACE			1 ? (void)0 : __trace
  #define ASSERT(f)		((void)0)
  #define DEBUG_ONLY(f)		((void)0)
  #define UNUSED(x)		x
#endif

#define UNUSED_ALWAYS(x)	x

#define FIX_SCALER		8388608.0f

static __inline float fix_to_float(rtems_signed32 n)
{
	if (n & 0x800000)
		return (float) ((n & 0x007FFFFF) / FIX_SCALER) - 1.0;

	return (float) n / FIX_SCALER;
}

static __inline rtems_signed32 float_to_fix(float f)
{
	if (f >= 1.0f)
		return 0x007FFFFF;
	else if (f <= -1.0f)
		return 0xFF800000;

	return (rtems_signed32) (f * FIX_SCALER);
}

static __inline rtems_signed32 float_to_fix_round(float f)
{
	f *= FIX_SCALER;
	if (f >= 0.0f)
	{
		f += 0.5f;
		if (f >= FIX_SCALER)
			f = FIX_SCALER - 1.0;
	}
	else
	{
		f -= 0.5f;
		if (f < -FIX_SCALER)
			f = -FIX_SCALER;
	}

	return (rtems_signed32) f;
}


#ifdef __cplusplus
}
#endif

#endif

