;*******************************************************************************
; C H A M E L E O N    S. D. K.                                                *
;*******************************************************************************
;  $Archive:: /Chameleon.sdk/include/dsp/dma.asm                               $
;     $Date:: 14/05/02 17:10                                                   $
; $Revision:: 1                                                                $
;------------------------------------------------------------------------------*
; This file is part of the Chameleon Software Development Kit                  *
;                                                                              *
; Copyright (C) 2001-2002 Soundart                                             *
; www.soundart-hot.com                                                         *
; support@soundart-hot.com                                                     *
;*******************************************************************************
;
; This macro helps to configure DMA for audio I/O with double buffer
;
;*******************************************************************************

ConfigDMA	macro	BUFFER_SIZE,BUFFER_DAC,BUFFER_ADC

	;**********************************************************************
	; DOR0: DMA OFFSET REGISTER 0 (DSP56300FM page 8-3)
	; Offset: -(2*BUFFER_SIZE-1)
	;**********************************************************************
	MOVEP	#>-(2*BUFFER_SIZE-1),X:<<DOR0

	;**********************************************************************
	; DCO0: DMA COUNTER CHANNEL 0 (DSP56300FM page 8-5)
	; DCOL: 0
	; DCOH: 2*BUFFER_SIZE-1
	;**********************************************************************
	MOVEP	#>(2*BUFFER_SIZE-1),X:<<DCO0

	;**********************************************************************
	; DSR0: DMA SOURCE ADDRESS REGISTER CHANNEL 0 (DSP56300FM page 8-2)
	;**********************************************************************
	MOVEP	#>BUFFER_DAC,X:<<DSR0

	;**********************************************************************
	; DDR0: DMA DESTINATION ADDRESS REGISTER CHANNEL 0 (DSP56300FM page 8-2)
	;**********************************************************************
	MOVEP	#>TX00,X:<<DDR0

	;**********************************************************************
	; DCR0: DMA CONTROL REGISTER CHANNEL 0 (DSP56300FM page 8-8)
	; DMA Source Space: X mem				(00)
	; DMA Destination Space: X mem				(00)
	; DMA Address Mode: Mode B, DOR0, src 2D, dst no update	(100000)
	; DMA Three Dimensional Mode: Off			(0)
	; DMA Request Source: ESSI0Tx				(01011)
	; DMA Continuous Mode					(0)
	; DMA Channel Priority: 3				(11)
	; DMA Transfer Mode: word transfer, no clear DE		(101)
	; DMA Interrupt Enable: Off				(0)
	; DMA Channel Enable: Off				(0)
	;**********************************************************************
	; 0 0 101 11 0 01011 0 100000 00 00 = $2E5A00
	;**********************************************************************
	MOVEP	#>$2E5A00,X:<<DCR0

	;**********************************************************************
	; DOR1: DMA OFFSET REGISTER 1 (DSP56300FM page 8-3)
	; Offset: -(2*BUFFER_SIZE-1)
	;**********************************************************************
	MOVEP	#-(2*BUFFER_SIZE-1),X:<<DOR1

	;**********************************************************************
	; DCO1: DMA COUNTER CHANNEL 1 (DSP56300FM page 8-5)
	; DCOL: 0
	; DCOH: 2*BUFFER_SIZE-1
	;**********************************************************************
	MOVEP	#>(2*BUFFER_SIZE-1),X:<<DCO1

	;**********************************************************************
	; DDR1: DMA DESTINATION ADDRESS REGISTER CHANNEL 1 (DSP56300FM page 8-2)
	;**********************************************************************
	MOVEP	#>BUFFER_ADC,X:<<DDR1

	;**********************************************************************
	; DSR1: DMA SOURCE ADDRESS REGISTER CHANNEL 1 (DSP56300FM page 8-2)
	;**********************************************************************
	MOVEP	#>RX0,X:<<DSR1

	;**********************************************************************
	; DCR1: DMA CONTROL REGISTER CHANNEL 1 (DSP56300FM page 8-8)
	; DMA Source Space: X mem				(00)
	; DMA Destination Space:X mem				(00)
	; DMA Address Mode: Mode B, DOR1, dst 2D, src no update	(001100)
	; DMA Three Dimensional Mode: Off			(0)
	; DMA Request Source: ESSI0Rx				(01010)
	; DMA Continuous Mode					(0)
	; DMA Channel Priority: 3				(11)
	; DMA Transfer Mode: word transfer, no clear DE		(101)
	; DMA Interrupt Enable: Off				(0)
	; DMA Channel Enable: Off				(0)
	;**********************************************************************
	; 0 0 101 11 0 01010 0 001100 00 00 = $2E50C0
	;**********************************************************************
	MOVEP	#>$2E50C0,X:<<DCR1

	;**********************************************************************
	; Enable DMA Channels 0 and 1
	;**********************************************************************
	BSET	#DCR_DE,X:<<DCR0
	BSET	#DCR_DE,X:<<DCR1

	endm
