/*******************************************************************************
 * C H A M E L E O N    S. D. K.                                               *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/include/rtems/error.h                            $
 *     $Date:: 18/02/02 11:55                                                  $
 * $Revision:: 6                                                               $
 *-----------------------------------------------------------------------------*
 * This file is part of the Chameleon Software Development Kit                 *
 *                                                                             *
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

/*
 *  Defines and externs for rtems error reporting
 *
 *  $Id: error.h,v 1.6 2000/06/12 14:59:52 joel Exp $
 */

#ifndef __RTEMS_ERROR_h
#define __RTEMS_ERROR_h

#include <status.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 * rtems_error() and rtems_panic() support
 */

#define RTEMS_ERROR_ERRNO  (1<<((sizeof(int) * 8) - 2))	/* hi bit; use 'errno' */
#define RTEMS_ERROR_PANIC  (RTEMS_ERROR_ERRNO / 2)	/* err fatal; no return */

const char *rtems_status_text(rtems_status_code status);
int   rtems_error(int error_code, const char *printf_format, ...);
void  rtems_panic(const char *printf_format, ...);

#ifdef DEBUG
  #define RTEMS_STATUS_TEXT(f)	rtems_status_text(f)
  #define RTEMS_ERROR		rtems_error
  #define RTEMS_PANIC		rtems_pannic
#else
  #ifndef __error1
    static __inline void __error1(int error_code, const char *format, ...) { ; };
  #endif
  #ifndef __error2
    static __inline void __error2(const char *format, ...) { ; };
  #endif

  #define RTEMS_STATUS_TEXT(f)	((void)0)
  #define RTEMS_ERROR		1 ? (void)0 : __error1
  #define RTEMS_PANIC		1 ? (void)0 : __error2
#endif

#ifdef __cplusplus
}
#endif


#endif
/* end of include file */
