/*******************************************************************************
 * C H A M E L E O N    S. D. K.                                               *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/include/rtems/event.h                            $
 *     $Date:: 18/02/02 11:55                                                  $
 * $Revision:: 4                                                               $
 *-----------------------------------------------------------------------------*
 * This file is part of the Chameleon Software Development Kit                 *
 *                                                                             *
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

/*  event.h
 *
 *  This include file contains the information pertaining to the Event
 *  Manager.  This manager provides a high performance method of communication
 *  and synchronization.
 *
 *  Directives provided are:
 *
 *     + send an event set to a task
 *     + receive event condition
 *
 */

#ifndef __RTEMS_EVENT_h
#define __RTEMS_EVENT_h

#ifdef __cplusplus
extern "C" {
#endif

#include "types.h"
#include "status.h"
#include "options.h"


/*
 *  The following defines the type used to control event sets.
 */

typedef rtems_unsigned32 rtems_event_set;

/*
 *  The following constants define the individual events which may
 *  be used to compose an event set.
 */

#define RTEMS_PENDING_EVENTS	0           /* receive pending events   */
#define RTEMS_ALL_EVENTS		0xFFFFFFFF

#define RTEMS_EVENT_0			0x00000001
#define RTEMS_EVENT_1			0x00000002
#define RTEMS_EVENT_2			0x00000004
#define RTEMS_EVENT_3			0x00000008
#define RTEMS_EVENT_4			0x00000010
#define RTEMS_EVENT_5			0x00000020
#define RTEMS_EVENT_6			0x00000040
#define RTEMS_EVENT_7			0x00000080
#define RTEMS_EVENT_8			0x00000100
#define RTEMS_EVENT_9			0x00000200
#define RTEMS_EVENT_10			0x00000400
#define RTEMS_EVENT_11			0x00000800
#define RTEMS_EVENT_12			0x00001000
#define RTEMS_EVENT_13			0x00002000
#define RTEMS_EVENT_14			0x00004000
#define RTEMS_EVENT_15			0x00008000
#define RTEMS_EVENT_16			0x00010000
#define RTEMS_EVENT_17			0x00020000
#define RTEMS_EVENT_18			0x00040000
#define RTEMS_EVENT_19			0x00080000
#define RTEMS_EVENT_20			0x00100000
#define RTEMS_EVENT_21			0x00200000
#define RTEMS_EVENT_22			0x00400000
#define RTEMS_EVENT_23			0x00800000
#define RTEMS_EVENT_24			0x01000000
#define RTEMS_EVENT_25			0x02000000
#define RTEMS_EVENT_26			0x04000000
#define RTEMS_EVENT_27			0x08000000
#define RTEMS_EVENT_28			0x10000000
#define RTEMS_EVENT_29			0x20000000
#define RTEMS_EVENT_30			0x40000000
#define RTEMS_EVENT_31			0x80000000

/*
 *  The following constant is the value of an event set which
 *  has no events pending.
 */

#define EVENT_SETS_NONE_PENDING 0

/*
 *  This constant is passed as the event_in to the
 *  rtems_event_receive directive to determine which events are pending.
 */

#define EVENT_CURRENT  0

/*
 *  rtems_event_send
 *
 *  DESCRIPTION:
 *
 *  This routine implements the rtems_event_send directive.  It sends
 *  event_in to the task specified by ID.  If the task is blocked
 *  waiting to receive events and the posting of event_in satisfies
 *  the task's event condition, then it is unblocked.
 */

rtems_status_code rtems_event_send (
  rtems_id        id,
  rtems_event_set event_in
);

/*
 *  rtems_event_receive
 *
 *  DESCRIPTION:
 *
 *  This routine implements the rtems_event_receive directive.  This
 *  directive is invoked when the calling task wishes to receive
 *  the event_in event condition.  One of the fields in the option_set
 *  parameter determines whether the receive request is satisfied if
 *  any or all of the events are pending.   If the event condition
 *  is not satisfied immediately, then the task may block with an
 *  optional timeout of TICKS clock ticks or return immediately.
 *  This determination is based on another field in the option_set
 *  parameter.  This directive returns the events received in the
 *  event_out parameter.
 */

rtems_status_code rtems_event_receive (
  rtems_event_set  event_in,
  rtems_option     option_set,
  rtems_interval   ticks,
  rtems_event_set *event_out
);


#ifdef __cplusplus
}
#endif

#endif
/* end of include file */
