/*******************************************************************************
 * C H A M E L E O N    S. D. K.                                               *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/include/rtems/extension.h                        $
 *     $Date:: 18/02/02 11:55                                                  $
 * $Revision:: 5                                                               $
 *-----------------------------------------------------------------------------*
 * This file is part of the Chameleon Software Development Kit                 *
 *                                                                             *
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

/*  extension.h
 *
 *  This include file contains all the constants, structures, and
 *  prototypes associated with the User Extension Manager.  This manager
 *  provides a mechanism for manipulating sets of user-defined extensions.
 *
 *  Directives provided are:
 *
 *     + create user extension set
 *     + get ID of user extension set
 *     + delete user extension set
 *
 */

#ifndef __RTEMS_EXTENSION_MANAGER_h
#define __RTEMS_EXTENSION_MANAGER_h

#ifdef __cplusplus
extern "C" {
#endif

#include "types.h"
#include "status.h"


/*
 *  Extension related types
 */

typedef void rtems_extension;
 
typedef rtems_boolean ( *rtems_task_create_extension )(
                 void *,
                 void *
             );
 
typedef rtems_extension ( *rtems_task_delete_extension )(
                 void *,
                 void *
             );
 
typedef rtems_extension ( *rtems_task_start_extension )(
                 void *,
                 void *
             );
 
typedef rtems_extension ( *rtems_task_restart_extension )(
                 void *,
                 void *
             );
 
typedef rtems_extension ( *rtems_task_switch_extension )(
                 void *,
                 void *
             );
 
typedef rtems_extension ( *rtems_task_post_switch_extension )(
                 void *
             );
 
typedef rtems_extension ( *rtems_task_begin_extension )(
                 void *
             );
 
typedef rtems_extension ( *rtems_task_exitted_extension )(
                 void *
             );
 
typedef rtems_extension ( *rtems_fatal_extension )(
                 rtems_unsigned32  	/* the_source  */,
                 rtems_boolean         	/* is_internal */,
                 rtems_unsigned32      	/* the_error   */
             );

 
typedef struct {
  rtems_task_create_extension       thread_create;
  rtems_task_start_extension        thread_start;
  rtems_task_restart_extension      thread_restart;
  rtems_task_delete_extension       thread_delete;
  rtems_task_switch_extension       thread_switch;
  rtems_task_begin_extension        thread_begin;
  rtems_task_exitted_extension      thread_exitted;
  rtems_fatal_extension             fatal;
} rtems_extensions_table;

/*
 *  rtems_extension_create
 *
 *  DESCRIPTION:
 *
 *  This routine implements the rtems_extension_create directive.  The
 *  extension will have the name name.   The entry points of the
 *  routines which constitute this extension set are in EXTENSION_TABLE.
 *  It returns the id of the created extension in ID.
 */

rtems_status_code rtems_extension_create(
  rtems_name              name,
  rtems_extensions_table *extension_table,
  rtems_id                *id
);

/*
 *  rtems_extension_ident
 *
 *  DESCRIPTION:
 *
 *  This routine implements the rtems_extension_ident directive.
 *  This directive returns the extension ID associated with name.
 *  If more than one extension is named name, then the extension
 *  to which the ID belongs is arbitrary.
 */

rtems_status_code rtems_extension_ident(
  rtems_name    name,
  rtems_id     *id
);

/*
 *  rtems_extension_delete
 *
 *  DESCRIPTION:
 *
 *  This routine implements the rtems_extension_delete directive.  The
 *  extension indicated by ID is deleted.
 */

rtems_status_code rtems_extension_delete(
  rtems_id   id
);


#ifdef __cplusplus
}
#endif

#endif
/* end of include file */
