/*******************************************************************************
 * C H A M E L E O N    S. D. K.                                               *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/include/rtems/intr.h                             $
 *     $Date:: 14/01/02 16:45                                                  $
 * $Revision:: 1                                                               $
 *-----------------------------------------------------------------------------*
 * This file is part of the Chameleon Software Development Kit                 *
 *                                                                             *
 * Copyright (C) 2001 soundart                                                 *
 * www.soundart-hot.com                                                        *
 * codemaster@soundart-hot.com                                                 *
 ******************************************************************************/

/*  intr.h
 *
 *  This include file contains all the constants and structures associated
 *  with the Interrupt Manager.
 *
 *  COPYRIGHT (c) 1989-1999.
 *  On-Line Applications Research Corporation (OAR).
 *
 *  The license and distribution terms for this file may be
 *  found in the file LICENSE in this distribution or at
 *  http://www.OARcorp.com/rtems/license.html.
 *
 *  $Id: intr.h,v 1.10 1999/11/17 17:50:24 joel Exp $
 */

#ifndef __RTEMS_INTERRUPT_h
#define __RTEMS_INTERRUPT_h

#ifdef __cplusplus
extern "C" {
#endif

#include "types.h"
#include "status.h"


/*
 *  Interrupt level type
 */

typedef rtems_unsigned32 rtems_interrupt_level;

/*
 *  The following type defines the control block used to manage
 *  the vectors.
 */
 

typedef rtems_unsigned32 rtems_vector_number;

/*
 *  Return type for ISR Handler
 */

typedef void rtems_isr;

/*
 *  Pointer to an ISR Handler
 */
 
typedef rtems_isr ( *rtems_isr_entry )(
                 rtems_vector_number
             );

/*
 *  rtems_interrupt_catch
 *
 *  DESCRIPTION:
 *
 *  This routine implements the rtems_interrupt_catch directive.  This
 *  directive installs new_isr_handler as the RTEMS interrupt service
 *  routine for vector.  The previous RTEMS interrupt service
 *  routine is returned in old_isr_handler.
 */

rtems_status_code rtems_interrupt_catch(
  rtems_isr_entry      new_isr_handler,
  rtems_vector_number  vector,
  rtems_isr_entry     *old_isr_handler
);

/*
 *  rtems_interrupt_disable
 *
 *  DESCRIPTION:
 *
 *  This routine disables all maskable interrupts and returns the
 *  previous level in _isr_cookie.
 */

void rtems_interrupt_disable(rtems_interrupt_level level);

/*
 *  rtems_interrupt_enable
 *
 *  DESCRIPTION:
 *
 *  This routine enables maskable interrupts to the level indicated
 *  _isr_cookie.
 */

void rtems_interrupt_enable(rtems_interrupt_level level);

/*
 *  rtems_interrupt_flash
 *
 *  DESCRIPTION:
 *
 *  This routine temporarily enables maskable interrupts to the
 *  level in _isr_cookie before redisabling them.
 */

void rtems_interrupt_flash(rtems_interrupt_level level);

/*
 *  rtems_interrupt_is_in_progress
 *
 *  DESCRIPTION:
 *
 *  This function returns TRUE if the processor is currently servicing
 *  an interrupt and FALSE otherwise.   A return value of TRUE indicates
 *  that the caller is an interrupt service routine, NOT a thread.  The
 *  directives available to an interrupt service routine are restricted.
 */
 
rtems_boolean rtems_interrupt_is_in_progress(void);

/*
 *  rtems_interrupt_cause
 *
 *  DESCRIPTION:
 *
 *  This routine generates an interrupt.
 *
 *  NOTE:  No implementation.
 */

void rtems_interrupt_cause(rtems_vector_number vector);

/*
 *  rtems_interrupt_clear
 *
 *  DESCRIPTION:
 *
 *  This routine clears the specified interrupt.
 *
 *  NOTE:  No implementation.
 */

void rtems_interrupt_clear(rtems_vector_number vector);


#ifdef __cplusplus
}
#endif

#endif
/* end of include file */
