/*******************************************************************************
 * C H A M E L E O N    S. D. K.                                               *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/include/rtems/types.h                            $
 *     $Date:: 18/02/02 11:55                                                  $
 * $Revision:: 6                                                               $
 *-----------------------------------------------------------------------------*
 * This file is part of the Chameleon Software Development Kit                 *
 *                                                                             *
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

/*  types.h
 *
 *  This include file defines the types used by the RTEMS API.
 *
 */

#ifndef __RTEMS_TYPES_h
#define __RTEMS_TYPES_h

#ifdef __cplusplus
extern "C" {
#endif

/*
 *  RTEMS basic type definitions
 */

typedef unsigned char		unsigned8;		/* unsigned 8-bit  integer */
typedef unsigned short		unsigned16;		/* unsigned 16-bit integer */
typedef unsigned int		unsigned32;		/* unsigned 32-bit integer */

typedef signed char		signed8;		/* 8-bit  signed integer */
typedef signed short		signed16;		/* 16-bit signed integer */
typedef signed int		signed32;		/* 32-bit signed integer */

typedef unsigned32		boolean;		/* Boolean value   */

typedef unsigned8		rtems_unsigned8;	/* unsigned 8-bit value  */
typedef unsigned16		rtems_unsigned16;	/* unsigned 16-bit value */
typedef unsigned32		rtems_unsigned32;	/* unsigned 32-bit value */

typedef signed8			rtems_signed8;		/* signed 8-bit value  */
typedef signed16		rtems_signed16;		/* signed 16-bit value */
typedef signed32		rtems_signed32;		/* signed 32-bit value */

/*
 * some C++ compilers (eg: HP's) don't do 'long long'
 */
#if defined(__GNUC__)
typedef unsigned long long	unsigned64;		/* unsigned 64-bit integer */
typedef signed long long	signed64;		/* signed 64-bit value */
typedef unsigned64		rtems_unsigned64;	/* unsigned 64-bit value */
typedef signed64		rtems_signed64;		/* 64 bit signed integer */
#endif

typedef float			rtems_single;		/* single precision float */
typedef double			rtems_double;		/* double precision float */

typedef boolean			rtems_boolean;

typedef rtems_unsigned32	rtems_id;
typedef rtems_unsigned32	rtems_name;

/*
 *  Time related 
 */

typedef rtems_unsigned32	rtems_interval;

typedef struct {			/* RTEID style time/date */
  rtems_unsigned32 year;                /* year, A.D. */
  rtems_unsigned32 month;               /* month, 1 -> 12 */
  rtems_unsigned32 day;                 /* day, 1 -> 31 */
  rtems_unsigned32 hour;                /* hour, 0 -> 23 */
  rtems_unsigned32 minute;              /* minute, 0 -> 59 */
  rtems_unsigned32 second;              /* second, 0 -> 59 */
  rtems_unsigned32 ticks;               /* elapsed ticks between secs */
} rtems_time_of_day;

/*
 *  Define NULL
 */

#ifndef NULL
#define NULL      0          /* NULL value */
#endif

/*
 *  Boolean constants
 */

#if !defined( TRUE ) || (TRUE != 1)
#undef TRUE
#define TRUE     (1)
#endif

#if !defined( FALSE ) || (FALSE != 0)
#undef FALSE
#define FALSE     (0)
#endif

/*
 *  Define BYTE/WORD/DWORD/BOOL types
 */

#ifndef BYTE
	#define	BYTE		unsigned char
#endif
#ifndef WORD
	#define	WORD		unsigned short
#endif
#ifndef DWORD
	#define	DWORD		unsigned long
#endif
#ifndef BOOL
	#define	BOOL		int
#endif


#ifdef __cplusplus
}
#endif

#endif
/* end of include file */
