################################################################################
# C H A M E L E O N    S. D. K.                                                #
################################################################################
#  $Archive:: /Chameleon.sdk/make/main.mak                                     $
#     $Date:: 14/05/02 17:12                                                   $
# $Revision:: 9                                                                $
#------------------------------------------------------------------------------#
# This file is part of the Chameleon Software Development Kit                  #
#                                                                              #
# Copyright (C) 2001 Soundart                                                  #
# www.soundart-hot.com                                                         #
# support@soundart-hot.com                                                     #
################################################################################
#
# Specify the file name of the final application
# APP := app.elf
#
# Specify the output path name of the files generated
# OUTDIR := examples/app1
#
# Specify extra files to be linked with the application
# EXTRAOBJS := shared/common.o
#
# Specify search paths for headers
# CCINCLUDES := /Chameleon.sdk/include
#
# Specify extra definitions for compilation
# CCDEFINES := -DHAVE_MALLOC
#
# Specify extra goals to satisfy
# EXTRAGOALS := dsp

CHAMELEON_MODEL ?= 01

CCINCLUDES += 	/Chameleon.sdk/include/chameleon		\
		/Chameleon.sdk/include/rtems			\
		/Chameleon.sdk/include/midishare		\
		/Chameleon.sdk/include/newlib

override CCDEFINES  += -DCHAMELEON_MODEL=$(CHAMELEON_MODEL)

include /Chameleon.sdk/make/root.mak

bin2header =	$(ECHO) "Generating \"$(CURDIR)/$@\"..."; \
		/Chameleon.sdk/bin/bin2header $(call getpath, $^) $(call getpath, $@) -quiet $1

ifdef IMFS
  VERSION = model$(CHAMELEON_MODEL)-imfs
else
  VERSION = model$(CHAMELEON_MODEL)
endif

OUTDIR ?= noname

ifeq ($(findstring debug, $(MAKECMDGOALS)), debug)
  OBJBASE = /Chameleon.sdk/out/$(VERSION)/debug
  LIBDIR  = /Chameleon.sdk/lib/$(VERSION)/debug
else
  OBJBASE = /Chameleon.sdk/out/$(VERSION)/release
  LIBDIR  = /Chameleon.sdk/lib/$(VERSION)/release
endif

OBJDIR    = $(OBJBASE)/$(OUTDIR)

SRC_CC   ?= $(wildcard *.c)
SRC_CPP  ?= $(wildcard *.cpp)
SRC_AS   ?= $(wildcard *.s)

OBJ_CC   = $(patsubst %.c, %.o, $(addprefix $(OBJDIR)/, $(notdir $(SRC_CC))))
OBJ_CPP  = $(patsubst %.cpp, %.o, $(addprefix $(OBJDIR)/, $(notdir $(SRC_CPP))))
OBJ_AS   = $(patsubst %.s, %.o, $(addprefix $(OBJDIR)/, $(notdir $(SRC_AS))))

SRC_FILE = "$(addprefix $(CURDIR)/, $(notdir $<))"
DST_FILE = "$@"

ifeq ($(findstring rebuild, $(MAKECMDGOALS)), rebuild)

.PHONY  : all
all	: debug release ;

.PHONY  : debug
debug	:
	$(ECHO) "Removing output files in \"/Chameleon.sdk/out/$(VERSION)/debug/$(OUTDIR)\"..."
	-$(RM) -r /Chameleon.sdk/out/$(VERSION)/debug/$(OUTDIR)/
	$(MAKE) debug

.PHONY  : release
release :
	$(ECHO) "Removing output files in \"/Chameleon.sdk/out/$(VERSION)/release/$(OUTDIR)\"..."
	-$(RM) -r /Chameleon.sdk/out/$(VERSION)/release/$(OUTDIR)/
	$(MAKE) release

ifneq ($(findstring debug, $(MAKECMDGOALS)), debug)
  ifneq ($(findstring release, $(MAKECMDGOALS)), release)
    REBUILD = all
  endif
endif

.PHONY  : rebuild
rebuild : $(REBUILD) ;

else

.PHONY  : all
all	:
	$(MAKE) debug
	$(MAKE) release

ifneq ($(findstring clean, $(MAKECMDGOALS)), clean)
  COMMON = common
  .PHONY  : $(COMMON)
  $(COMMON) : $(OBJDIR) $(EXTRAGOALS) $(OBJ_CC) $(OBJ_CPP) $(OBJ_AS) $(OBJDIR)/$(APP) ;
	$(ECHO) "Application \"$(APP)\" built successfully."
endif

.PHONY  : release
release : $(COMMON);

.PHONY  : debug
debug	: $(COMMON);

.PHONY  : clean
clean	:
	$(ECHO) "Removing output files in \"/Chameleon.sdk/out/$(VERSION)/debug/$(OUTDIR)\"..."
	-$(RM) -r /Chameleon.sdk/out/$(VERSION)/debug/$(OUTDIR)/
	$(ECHO) "Removing output files in \"/Chameleon.sdk/out/$(VERSION)/release/$(OUTDIR)\"..."
	-$(RM) -r /Chameleon.sdk/out/$(VERSION)/release/$(OUTDIR)/
endif

#------------------------------------------------------------------------------

$(OBJDIR)/%.o : %.c
	$(ECHO) "Compiling \"$(SRC_FILE)\"..."
	$(CC) $(CFLAGS) $(SRC_FILE) -c -o $(DST_FILE)

$(OBJDIR)/%.o : %.cpp
	$(ECHO) "Compiling \"$(SRC_FILE)\"..."
	$(CPP) $(CFLAGS) $(SRC_FILE) -c -o $(DST_FILE)

$(OBJDIR)/%.o : %.s
	$(ECHO) "Assembling \"$(SRC_FILE)\"..."
	$(AS) $(ASFLAGS) -o $(DST_FILE) $(SRC_FILE)

$(OBJDIR)/%.d: %.c
	-$(MKDIR) $(OBJDIR)
	$(ECHO) "Generating dependencies for \"$(CURDIR)/$<\"..."
	$(CC) -M -MG $(CFLAGS) $< | $(SED) 's@^\(.*\)\.o:@$(OBJDIR)/\1.d $(OBJDIR)/\1.o:@' > $@

$(OBJDIR)/%.d: %.cpp
	-$(MKDIR) $(OBJDIR)
	$(ECHO) "Generating dependencies for \"$(CURDIR)/$<\"..."
	$(CPP) -M -MG $(CFLAGS) $< | $(SED) 's@^\(.*\)\.o:@$(OBJDIR)/\1.d $(OBJDIR)/\1.o:@' > $@

$(OBJDIR)/%.d: %.s
	-$(MKDIR) $(OBJDIR)
	$(ECHO) "Generating dependencies for \"$(CURDIR)/$<\"..."
	$(AS) -MD tmp $(ASFLAGS) -o $(<:.s=.o) $<
	-$(RM) $(<:.s=.o)
	$(SED) -e 's@^\(.*\)\.o:@$(OBJDIR)/\1.d $(OBJDIR)/\1.o:@' tmp > $@
	-$(RM) tmp

#------------------------------------------------------------------------------

ifneq ($(findstring clean, $(MAKECMDGOALS)), clean)
  ifneq ($(findstring rebuild, $(MAKECMDGOALS)), rebuild)
    -include $(OBJ_CC:.o=.d) $(OBJ_CPP:.o=.d) $(OBJ_AS:.o=.d)
  endif
endif

#------------------------------------------------------------------------------

$(OBJDIR):
	$(MKDIR) $@

ifdef APP

LD_SCRIPT  := /Chameleon.sdk/lib/$(VERSION).ld

ifdef IMFS
  IMFS_LIB := $(LIBDIR)/rtems_imfs.a
endif

$(OBJDIR)/$(APP) : $(OBJ_CC) $(OBJ_CPP) $(OBJ_AS) $(IMFS_LIB)	\
	$(LIBDIR)/chameleon.a					\
	$(LIBDIR)/rtems.a					\
	$(LIBDIR)/rtems_libcsupport.a				\
	$(LIBDIR)/midishare.a					\
	$(LIBDIR)/libc.a					\
	$(LIBDIR)/libm.a					\
	$(addprefix $(OBJBASE)/, $(EXTRAOBJS))
	$(ECHO) "Linking \"$@\"..."
	$(CC) -o $(DST_FILE) -nostdlib -Wl,--cref -Wl,-Map,$(basename $@).map -Wl,-T$(LD_SCRIPT) -Wl,-u,start $(LDFLAGS) $^ -lgcc
	$(STRIP) -s -R.comment $(DST_FILE)
endif
