################################################################################
# C H A M E L E O N    S. D. K.                                                #
################################################################################
#  $Archive:: /Chameleon.sdk/make/maindsp.mak                                  $
#     $Date:: 14/05/02 17:12                                                   $
# $Revision:: 10                                                               $
#------------------------------------------------------------------------------#
# This file is part of the Chameleon Software Development Kit                  #
#                                                                              #
# Copyright (C) 2001 Soundart                                                  #
# www.soundart-hot.com                                                         #
# support@soundart-hot.com                                                     #
################################################################################
#
# Specify the file name of the final application
# DSPAPP := app.cld
#
# Specify the output path name of the files generated
# OUTDIR := src/app1/dsp
#
# Specify extra goals to satisfy
# EXTRAGOALS := local

CHAMELEON_MODEL ?= 01

include /Chameleon.sdk/make/root.mak

cld2header =	$(ECHO) "Generating \"$(CURDIR)/$@\"..."; \
		/Chameleon.sdk/bin/cld2header $(call getpath, $^) $(call getpath, $@) -quiet $1

ifdef IMFS
  VERSION = model$(CHAMELEON_MODEL)-imfs
else
  VERSION = model$(CHAMELEON_MODEL)
endif

DSPDEPENDS += /Chameleon.sdk/include/dsp/dsp_equ.asm

OUTDIR ?= noname/dsp

ifeq ($(findstring debug, $(MAKECMDGOALS)), debug)
  OBJBASE = /Chameleon.sdk/out/$(VERSION)/debug
else
  OBJBASE = /Chameleon.sdk/out/$(VERSION)/release
endif

OBJDIR  = $(OBJBASE)/$(OUTDIR)

override DSPAPP := $(OBJDIR)/$(DSPAPP)

ifeq ($(findstring rebuild, $(MAKECMDGOALS)), rebuild)

.PHONY  : all
all	: debug release ;

.PHONY  : debug
debug	:
	$(ECHO) "Removing output files in \"/Chameleon.sdk/out/$(VERSION)/debug/$(OUTDIR)\"..."
	-$(RM) -r /Chameleon.sdk/out/$(VERSION)/debug/$(OUTDIR)/
	$(MAKE) debug

.PHONY  : release
release :
	$(ECHO) "Removing output files in \"/Chameleon.sdk/out/$(VERSION)/release/$(OUTDIR)\"..."
	-$(RM) -r /Chameleon.sdk/out/$(VERSION)/release/$(OUTDIR)/
	$(MAKE) release

ifneq ($(findstring debug, $(MAKECMDGOALS)), debug)
  ifneq ($(findstring release, $(MAKECMDGOALS)), release)
    REBUILD = all
  endif
endif

.PHONY  : rebuild
rebuild : $(REBUILD) ;

else

.PHONY  : all
all	:
	$(MAKE) debug
	$(MAKE) release

ifneq ($(findstring clean, $(MAKECMDGOALS)), clean)
  COMMON = common
  .PHONY  : $(COMMON)
  $(COMMON) : $(OBJDIR) $(EXTRAGOALS) $(DSPAPP) $(DSPHEADER) ;
endif

.PHONY  : release
release : $(COMMON);

.PHONY  : debug
debug	: $(COMMON);

.PHONY  : clean
clean	:
	$(ECHO) "Removing output files in \"/Chameleon.sdk/out/$(VERSION)/debug/$(OUTDIR)\"..."
	-$(RM) -r /Chameleon.sdk/out/$(VERSION)/debug/$(OUTDIR)/
	$(ECHO) "Removing output files in \"/Chameleon.sdk/out/$(VERSION)/release/$(OUTDIR)\"..."
	-$(RM) -r /Chameleon.sdk/out/$(VERSION)/release/$(OUTDIR)/
endif

#------------------------------------------------------------------------------

ifeq ($(words $(DSPFILES)), 0)
  DSPFILES := $(patsubst %.cld,%.asm,$(DSPAPP))
endif

DSPCLIFILES := $(patsubst %.asm,%.cli,$(filter %.asm,$(addprefix $(OBJDIR)/, $(notdir $(DSPFILES)))))
DSPCLBFILES := $(filter %.clb,$(DSPFILES))
DSPCLNFILES := $(filter %.cln,$(DSPFILES)) 
DSPLNKCTL := $(filter %.ctl,$(DSPFILES))

ifeq ($(words $(DSPLNKCTL)), 1)
  CTLOPT := -R$(call getpath, $(DSPLNKCTL))
endif

$(DSPAPP) : $(DSPCLIFILES) $(DSPCLNFILES) $(DSPCLBFILES)
	$(ECHO) "DSP Linking \"$@\"..."
	$(DSPLNK) $(DSPLNKFLAGS) -B$(call getpath, $@) -M$(call getpath, $(basename $@).map) $(CTLOPT) $(foreach f,$(DSPCLIFILES) $(DSPCLNFILES),$(call getpath, $(f))) $(foreach f,$(DSPCLBFILES),-L$(call getpath, $(f)))

$(OBJDIR)/%.cli : %.asm $(DSPDEPENDS)
	$(ECHO) "DSP Assembling \"$(CURDIR)/$<\"..."
	$(DSPASM) $(DSPASMFLAGS) -D SDK $(call getpath, /Chameleon.sdk) -B$(call getpath, $@) -L$(call getpath, $(basename $@).lst) $(call getpath, $<)

#------------------------------------------------------------------------------

DSPVARIABLE ?= dspCode

$(DSPHEADER) : $(DSPAPP)
	$(call cld2header, $(DSPVARIABLE))

#------------------------------------------------------------------------------

$(OBJDIR):
	$(MKDIR) $@
