;*******************************************************************************
; C H A M E L E O N    S. D. K.                                                *
;*******************************************************************************
;  $Archive:: /Chameleon.sdk/src/examples/cfthru/dsp/main.asm                  $
;     $Date:: 14/05/02 17:12                                                   $
; $Revision:: 5                                                                $
;------------------------------------------------------------------------------*
; This file is part of the Chameleon Software Development Kit                  *
;                                                                              *
; Copyright (C) 2001-2002 Soundart                                             *
; www.soundart-hot.com                                                         *
; support@soundart-hot.com                                                     *
;*******************************************************************************

;*******************************************************************************
; This example shows how access audio I/O using transparent DMA with a double
; buffering technique
;*******************************************************************************

 	nolist
	page	255,0
	opt	MU,S,CC,CEX,MEX,MD
	list

	nolist
	include	"SDK\include\dsp\dsp_equ.asm"
	include "SDK\include\dsp\dma.asm"
	list

;*******************************************************************************
BUFFER_SIZE	equ	256
;*******************************************************************************

	org	X:$400000

	;**********************************************************************
	; NOTE: for BUFFER_SIZE < 128, external memory must to be used
	;**********************************************************************
BufferDAC1:
	ds	2*BUFFER_SIZE
BufferDAC2:
	ds	2*BUFFER_SIZE
BufferADC1:
	ds	2*BUFFER_SIZE
BufferADC2:
	ds	2*BUFFER_SIZE


;*******************************************************************************
	org	Y:$000000

ActiveBufferDAC:
	ds	1		; Filled on Start
ActiveBufferADC:
	ds	1		; Filled on Start


;*******************************************************************************
	org	P:$000100

Start:
	CLR	A

	;**********************************************************************
	; Empty All ADC and DAC buffers
	;**********************************************************************
	MOVE	#BufferADC1,R0
	REP	#(2*2*2*BUFFER_SIZE)	; 2 DAC and 2 ADC stereo buffers
	  MOVE	A,X:(R0)+

	;**********************************************************************
	; Init Active Buffer to 2, so the first filled will be 1
	;**********************************************************************
	MOVE	#BufferDAC2,X0
	MOVE	X0,Y:ActiveBufferDAC
	MOVE	#BufferADC2,X0
	MOVE	X0,Y:ActiveBufferADC

	;**********************************************************************
	; Configure DMA for using audio double buffering
	;**********************************************************************
	ConfigDMA 2*BUFFER_SIZE,BufferDAC1,BufferADC1

	;**********************************************************************
	; Enable ESSI0 transmit and receive
	;**********************************************************************
	BSET	#CRB_TE0,X:<<CRB0
	BSET	#CRB_RE,X:<<CRB0

	BRSET	#SSISR_RFS,X:<<SSISR0,*		; Wait while receiving left frame
	BRCLR	#SSISR_RFS,X:<<SSISR0,*		; Wait while receiving right frame

MainLoop:	
	MOVE	Y:ActiveBufferADC,R0
	MOVE	Y:ActiveBufferDAC,R1

	DO	#2*BUFFER_SIZE,CopyInLoop
	  MOVE	X:(R0)+,X0
	  NOP
	  BRCLR	#1,X:<<HSR,*		; Wait for data output
	  MOVEP	X0,X:<<HTX		; Send data
CopyInLoop:

	DO	#2*BUFFER_SIZE,CopyOutLoop
	  BRCLR	#HSR_HRDF,X:<<HSR,*	; Wait for data input
	  MOVEP	X:<<HRX,X0		; Receive data
	  NOP
	  MOVE	X0,X:(R1)+
CopyOutLoop:

	BSR	ChangeActiveBuffer
	BRA	MainLoop


;------------------------------------------------------------------------------
; ChangeActiveBuffer
; This routine changes the active ADC and DAC buffers waiting if CODEC still 
; is working with it.
;------------------------------------------------------------------------------
ChangeActiveBuffer:
	MOVE	Y:ActiveBufferDAC,A
	MOVE	#BufferDAC2,Y0
	CMP	Y0,A
	BEQ	ChangeBuffer1

ChangeBuffer2:
	MOVEP	X:<<DSR0,A
	CMP	Y0,A		#BufferDAC2,X0
	BGT	ChangeBuffer2
	MOVE	X0,Y:ActiveBufferDAC
	MOVE	#BufferADC2,X0
	MOVE	X0,Y:ActiveBufferADC
	RTS

ChangeBuffer1:
	MOVEP	X:<<DSR0,A	
	CMP	Y0,A		#BufferDAC1,X0
	BLT	ChangeBuffer1
	MOVE	X0,Y:ActiveBufferDAC
	MOVE	#BufferADC1,X0
	MOVE	X0,Y:ActiveBufferADC
	RTS


	end	Start
