/*******************************************************************************
 * C H A M E L E O N    S. D. K.                                               *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/dspmem/main.c                       $
 *     $Date:: 18/02/02 11:53                                                  $
 * $Revision:: 8                                                               $
 *-----------------------------------------------------------------------------*
 * This file is part of the Chameleon Software Development Kit                 *
 *                                                                             *
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <rtems.h>
#include <chameleon.h>

#include "dsp/dsp_code.h"

/******************************************************************************/
#define WORKSPACE_SIZE	128*1024
rtems_unsigned32 rtems_workspace_size = WORKSPACE_SIZE;
rtems_unsigned32 rtems_workspace_start[WORKSPACE_SIZE];
/******************************************************************************/

rtems_boolean work(void)
{
	rtems_signed32   i, j;
	char		 text[17];
	int		 panel, dsp;

	TRACE("******************************************************************************\n");
	TRACE("DSP memory example\n");
	TRACE("******************************************************************************\n");

	panel = panel_init();
	if (!panel)
	{
		TRACE("ERROR: cannot access the panel.\n");
		return FALSE;
	}

	dsp = dsp_init(1, dspCode);
	if (!dsp)
	{
		TRACE("ERROR: cannot access the DSP.\n");
		return FALSE;
	}

	panel_out_lcd_print(panel, 0, 0, "Testing DSP DRAM");
	panel_out_lcd_print(panel, 1, 0, "Addr $XXXXXX    ");

	for (i=0x400000; i<0x800000 ;i++)
	{
		if ((i % 0x80) == 0)
		{
			TRACE("Testing range 0x%08X - 0x%08X ", i, i+0x80-1);
			fflush(stdout);
		}
		
		sprintf(text, "%06X", i);
		panel_out_lcd_print(panel, 1, 6, text);
		
		if (!dsp_write_data(dsp, &i, 1))
		{
			TRACE("BAD\n");
			TRACE("ERROR: cannot write to the DSP.\n");
			return FALSE;
		}

		if (!dsp_read_data(dsp, &j, 1))
		{
			TRACE("BAD\n");
			TRACE("ERROR: cannot read from the DSP.\n");
			return FALSE;
		}

		if (i != j)
		{
			panel_out_lcd_print(panel, 1, 13, "BAD");
			TRACE("BAD\n");
			TRACE("ERROR: bad data read.\n");
			return FALSE;
		}

		panel_out_lcd_print(panel, 1, 13, "OK");

		if ((i % 0x80) == (0x80-1))
			TRACE("OK\n");

	}

	panel_exit(panel);
	dsp_exit(dsp);

	return TRUE;
}

rtems_task rtems_main(rtems_task_argument ignored)
{
	work();
	rtems_task_delete(RTEMS_SELF);
}
