;*******************************************************************************
; C H A M E L E O N    S. D. K.                                                *
;*******************************************************************************
;  $Archive:: /Chameleon.sdk/src/examples/dspthru/dsp/dma.asm                  $
;     $Date:: 14/05/02 17:12                                                   $
; $Revision:: 5                                                                $
;------------------------------------------------------------------------------*
; This file is part of the Chameleon Software Development Kit                  *
;                                                                              *
; Copyright (C) 2001-2002 Soundart                                             *
; www.soundart-hot.com                                                         *
; support@soundart-hot.com                                                     *
;*******************************************************************************

;*******************************************************************************
; This example shows how access audio I/O using transparent DMA with a double
; buffering technique. The program uses a ramp to smooth the volume changes.
;*******************************************************************************

 	nolist
	page	255,0
	opt	MU,S,CC,CEX,MEX,MD
	list

	nolist
	include	"SDK\include\dsp\dsp_equ.asm"
	include "SDK\include\dsp\dma.asm"
	list

;*******************************************************************************
BUFFER_SIZE	equ	16
;*******************************************************************************

	org	X:$400000

	;**********************************************************************
	; NOTE: for BUFFER_SIZE < 128, external memory must to be used
	;**********************************************************************
BufferDAC1:
	ds	2*BUFFER_SIZE
BufferDAC2:
	ds	2*BUFFER_SIZE
BufferADC1:
	ds	2*BUFFER_SIZE
BufferADC2:
	ds	2*BUFFER_SIZE


;*******************************************************************************
	org	Y:$000000

ActiveBufferDAC:
	ds	1		; Filled on Start
ActiveBufferADC:
	ds	1		; Filled on Start
MasterVolumeTarget:
	ds	1		; Filled on Start
MasterVolume:
	ds	1		; Filled on Start


;*******************************************************************************
	org	P:$000100

Start:
	CLR	A

	;**********************************************************************
	; Configure DMA for using audio double buffering
	;**********************************************************************
	ConfigDMA 2*BUFFER_SIZE,BufferDAC1,BufferADC1

	;**********************************************************************
	; Init Master Volume variables
	;**********************************************************************
	MOVE	A,Y:MasterVolumeTarget
	MOVE	A,Y:MasterVolume

	;**********************************************************************
	; Empty All ADC and DAC buffers
	;**********************************************************************
	MOVE	#BufferADC1,R0
	REP	#(2*2*2*BUFFER_SIZE)	; 2 DAC and 2 ADC stereo consecutive buffers
	  MOVE	A,X:(R0)+

	;**********************************************************************
	; Init Active Buffer to 2, so the first filled will be 1
	;**********************************************************************
	MOVE	#BufferDAC2,X0
	MOVE	X0,Y:ActiveBufferDAC
	MOVE	#BufferADC2,X0
	MOVE	X0,Y:ActiveBufferADC

	;**********************************************************************
	; Enable ESSI0 transmit and receive
	;**********************************************************************
	BSET	#CRB_TE0,X:<<CRB0
	BSET	#CRB_RE,X:<<CRB0

	BRSET	#SSISR_RFS,X:<<SSISR0,*		; Wait while receiving left frame
	BRCLR	#SSISR_RFS,X:<<SSISR0,*		; Wait while receiving right frame

MainLoop:	
	BRCLR	#HSR_HRDF,X:<<HSR,NoHostData	; host data input?
	MOVEP	X:<<HRX,X0
	MOVE	X0,Y:MasterVolumeTarget

NoHostData:
	MOVE	Y:ActiveBufferADC,R0
	MOVE	Y:ActiveBufferDAC,R1

	MOVE	Y:MasterVolumeTarget,B		; target volume value
	MOVE	Y:MasterVolume,A		; current volume value
	SUB	A,B				
	ASR	#10,B,B				; increment

	MOVE	X:(R0)+,X1			; X1=x1(n)
	MOVE	B,X0
	DO	#BUFFER_SIZE,CopyLoop
	  MOVE	A,Y1
	  MPY	+Y1,X1,A	X:(R0)+,X1	; A=x1(n)*volume, X1=x2(n)
	  MPY	+Y1,X1,B	X:(R0)+,X1	; B=x2(n)*volume, X1=x1(n)
	  TFR	Y1,A		A,X:(R1)+	; y1(n)=A
	  ADD	X0,A		B,X:(R1)+	; current volume += increment, y2(n)=B
 	  NOP
CopyLoop:
	MOVE	A,Y:MasterVolume

	BSR	ChangeActiveBuffer

	BRA	MainLoop

;------------------------------------------------------------------------------
; ChangeActiveBuffer
; This routine changes the active ADC and DAC buffers waiting if CODEC still 
; is working with it.
;------------------------------------------------------------------------------
ChangeActiveBuffer:
	MOVE	Y:ActiveBufferDAC,A
	MOVE	#BufferDAC2,Y0
	CMP	Y0,A
	BEQ	ChangeBuffer1

ChangeBuffer2:
	MOVEP	X:<<DSR0,A
	CMP	Y0,A		#BufferDAC2,X0
	BGT	ChangeBuffer2
	MOVE	X0,Y:ActiveBufferDAC
	MOVE	#BufferADC2,X0
	MOVE	X0,Y:ActiveBufferADC
	RTS

ChangeBuffer1:
	MOVEP	X:<<DSR0,A	
	CMP	Y0,A		#BufferDAC1,X0
	BLT	ChangeBuffer1
	MOVE	X0,Y:ActiveBufferDAC
	MOVE	#BufferADC1,X0
	MOVE	X0,Y:ActiveBufferADC
	RTS


	end	Start
