;*******************************************************************************
; C H A M E L E O N    S. D. K.                                                *
;*******************************************************************************
;  $Archive:: /Chameleon.sdk/src/examples/dspthru/dsp/interrupt.asm            $
;     $Date:: 14/05/02 17:12                                                   $
; $Revision:: 5                                                                $
;------------------------------------------------------------------------------*
; This file is part of the Chameleon Software Development Kit                  *
;                                                                              *
; Copyright (C) 2001-2002 Soundart                                             *
; www.soundart-hot.com                                                         *
; support@soundart-hot.com                                                     *
;*******************************************************************************

;*******************************************************************************
; This example shows how access audio I/O using interrupts
; The program uses a ramp to smooth the volume changes.
;*******************************************************************************

 	nolist
	page	255,0
	opt	MU,S,CC,CEX,MEX,MD
	list

	nolist
	include	"SDK\include\dsp\dsp_equ.asm"
	list


;*******************************************************************************
	org	L:$000000

SampleMainInput:
	ds	1	; X:left, Y:right
SampleMainOutput:
	ds	1	; X:left, Y:right
InterruptStack:
	ds	17

EndMemL:


;*******************************************************************************
	org	Y:EndMemL

MasterVolumeTarget:
	ds	1		; Filled on Start
MasterVolume:
	ds	1		; Filled on Start


;*******************************************************************************
	org	P:VecEssi0RxData

	JSR	>IntEssi0RxData	


;*******************************************************************************
	org	P:$000100

Start:
	CLR	A

	;**********************************************************************
	; Init Master Volume variables
	;**********************************************************************
	NOP
	MOVE	A,L:SampleMainOutput		; Clear output audio buffer
	MOVE	A,Y:MasterVolumeTarget
	MOVE	A,Y:MasterVolume

	;**********************************************************************
	; Enable ESSI0 transmit, receive, and receive interrupt
	;**********************************************************************
	BSET	#CRB_RIE,X:<<CRB0
	BSET	#CRB_RE,X:<<CRB0
	BSET	#CRB_TE0,X:<<CRB0

	ANDI	#<$FC,MR			; Enable all interrupts

MainLoop:	
	BRCLR	#HSR_HRDF,X:<<HSR,*		; host data input?
	MOVEP	X:<<HRX,Y:MasterVolumeTarget	; set new master volume value
	BRA	MainLoop


;*******************************************************************************

IntEssi0RxData:
	BRCLR	#SSISR_RFS,X:<<SSISR0,ReceivedChannelRight
ReceivedChannelLeft:
	MOVEP	X:<<RX0,X:SampleMainInput
	MOVEP	X:SampleMainOutput,X:<<TX00
	RTI

ReceivedChannelRight:
	MOVEP	X:<<RX0,Y:SampleMainInput
	MOVEP	Y:SampleMainOutput,X:<<TX00

	;**********************************************************************
	; Save processor context to use any register in the ISR
	;**********************************************************************
	MOVE	R7,X:InterruptStack
	MOVEC	M7,Y:InterruptStack
	MOVE	#InterruptStack+1,R7
	MOVEC	#$FFFFFF,M7

	ANDI	#<$FC,MR		; ReEnable all interrupts to enable Left Channel
					; (reentrant interrupt)
	
	MOVE	X,L:(R7)+		; InterruptStack + 00
	MOVE	Y,L:(R7)+		; InterruptStack + 01
	MOVE	A10,L:(R7)+		; InterruptStack + 02
	MOVE	B10,L:(R7)+		; InterruptStack + 03
	MOVE	A2,X:(R7)		; InterruptStack + 04
	MOVE	B2,Y:(R7)+
	MOVE	R0,X:(R7)		; InterruptStack + 05
	MOVEC	M0,Y:(R7)+
	MOVEC	M7,M0
	MOVE	R1,X:(R7)		; InterruptStack + 06
	MOVEC	M1,Y:(R7)+
	MOVEC	M7,M1
	MOVE	R2,X:(R7)		; InterruptStack + 07
	MOVEC	M2,Y:(R7)+
	MOVEC	M7,M2
	MOVE	R3,X:(R7)		; InterruptStack + 08
	MOVEC	M3,Y:(R7)+
	MOVEC	M7,M3
	MOVE	R4,X:(R7)		; InterruptStack + 09
	MOVEC	M4,Y:(R7)+
	MOVEC	M7,M4
	MOVE	R5,X:(R7)		; InterruptStack + 10
	MOVEC	M5,Y:(R7)+
	MOVEC	M7,M5
	MOVE	R6,X:(R7)		; InterruptStack + 11
	MOVEC	M6,Y:(R7)+
	MOVEC	M7,M6
	MOVE	N0,X:(R7)		; InterruptStack + 12
	MOVE	N1,Y:(R7)+
	MOVE	N2,X:(R7)		; InterruptStack + 13
	MOVE	N3,Y:(R7)+
	MOVE	N4,X:(R7)		; InterruptStack + 14
	MOVE	N5,Y:(R7)+
	MOVE	N6,X:(R7)		; InterruptStack + 15
	MOVE	N7,Y:(R7)+

	BSR	DoProcessing

	;**********************************************************************
	; Restore the processor context saved before
	;**********************************************************************
	MOVE	L:(R7)+,X		; InterruptStack + 00
	MOVE	L:(R7)+,Y		; InterruptStack + 01
	MOVE	L:(R7)+,A10		; InterruptStack + 02
	MOVE	L:(R7)+,B10		; InterruptStack + 03
	MOVE	X:(R7),A2		; InterruptStack + 04
	MOVE	Y:(R7)+,B2
	MOVE	X:(R7),R0		; InterruptStack + 05
	MOVEC	Y:(R7)+,M0
	MOVE	X:(R7),R1		; InterruptStack + 06
	MOVEC	Y:(R7)+,M1
	MOVE	X:(R7),R2		; InterruptStack + 07
	MOVEC	Y:(R7)+,M2
	MOVE	X:(R7),R3		; InterruptStack + 08
	MOVEC	Y:(R7)+,M3
	MOVE	X:(R7),R4		; InterruptStack + 09
	MOVEC	Y:(R7)+,M4
	MOVE	X:(R7),R5		; InterruptStack + 10
	MOVEC	Y:(R7)+,M5
	MOVE	X:(R7),R6		; InterruptStack + 11
	MOVEC	Y:(R7)+,M6
	MOVE	X:(R7),N0		; InterruptStack + 12
	MOVE	Y:(R7)+,N1
	MOVE	X:(R7),N2		; InterruptStack + 13
	MOVE	Y:(R7)+,N3
	MOVE	X:(R7),N4		; InterruptStack + 14
	MOVE	Y:(R7)+,N5
	MOVE	X:(R7),N6		; InterruptStack + 15
	MOVE	Y:(R7)+,N7

	MOVE	X:InterruptStack,R7
	MOVEC	Y:InterruptStack,M7
	RTI

DoProcessing:
	MOVE	Y:MasterVolumeTarget,B			; target volume value
	MOVE	Y:MasterVolume,A			; current volume value
	SUB	A,B				
	ASR	#10,B,B					; increment
	ADD	A,B					; current volume value += increment
	NOP
	MOVE	B,Y:MasterVolume

	MOVE	X:SampleMainInput,X0	B,Y0		; X0=left sample input
	MPYR	X0,Y0,A		Y:SampleMainInput,Y1	; Y1=right sample input
	MPYR	Y1,Y0,B
	MOVE	A,X:SampleMainOutput			; A=left sample output
	MOVE	B,Y:SampleMainOutput			; B=right sample output

	RTS

	end	Start
