;*******************************************************************************
; C H A M E L E O N    S. D. K.                                                *
;*******************************************************************************
;  $Archive:: /Chameleon.sdk/src/examples/HOSTCOMMANDS/dsp/main.asm            $
;     $Date:: 14/05/02 16:11                                                   $
; $Revision:: 1                                                                $
;------------------------------------------------------------------------------*
; This file is part of the Chameleon Software Development Kit                  *
;                                                                              *
; Copyright (C) 2001-2002 Soundart                                             *
; www.soundart-hot.com                                                         *
; support@soundart-hot.com                                                     *
;*******************************************************************************

;*******************************************************************************
; This example shows how to use DSP host commands
; The program uses a ramp to smooth the volume changes.
;*******************************************************************************

 	nolist
	page	255,0
	opt	MU,S,CC,CEX,MEX,MD
	list

	nolist
	include	"SDK\include\dsp\dsp_equ.asm"
	list


;*******************************************************************************
HOST_BUFFER_SIZE	equ	16
;*******************************************************************************

	org 	X:$000000

MasterVolTarget_L:
		ds	1	; Stores the Channel L target volume
MasterVolTarget_R:
		ds	1	; Stores the Channel R target volume 

MasterVol_L:	ds	1	; Stores the Channel L current volume
MasterVol_R:	ds	1	; Stores the Channel R current volume 

Flags:		ds	1	; Bit #0 as Mute Flag (Flags[0] = 1 means muted)


	org	X:
		; Circular buffer to store the data received on the Host port
Host_Buffer:	dsm	HOST_BUFFER_SIZE


	;**********************************************************************
	; Interrupts
	;**********************************************************************

	; Interrupt on Host Data Received
	org	P:VecHostRxDataFull
	MOVEP	X:<<HRX,X:(R0)+		; Write incoming word to the input buffer
	NOP

	; Host Command Interrupts:
	org 	P:VecHostCommandDefault
VecHostCommandUpdateLevels:
	JSR	>Update_Levels
	
	; Toggle the Mute/Unmute flag
VecHostCommandToggleMute:
	BCHG	#0,X:>Flags


;*******************************************************************************
	org	P:$000100

Start:
	CLR	A

	;**********************************************************************
	; Init Pointers to the Host Receive Buffer
	;**********************************************************************

	MOVE	#HOST_BUFFER_SIZE-1,M0	
	MOVE	#Host_Buffer,R0		; R0 is used by the Host Receive interrupt routine
					; to fill the input buffer

	MOVE	#HOST_BUFFER_SIZE-1,M1		
	MOVE	#Host_Buffer,R1		; R1 is used to read the input buffer
	
	;**********************************************************************
	; Init Master Volume and Flags
	;**********************************************************************
	
	MOVE	A,X:MasterVolTarget_L
	MOVE	A,X:MasterVolTarget_R
	MOVE	A,X:MasterVol_L
	MOVE	A,X:MasterVol_R
	
	BCLR	#0,X:Flags		; Unmuted by default 

	;**********************************************************************
	; Enable HOST Receive and Command interrupts
	;**********************************************************************
	
	BSET	#HCR_HRIE,X:<<HCR	; Enable Host Receive Interrupts
	BSET	#HCR_HCIE,X:<<HCR	; Enable Host Command Interrupts

	;**********************************************************************
	; Enable ESSI0 transmit and receive
	;**********************************************************************

	BSET	#CRB_TE0,X:<<CRB0	; Enable Transmit 0
	BSET	#CRB_RE,X:<<CRB0	; Enable Receive  0

	;**********************************************************************
	; Synchronize L and R channels
	;**********************************************************************

	BRSET	#SSISR_RFS,X:<<SSISR0,*	; Wait while receiving left frame
	BRCLR	#SSISR_RFS,X:<<SSISR0,*	; Wait while receiving right frame

	ANDI	#<$FC,MR		; Enable interrupts	

	;**********************************************************************
	; Main Loop
	;**********************************************************************

MainLoop:	
	CLR	A
	BRSET	#0,X:Flags,Mute			; Check the Mute flag status
	MOVE	X:MasterVol_L,X0		; If not muted, read the normal volume values
	MOVE	X:MasterVol_R,X1
	BRA	Continue
Mute:
	MOVE	A,X0				; If muted, zero the gain multipliers
	MOVE	A,X1
Continue:

	; Channel L
	BRCLR	#SSISR_RDF,X:<<SSISR0,*		; Wait until receive register is full
	MOVEP	X:<<RX0,Y0			; Get new input sample in Y0
	MPY	X0,Y0,A				; Scale the input sample according to the volume
	BRCLR	#SSISR_TDE,X:<<SSISR0,*		; Wait until transmiter register is empty
	MOVEP	A,X:<<TX00			; Set new output sample

	; Channel R
	BRCLR	#SSISR_RDF,X:<<SSISR0,*		; Wait until receive register is full
	MOVEP	X:<<RX0,Y0			; Get new input sample in Y0
	MPY	X1,Y0,A				; Scale the input sample according to the volume
	BRCLR	#SSISR_TDE,X:<<SSISR0,*		; Wait until transmiter register is empty
	MOVEP	A,X:<<TX00			; Set new output sample

	;**********************************************************************
	; Makes the volume ramping to smooth abrupt volume changes
	;**********************************************************************

	MOVE	X:MasterVol_L,A			; current left volume value
	MOVE	X:MasterVolTarget_L,B		; target left volume value
	SUB	A,B				
	ASR	#10,B,B				; increment
	ADD	A,B				; current volume value += increment
	NOP
	MOVE	B,X:MasterVol_L

	MOVE	X:MasterVol_R,A			; current left volume value
	MOVE	X:MasterVolTarget_R,B		; target left volume value
	SUB	A,B				
	ASR	#10,B,B				; increment
	ADD	A,B				; current volume value += increment
	NOP
	MOVE	B,X:MasterVol_R

	BRA	MainLoop


;------------------------------------------------------------------------------

Update_Levels:
	MOVE 	X:(R1)+,Y1			; Read the new volumes from the input buffer
	MOVE	Y1,X:MasterVolTarget_L
	MOVE 	X:(R1)+,Y1
	MOVE	Y1,X:MasterVolTarget_R
	RTI


	end	Start
