;*******************************************************************************
; C H A M E L E O N    S. D. K.                                                *
;*******************************************************************************
;  $Archive:: /Chameleon.sdk/src/examples/levelmeter/dsp/main.asm              $
;     $Date:: 14/05/02 19:03                                                   $
; $Revision:: 7                                                                $
;------------------------------------------------------------------------------*
; This file is part of the Chameleon Software Development Kit                  *
;                                                                              *
; Copyright (C) 2001-2002 Soundart                                             *
; www.soundart-hot.com                                                         *
; support@soundart-hot.com						       *
;*******************************************************************************

;*******************************************************************************
; This example implements a simple level meter
;*******************************************************************************

 	nolist
	page	255,0
	opt	INTR,MU,S,CC,CEX,MEX,MD
	list

	nolist
	include	"SDK\include\dsp\dsp_equ.asm"
	include 'process.asm'
	list

;*******************************************************************************

SIMULATING	equ	0		; Change this value to 1 to load the code 
					; into the DSP simulator

SIM_LOOP_SIZE	equ	4095		; Simulation loop size in samples

;*******************************************************************************

AT		equ	0.15		; Attack time constant
RT		equ	0.999		; Release time constant
GAT		equ	1-AT		; System gain for the attack case
GRT		equ	1-RT		; System gain for the release case

;*******************************************************************************
		
		org	Y:$000000
		
		buffer	M,6

EnvelopeGainR:	ds	1		; R Channel system coeffs.
TimeConstR:	ds	1	
LevelR:		ds	1
EnvelopeGainL:	ds	1		; L Channel system coeffs.
Time_ConstL:	ds	1	
LevelL:		ds	1

		endbuf
	
Levels:		ds	1		; Concatenate R & L levels to send to
					; the ColdFire

		org	X:$000000

MasterVolume:		ds	1	; Current Master Volume
MasterVolumeTarget:	ds	1	; Auxiliary for Volume change smoothing ramp


;*******************************************************************************
; DSP registers usage
;*******************************************************************************

; - Volume is readed from host port (X:<<HRX) and stored in X0
; - Audio input is readed from the ESSI0 port (X:<<RX0) and stored in Y0
; - Audio Output is writed to the ESSI0 port (X:<<TX00)
; - Envelope values are truncated to 12 bits for each channel and
;   concatenated into a single 24 bit word which is written to the 
;   host port (X:<<HTX)
; - The monitor source is selected by reading the Host port
;   FLAG0 (X:<<HSR, bit 3). The Coldfire writes this flag when the 
;   user switches the monitor source.

;*******************************************************************************

	org	P:$000100

Start:
	CLR	A

	;**********************************************************************
	; Init Level Meter Coefficients & Variables
	;**********************************************************************

	MOVE	#5,M4
	MOVE	#EnvelopeGainR,R4
	
	MOVE	#GAT,X0
	MOVE	#AT,X1

	MOVE	X0,Y:(R4)+
	MOVE	X1,Y:(R4)+
	MOVE	A,Y:(R4)+

	MOVE	X0,Y:(R4)+
	MOVE	X1,Y:(R4)+
	MOVE	A,Y:(R4)+
	
	MOVE	A,Y:Levels

	;**********************************************************************
	; Init Master Volume volume
	;**********************************************************************

	MOVE	A,X:MasterVolume	; Default Volume set to zero
	MOVE	A,X:MasterVolumeTarget

	;**********************************************************************
	; Init Monitor Source
	;**********************************************************************

	BSET	#HSR_HF0,X:<<HSR	; Monitor input by default

	;**********************************************************************
	; Enable ESSI0 transmit and receive
	;**********************************************************************

  if !SIMULATING

  	; ESSI0 port signals are not set and checked when simulating the algorithm

	BSET	#CRB_TE0,X:<<CRB0
	BSET	#CRB_RE,X:<<CRB0

	BRSET	#SSISR_RFS,X:<<SSISR0,*	; Wait while receiving left frame
	BRCLR	#SSISR_RFS,X:<<SSISR0,*	; Wait while receiving right frame

  endif


MainLoop:	

  if !SIMULATING

	include 'main_loop.asm'

  else

	include 'main_loop_sim.asm'

  endif

	BRA	MainLoop


	end	Start
