/*******************************************************************************
 * C H A M E L E O N    S. D. K.                                               *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/midimon/console.c                   $
 *     $Date:: 14/05/02 17:12                                                  $
 * $Revision:: 3                                                               $
 *-----------------------------------------------------------------------------*
 * This file is part of the Chameleon Software Development Kit                 *
 *                                                                             *
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <rtems.h>
#include <midishare.h>
#include <chameleon.h>
#include "main.h"

static const char* gEvNames_0_19[] = 
{
		"Note","KeyOn","KeyOff","KeyPress","CtrlChange","ProgChange",
		"ChanPress","PitchWheel","SongPos","SongSel","Clock","Start",
		"Continue","Stop","Tune","ActiveSens","Reset","SysEx","Stream", "Private"
};

static const char* gEvNames_128_150[] = 
{
		"Process", "DProcess", "QuarterFrame", "Ctrl14b", 
		"NonRegParam", "RegParam", "SeqNum", "Text", "Copyright", "SeqName", 
		"InstrName", "Lyric", "Marker", "CuePoint", "ChanPrefix", "EndTrack", 
		"Tempo", "SMPTEOffset", "TimeSign", "KeySign", "Specific", "RcvAlarm", 
		"ApplAlarm"
};

static Ev2StreamRec xmt;

static const char* EvName(MidiEvPtr e)
{
	if (EvType(e) < typePrivate) 
		return gEvNames_0_19[EvType(e)];

	if (EvType(e) < typeProcess) 
		return "Private";

	if (EvType(e) < typeReserved) 
		return gEvNames_128_150[EvType(e) - typeProcess];

	return "Undefined";
}

static void display_1_event(MidiEvPtr ev)
{
	long t, hh, mn, ss, nnn;
	MidiEvPtr tmp;
	int i;
	BYTE c;
	
	t = Date(ev);
	nnn = t%1000; t /= 1000;
	ss = t%60; t /= 60;
	mn = t%60; t /= 60;
	hh = t%60; 
	
	TRACE("%02ld:%02ld:%02ld.%03ld ", hh,mn,ss,nnn);
	TRACE("  %d   ", Port(ev));

	MidiStreamReset(&xmt);
	tmp = MidiStreamPutEvent(&xmt, ev);
	if (tmp)
		MidiFreeCell(tmp);

	for (i=0; i<3 ;i++)
	{
		if (MidiStreamGetByte(&xmt, &c))
			TRACE("  %02X  ", c);
		else
			TRACE("  --  ");
	}

	TRACE(" %2d  ", Chan(ev) + 1);

	if ((EvType(ev) == typeKeyOn) || (EvType(ev) == typeKeyOff))
	{
		long Note = MidiGetField(ev, 0);
		TRACE("%s%ld", gNoteNames[Note%12], (Note/12) - 1);
		if (strlen(gNoteNames[Note%12]) == 1)
			TRACE(" ");
	}
	else
		TRACE("---");

	TRACE("  %s", EvName(ev));
	TRACE("\n");
}

static void receive_alarm(short ref)
{
	rtems_event_send((rtems_id) MidiGetInfo(ref), EVENT_MIDI);
}

static rtems_task console_task(rtems_task_argument argument)
{
	MidiEvPtr		ev;
	rtems_event_set		pending;
	rtems_status_code	status;
	rtems_id		task_id;
	short			refMidi;

	TRACE("Starting console_task...\n");

	MidiStreamInit(&xmt, lTbl);

	refMidi = MidiOpen("Midi Monitor (console)");
	if (refMidi < 0)
	{
		TRACE("ERROR: cannot open MidiShare.\n");
		rtems_task_delete(RTEMS_SELF);
	}

	rtems_task_ident(RTEMS_SELF, 0, &task_id);

	MidiSetInfo(refMidi, (void *) task_id);
	MidiSetRcvAlarm(refMidi, receive_alarm);

	MidiConnect(0, refMidi, TRUE);

	TRACE("\n");
	TRACE("TIMESTAMP    PORT STATUS DATA1 DATA2 CHAN NOTE EVENT\n");
	TRACE("------------------------------------------------------------------\n");

	while (TRUE)
	{
		status = rtems_event_receive(
			EVENT_MIDI,
			RTEMS_WAIT | RTEMS_EVENT_ANY,
			RTEMS_NO_TIMEOUT,
			&pending
		);
		if (status != RTEMS_SUCCESSFUL)
			break;

		while ((ev = MidiGetEv(refMidi)))
			display_1_event(ev);
	}

	MidiConnect(0, refMidi, FALSE);

	MidiClose(refMidi);

	TRACE("Ending console_task...\n");
	rtems_task_delete(RTEMS_SELF);
}

rtems_boolean create_console_task(void)
{
	rtems_id		task_id;
	rtems_status_code	status;

	status = rtems_task_create(
		rtems_build_name('C', 'M', 'O', 'N'),
		100,
		RTEMS_MINIMUM_STACK_SIZE,
		RTEMS_DEFAULT_MODES,
		RTEMS_DEFAULT_ATTRIBUTES,
		&task_id
	);
	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: can't create console_task.\n");
		return FALSE;
	}

	status = rtems_task_start(task_id, console_task, 0);
	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: can't start console_task.\n");
		return FALSE;
	}

	return TRUE;
}
