/*******************************************************************************
 * C H A M E L E O N    S. D. K.                                               *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/midimon/panel.c                     $
 *     $Date:: 14/05/02 17:12                                                  $
 * $Revision:: 3                                                               $
 *-----------------------------------------------------------------------------*
 * This file is part of the Chameleon Software Development Kit                 *
 *                                                                             *
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <rtems.h>
#include <midishare.h>
#include <chameleon.h>
#include "main.h"

static const char* gEvNames_0_19[] = 
{
	"Note",
	"Key On ",
	"Key Off",
	"Key Press",
	"Control Change",
	"Program Change",
	"Channel Press",
	"Pitch Wheel",
	"Song Pos",
	"Song Sel",
	"Clock",
	"Start",
	"Continue",
	"Stop",
	"Tune",
	"Active Sense",
	"Reset",
	"Sys Exclusive",
	"Stream", 
	"Private"
};

static Ev2StreamRec xmt;

static const char* EvName(MidiEvPtr e)
{
	if (EvType(e) < typePrivate) 
		return gEvNames_0_19[EvType(e)];

	return "Undefined";
}

static void display_1_event(int panel, MidiEvPtr ev)
{
	MidiEvPtr tmp;
	int i, j;
	BYTE c;
	char text1[17];
	char text2[17];
	
	MidiStreamReset(&xmt);
	tmp = MidiStreamPutEvent(&xmt, ev);
	if (tmp)
		MidiFreeCell(tmp);

	sprintf(text1, "%16s", "");
	if ((EvType(ev) == typeKeyOn) || (EvType(ev) == typeKeyOff))
	{
		long Note = MidiGetField(ev, 0);
		sprintf(text2, "%s %s%ld", EvName(ev), gNoteNames[Note%12], (Note/12) - 1);
	}
	else
		sprintf(text2, "%s", EvName(ev));

	for (i=0, j=(16-strlen(text2))/2; i<strlen(text2) ;i++, j++)
		text1[j] = text2[i];

	panel_out_lcd_print(panel, 0, 0, text1);

	text1[0] = ' ';
	text1[1] = '\0';
	for (i=0; i<3 ;i++)
	{
		if (MidiStreamGetByte(&xmt, &c))
		{
			sprintf(text2, "%02X ", c);
			strcat(text1, text2);
		}
		else
			strcat(text1, "-- ");
	}

	panel_out_lcd_print(panel, 1, 3, text1);
}

static void receive_alarm(short ref)
{
	rtems_event_send((rtems_id) MidiGetInfo(ref), EVENT_MIDI);
}

static rtems_task panel_task(rtems_task_argument argument)
{
	MidiEvPtr		ev;
	rtems_event_set		pending;
	rtems_status_code	status;
	rtems_id		task_id;
	rtems_id		clear_display_task_id = (rtems_id) argument;
	short			refMidi;
	int			panel;

	TRACE("Starting panel_task...\n");

	panel = panel_init();
	if (!panel)
	{
		TRACE("ERROR: cannot access the panel.\n");
		rtems_task_delete(RTEMS_SELF);
	}

	rtems_task_restart(clear_display_task_id, panel);

	MidiStreamInit(&xmt, lTbl);

	refMidi = MidiOpen("Midi Monitor (panel)");
	if (refMidi < 0)
	{
		TRACE("ERROR: cannot open MidiShare.\n");
		rtems_task_delete(RTEMS_SELF);
	}

	rtems_task_ident(RTEMS_SELF, 0, &task_id);

	MidiSetInfo(refMidi, (void *) task_id);
	MidiSetRcvAlarm(refMidi, receive_alarm);

	MidiConnect(0, refMidi, TRUE);
	MidiConnect(refMidi, 0, TRUE);

	while (TRUE)
	{
		status = rtems_event_receive(
			EVENT_MIDI,
			RTEMS_WAIT | RTEMS_EVENT_ANY,
			RTEMS_NO_TIMEOUT,
			&pending
		);
		if (status != RTEMS_SUCCESSFUL)
			break;

		ev = NULL; 
		while (MidiCountEvs(refMidi))
		{
			if (ev)
				MidiFreeEv(ev);

			ev = MidiGetEv(refMidi);
		}

		if (ev)
		{
			panel_out_led(panel, PANEL01_LED_EDIT);
			display_1_event(panel, ev);
			panel_out_led(panel, 0);
		}

		rtems_task_restart(clear_display_task_id, panel);
	}

	panel_exit(panel);

	MidiConnect(0, refMidi, FALSE);
	MidiConnect(refMidi, 0, FALSE);

	MidiClose(refMidi);

	TRACE("Ending panel_task...\n");
	rtems_task_delete(RTEMS_SELF);
}

static rtems_task clear_display_task(rtems_task_argument argument)
{
	int panel = argument;

	if (panel)
	{
		rtems_task_wake_after(750);
		panel_out_lcd_print(panel, 0, 0, "  Midi Monitor  ");

		while (TRUE)
		{
			panel_out_lcd_print(panel, 1, 0, "   (Waiting)    ");
			rtems_task_wake_after(500);
			panel_out_lcd_print(panel, 1, 0, "    Waiting     ");
			rtems_task_wake_after(500);
		}
	}

	rtems_task_suspend(RTEMS_SELF);
}

static rtems_id create_clear_display_task(rtems_task_argument argument)
{
	rtems_id		task_id;
	rtems_status_code	status;
	rtems_name		name;

	name = rtems_build_name('T', 'C', 'L', 'R');

	status = rtems_task_create(
		name,
		100,
		RTEMS_MINIMUM_STACK_SIZE,
		RTEMS_DEFAULT_MODES,
		RTEMS_DEFAULT_ATTRIBUTES,
		&task_id
	);
	if (status != RTEMS_SUCCESSFUL)
		return 0;

	status = rtems_task_start(task_id, clear_display_task, argument);
	if (status != RTEMS_SUCCESSFUL)
		return 0;

	return task_id;
}

rtems_boolean create_panel_task(void)
{
	rtems_id		task_id;
	rtems_status_code	status;

	status = rtems_task_create(
		rtems_build_name('P', 'M', 'O', 'N'),
		50,
		RTEMS_MINIMUM_STACK_SIZE,
		RTEMS_DEFAULT_MODES,
		RTEMS_DEFAULT_ATTRIBUTES,
		&task_id
	);
	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: can't create panel_task.\n");
		return FALSE;
	}

	status = rtems_task_start(task_id, panel_task, create_clear_display_task(0));
	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: can't start panel_task.\n");
		return FALSE;
	}

	return TRUE;
}
