/*******************************************************************************
 * C H A M E L E O N                                                           *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/monosynth/MonoSynthEngine.cpp       $
 *     $Date:: 17/05/02 12:19                                                  $
 * $Revision:: 3                                                               $
 *-----------------------------------------------------------------------------*
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#include <math.h>
#include <rtems.h>
#include <midishare.h>
#include <player.h>
#include <chameleon.h>

#include "MonoSynthEngine.h"
#include "MonoSynthPanel.h"
#include "MonoSynthParameters.h"

//////////////////////////////////////////////////////////////////////////////
// NOTE: You can send the file DEMOS/BANK.MID using MIDI to load some presets
//////////////////////////////////////////////////////////////////////////////
enum
{
	kEngineMsgParameterChanged = kEngineMsgUser,
};
//////////////////////////////////////////////////////////////////////////////

struct CEngineMsg_ParameterChanged : public CMessage
{
	WORD wID;

	CEngineMsg_ParameterChanged(void)
	{
		ASSERT(GetSize() <= ENGINE_MAX_MESSAGE_SIZE);
		dwMsg = kEngineMsgParameterChanged;
	}

	virtual const DWORD GetSize(void) { return sizeof(CEngineMsg_ParameterChanged); }
};

//////////////////////////////////////////////////////////////////////////////
#define MONOSYNTH_CONFIG_MAGIC		"Chameleon MonoSynth"
#define MONOSYNTH_CONFIG_VERSION	0x0100
//////////////////////////////////////////////////////////////////////////////
#define MONOSYNTH_PRESET_VERSION	0x00
//////////////////////////////////////////////////////////////////////////////

struct CPreset
{
	char	strName[kMaxPresetNameLen];

	BYTE	byVersion;
	BYTE	byData[kMaxPresetParameters];
};

struct CConfiguration
{
	char	strMagic[32];
	WORD	wVersion;

	BYTE	byData[kMaxGlobalParameters];
};

//////////////////////////////////////////////////////////////////////////////

void CMonoSynthEngine::InitializeGlobalParameters(void)
{
	for (int i=0; i<kMaxGlobalParameters ;i++)
		m_tableGlobalParameters[i] = kParameterNone;
	
	m_tableGlobalParameters[  0] = kGlobalInputThru;
	m_tableGlobalParameters[  1] = kGlobalMidiChannel;
	m_tableGlobalParameters[  2] = kGlobalMidiThru;
	m_tableGlobalParameters[  3] = kGlobalMidiDeviceID;
}

void CMonoSynthEngine::InitializePresetParameters(void)
{
	for (int i=0; i<kMaxPresetParameters ;i++)
		m_tablePresetParameters[i] = kParameterNone;
	
	m_tablePresetParameters[  0] = kOsc1Waveform;
	m_tablePresetParameters[  1] = kOsc1Transpose;
	m_tablePresetParameters[  2] = kOsc1Volume;

	m_tablePresetParameters[  3] = kOsc2Waveform;
	m_tablePresetParameters[  4] = kOsc2Transpose;
	m_tablePresetParameters[  5] = kOsc2Detune;
	m_tablePresetParameters[  6] = kOsc2Volume;

	m_tablePresetParameters[  7] = kMixNoiseVolume;
	m_tablePresetParameters[  8] = kMixInputVolume;

	m_tablePresetParameters[  9] = kFilterCutoff;
	m_tablePresetParameters[ 10] = kFilterResonance;
	m_tablePresetParameters[ 11] = kFilterEnvDepth;
	m_tablePresetParameters[ 12] = kFilterKeyDepth;

	m_tablePresetParameters[ 13] = kFilterEnvAttack;
	m_tablePresetParameters[ 14] = kFilterEnvDecay;
	m_tablePresetParameters[ 15] = kFilterEnvSustain;
	m_tablePresetParameters[ 16] = kFilterEnvRelease;

	m_tablePresetParameters[ 17] = kAmpEnvDepth;
	m_tablePresetParameters[ 18] = kAmpKeyDepth;

	m_tablePresetParameters[ 19] = kAmpEnvAttack;
	m_tablePresetParameters[ 20] = kAmpEnvDecay;
	m_tablePresetParameters[ 21] = kAmpEnvSustain;
	m_tablePresetParameters[ 22] = kAmpEnvRelease;
	
	m_tablePresetParameters[ 23] = kDelayLeftLevel;
	m_tablePresetParameters[ 24] = kDelayLeftTime;
	m_tablePresetParameters[ 25] = kDelayLeftFeedback;
	m_tablePresetParameters[ 26] = kDelayRightLevel;
	m_tablePresetParameters[ 27] = kDelayRightTime;
	m_tablePresetParameters[ 28] = kDelayRightFeedback;

	m_tablePresetParameters[ 29] = kPotControllerAssign1;
	m_tablePresetParameters[ 30] = kPotControllerAssign2;
	m_tablePresetParameters[ 31] = kPotControllerAssign3;
}

void CMonoSynthEngine::InitializeMidiCtrlParameters(void)
{
	for (int i=0; i<kMaxMidiCtrlParameters ;i++)
		m_tableMidiCtrlParameters[i] = kParameterNone;
	
	// Modulation Wheel
	m_tableMidiCtrlParameters[  1] = kFilterEnvDepth;

	// Effect Control 1
	m_tableMidiCtrlParameters[ 12] = kDelayLeftTime;
	// Effect Control 2
	m_tableMidiCtrlParameters[ 13] = kDelayRightTime;
	// Undefined
	m_tableMidiCtrlParameters[ 14] = kDelayLeftLevel;
	// Undefined
	m_tableMidiCtrlParameters[ 15] = kDelayRightLevel;
	// General Purpose Controller #1
	m_tableMidiCtrlParameters[ 16] = kDelayLeftFeedback;
	// General Purpose Controller #2
	m_tableMidiCtrlParameters[ 17] = kDelayRightFeedback;

	// General Purpose Controller #3
	m_tableMidiCtrlParameters[ 18] = kFilterEnvAttack;
	// Undefined
	m_tableMidiCtrlParameters[ 19] = kFilterEnvDecay;
	// Undefined
	m_tableMidiCtrlParameters[ 20] = kFilterEnvSustain;
	// Undefined
	m_tableMidiCtrlParameters[ 21] = kFilterEnvRelease;
	// Undefined
	m_tableMidiCtrlParameters[ 22] = kFilterKeyDepth;

	// Undefined
	m_tableMidiCtrlParameters[ 24] = kAmpEnvAttack;
	// Undefined
	m_tableMidiCtrlParameters[ 25] = kAmpEnvDecay;
	// Undefined
	m_tableMidiCtrlParameters[ 26] = kAmpEnvSustain;
	// Undefined
	m_tableMidiCtrlParameters[ 27] = kAmpEnvRelease;
	// Undefined
	m_tableMidiCtrlParameters[ 28] = kAmpEnvDepth;
	// Undefined
	m_tableMidiCtrlParameters[ 29] = kAmpKeyDepth;

	// Sound Controller #2 (Timbre)
	m_tableMidiCtrlParameters[ 71] = kFilterResonance;
	// Sound Controller #5 (Brightness)
	m_tableMidiCtrlParameters[ 74] = kFilterCutoff;
}

void CMonoSynthEngine::InitializePot2MidiCtrl(void)
{
	for (int i=0; i<kMaxPot2MidiCtrl ;i++)
	{
		m_tablePot2MidiCtrl[i].wID = kParameterNone;
	}

	m_tablePot2MidiCtrl[  0].wID = kPotController1;
	m_tablePot2MidiCtrl[  0].byMidiCtrl = 110;
	
	m_tablePot2MidiCtrl[  1].wID = kPotController2;
	m_tablePot2MidiCtrl[  1].byMidiCtrl = 111;

	m_tablePot2MidiCtrl[  2].wID = kPotController3;
	m_tablePot2MidiCtrl[  2].byMidiCtrl = 112;
}

void CMonoSynthEngine::UpdatePresetParameters(void)
{
	for (int i=0; i<kMaxPresetParameters ;i++)
	{
		if (m_tablePresetParameters[i] != kParameterNone)
			m_renderer.Update(m_tablePresetParameters[i]);
	}
}

BOOL CMonoSynthEngine::Initialize(void)
{
	if (!m_renderer.Initialize())
	{
		TRACE("ERROR: cannot initialize renderer in CMonoSynthEngine::Initialize.\n");
		return FALSE;
	}

	m_nPlayerRef = OpenPlayer("Player");
	if (m_nPlayerRef < 0)
	{
		TRACE("ERROR: cannot open Player in CMonoSynthEngine::Initialize\n");
		return FALSE;
	}

	MidiConnect(m_nPlayerRef, m_nMidiRef, TRUE);

	InitializeGlobalParameters();
	InitializePresetParameters();
	InitializeMidiCtrlParameters();
	InitializePot2MidiCtrl();

	// Check FLASH
	rtems_unsigned32 flash_size;

	m_nFlashRef = flash_init();
	if (!m_nFlashRef)
	{
		TRACE("ERROR: cannot open flash memory in CMonoSynthEngine::Initialize.\n");
		return FALSE;
	}

	if (!flash_get_size(m_nFlashRef, &flash_size))
	{
		TRACE("ERROR: cannot get flash size in CMonoSynthEngine::Initialize\n");
		return FALSE;
	}

	while (!LoadConfiguration())
	{
		TRACE("Invalid Bank data format stored in flash. Initializing...\n");
		
		SaveConfiguration();

		for (int i=0; i<128 ;i++)
			SavePreset(i, "No Name");
	}

	m_byPreset = 0;
	SetPresetModified(FALSE);
	m_bDemoPlaying = FALSE;

	m_idTimerConf = CreateTimer(rtems_build_name('T', 'C', 'O', 'N'), 500);
	
	m_renderer.EnableProcessing(TRUE);

	return TRUE;
}

void CMonoSynthEngine::OnMessage(CMessage *pMsg)
{
	ASSERT(pMsg);
	ASSERT(m_pPanel);

	switch (pMsg->dwMsg)
	{
		case kEngineMsgMidi:
		{
			CEngineMsg_Midi *msg = (CEngineMsg_Midi *) pMsg;
			CParameter *pMidiChan = GetParameterFromID(kGlobalMidiChannel);
			ASSERT(pMidiChan);
			
			switch (EvType(msg->ev))
			{
				case typeKeyOn:
					if (msg->ev->chan == pMidiChan->GetValue())
						m_renderer.DoMidiKeyOn(msg->ev->info.note.pitch, msg->ev->info.note.vel);
					break;

				case typeKeyOff:
					if (msg->ev->chan == pMidiChan->GetValue())
						m_renderer.DoMidiKeyOff(msg->ev->info.note.pitch, msg->ev->info.note.vel);
					break;
				
				case typePitchWheel:
					if (msg->ev->chan == pMidiChan->GetValue())
						m_renderer.DoMidiPitchWheel(((short) (msg->ev->info.data[1] << 7) | msg->ev->info.data[0]) - 0x2000);
					break;
				
				case typeCtrlChange:
					if ((msg->ev->chan == pMidiChan->GetValue()) &&
					    (m_tableMidiCtrlParameters[msg->ev->info.data[0]] != kParameterNone))
					{
						CParameter *pParameter = GetParameterFromID(m_tableMidiCtrlParameters[msg->ev->info.data[0]]);
						ASSERT(pParameter);
						pParameter->SetValueMidi(msg->ev->info.data[1]);
						m_renderer.Update(pParameter->GetID());
						PANEL(m_pPanel)->SendMsgParameterChanged(m_tableMidiCtrlParameters[msg->ev->info.data[0]]);
					}	
					break;
				
				case typeProgChange:
					if ((msg->ev->chan == pMidiChan->GetValue()) &&
					    (m_byPreset != msg->ev->info.data[0]))
					{
						LoadPreset(msg->ev->info.data[0], FALSE);
						PANEL(m_pPanel)->SendMsgPresetChanged(msg->ev->info.data[0]);
					}
					break;

				case typeSysEx:
					OnMidiSysEx(msg->ev);
					break;

				case typeTextual:
				case typeCopyright:
				case typeSeqName:
				case typeInstrName:
				case typeLyric:
				{
					TRACE("Midi Text: ");
					for (int i=0; i<MidiCountFields(msg->ev) ;i++)
						TRACE("%c", (char) MidiGetField(msg->ev, i));

					TRACE("\n");
					break;
				}
				default:
					TRACE("Unprocessed MIDI event: type %d\n", EvType(msg->ev));
			}
			
			CParameter *pParameter = GetParameterFromID(kGlobalMidiThru);
			ASSERT(pParameter);
			
			if (pParameter->GetValue() > 0)
				MidiSend(m_nMidiRef, msg->ev);
			else
				MidiFreeEv(msg->ev);
			break;
		}
		case kEngineMsgParameterChanged:
		{
			CEngineMsg_ParameterChanged *msg = (CEngineMsg_ParameterChanged *) pMsg;
			m_renderer.Update(msg->wID);
			
			CParameter *pParameter = NULL;
			BYTE byMidiCtrl = 0;

			for (int i=0; i<kMaxPot2MidiCtrl ;i++)
			{
				if (m_tablePot2MidiCtrl[i].wID == msg->wID)
				{
					pParameter = GetParameterFromID(msg->wID);
					byMidiCtrl = m_tablePot2MidiCtrl[i].byMidiCtrl;
					break;
				};
			}
			
			if (pParameter)
			{
				CParameter *pMidiChan = GetParameterFromID(kGlobalMidiChannel);
				ASSERT(pMidiChan);
				MidiEvPtr ev = MidiNewEv(typeCtrlChange);
				ASSERT(ev);
				ev->chan = pMidiChan->GetValue();
				ev->info.data[0] = byMidiCtrl;
				ev->info.data[1] = pParameter->GetValue();
				MidiSendIm(m_nMidiRef, ev);
			}
			
			break;
		}
		case kEngineMsgTimer:
		{
			CEngineMsg_Timer *msg = (CEngineMsg_Timer *) pMsg;

			TRACE("Engine timer received\n");
			
			if (msg->idTimer == m_idTimerConf)
				SaveConfiguration();
			break;
		}
		default:
			TRACE("ERROR: Invalid Message received in CMonoSynthEngine::OnMessage (%ld)\n", pMsg->dwMsg);
			ASSERT(FALSE);
	}
}

//////////////////////////////////////////////////////////////////////////////

void CMonoSynthEngine::SendMsgParameterChanged(WORD wID)
{
	CEngineMsg_ParameterChanged msg;
	
	msg.wID = wID;
	SendMessage(&msg);

	BOOL bModified = TRUE;

	if ((wID == kPotMasterVolume) ||
	    (wID == kPotController1) ||
	    (wID == kPotController2) ||
	    (wID == kPotController3))
	{
		bModified = FALSE;
	}
	else
	{
		// Look for global configuration changes
		for (int i=0; i<kMaxGlobalParameters ;i++)
		{
			if (m_tableGlobalParameters[i] == wID)
			{
				ResetTimer(m_idTimerConf);
				bModified = FALSE;
				break;
			}
		}
	}
	
	if (bModified)
		SetPresetModified(TRUE);
}

//////////////////////////////////////////////////////////////////////////////

void CMonoSynthEngine::GetPresetName(BYTE byPreset, char *strPresetName)
{
	ASSERT(strPresetName);

	DWORD dwOffset = sizeof(CConfiguration) + byPreset*sizeof(CPreset);

	if (!flash_read_data(m_nFlashRef, dwOffset, (BYTE *) strPresetName, kMaxPresetNameLen))
	{
		TRACE("ERROR: cannot read from flash memory in CMonoSynthEngine::SetCurrentPreset\n");
		return;
	}

	strPresetName[kMaxPresetNameLen] = '\0';
}

void CMonoSynthEngine::LoadPreset(BYTE byPreset, BOOL bSendMidiPrgChange = TRUE)
{
	DWORD dwOffset = sizeof(CConfiguration) + byPreset*sizeof(CPreset);
	CPreset preset;
	CParameter *pParameter;

	TRACE("Loading Preset %03d\n", byPreset+1);
	
	if (!flash_read_data(m_nFlashRef, dwOffset, (BYTE *) &preset, sizeof(CPreset)))
	{
		TRACE("ERROR: cannot read from flash memory in CMonoSynthEngine::SetCurrentPreset\n");
		return;
	}

	if (preset.byVersion == MONOSYNTH_PRESET_VERSION)
	{
		for (int i=0; i<kMaxPresetParameters ;i++)
		{
			if (m_tablePresetParameters[i] != kParameterNone)
			{
				pParameter = GetParameterFromID(m_tablePresetParameters[i]);
				ASSERT(pParameter);
				pParameter->SetValue(preset.byData[i]);
				PANEL(m_pPanel)->SendMsgParameterChanged(m_tablePresetParameters[i]);
			}
		}
	
		UpdatePresetParameters();

		m_byPreset = byPreset;
		SetPresetModified(FALSE);

		if (bSendMidiPrgChange)
		{
			MidiEvPtr ev = MidiNewEv(typeProgChange);
			ASSERT(ev);
			ev->chan = 0;
			ev->info.data[0] = m_byPreset;
			MidiSendIm(m_nMidiRef, ev);
		}
	}
}

void CMonoSynthEngine::SavePreset(BYTE byPreset, char *strPresetName)
{
	ASSERT(strPresetName);

	DWORD dwOffset = sizeof(CConfiguration) + byPreset*sizeof(CPreset);
	CPreset preset;
	CParameter *pParameter;
	
	TRACE("Saving Preset %03d\n", byPreset+1);

	for (int i=0; i<kMaxPresetParameters ;i++)
	{
		if (m_tablePresetParameters[i] != kParameterNone)
		{
			pParameter = GetParameterFromID(m_tablePresetParameters[i]);
			ASSERT(pParameter);
			preset.byData[i] = pParameter->GetValue();
		}
		else
			preset.byData[i] = 0;
	}

	preset.byVersion = MONOSYNTH_PRESET_VERSION;
	memset(preset.strName, ' ', kMaxPresetNameLen);
	strncpy(preset.strName, strPresetName, kMaxPresetNameLen);
	
	if (!flash_write_data(m_nFlashRef, dwOffset, (BYTE*) &preset, sizeof(CPreset)))
	{
		TRACE("ERROR: cannot write to flash memory in CMonoSynthEngine::SavePreset\n");
		return;
	}
}

void CMonoSynthEngine::DumpBank(void)
{
	OnStoredBankRequest();
}

void CMonoSynthEngine::DumpPreset(void)
{
	OnEditedPresetRequest();
}

//////////////////////////////////////////////////////////////////////////////

BOOL CMonoSynthEngine::LoadConfiguration(void)
{
	CConfiguration config;
	CParameter *pParameter;

	TRACE("Loading Configuration Data...\n");

	if (!flash_read_data(m_nFlashRef, 0, (BYTE*) &config, sizeof(CConfiguration)))
	{
		TRACE("ERROR: cannot read from flash memory in CMonoSynthEngine::LoadConfiguration\n");
		return FALSE;
	}	

	if (memcmp(config.strMagic, MONOSYNTH_CONFIG_MAGIC, strlen(MONOSYNTH_CONFIG_MAGIC)) != 0)
		return FALSE;

	if (config.wVersion != MONOSYNTH_CONFIG_VERSION)
		return FALSE;

	for (int i=0; i<kMaxGlobalParameters ;i++)
	{
		if (m_tableGlobalParameters[i] != kParameterNone)
		{
			pParameter = GetParameterFromID(m_tableGlobalParameters[i]);
			ASSERT(pParameter);
			pParameter->SetValue(config.byData[i]);
			m_renderer.Update(m_tableGlobalParameters[i]);
		}
	}

	return TRUE;
}

void CMonoSynthEngine::SaveConfiguration(void)
{
	CConfiguration config;
	CParameter *pParameter;

	TRACE("Saving Configuration Data...\n");

	for (int i=0; i<kMaxGlobalParameters ;i++)
	{
		if (m_tableGlobalParameters[i] != kParameterNone)
		{
			pParameter = GetParameterFromID(m_tableGlobalParameters[i]);
			ASSERT(pParameter);
			config.byData[i] = pParameter->GetValue();
		}
		else
			config.byData[i] = 0;
	}

	memcpy(config.strMagic, MONOSYNTH_CONFIG_MAGIC, strlen(MONOSYNTH_CONFIG_MAGIC));
	config.wVersion = MONOSYNTH_CONFIG_VERSION;

	if (!flash_write_data(m_nFlashRef, 0, (BYTE*) &config, sizeof(CConfiguration)))
	{
		TRACE("ERROR: cannot write to flash memory in CMonoSynthEngine::SaveConfiguration\n");
		return;
	}
}

/*******************************************************************************

	MonoSynth SysEx encoding
	------------------------
	
	0xF0		Start of SysEx
	0x00 0x20 0x45	SoundArt MIDI ID
	0x01		Product ID (Chameleon = 0x01)
	xxxx		Device ID (0x00..0x7E, 0x7F = broadcast)
	0x00 0x02	Application ID (0x00 0x02 = MonoSynth)
	xxxx		Message ID
	....		Data (size dependent of the message ID)
	xxxx		Checksum (sum of all databytes truncated to 7 bits, 0x7F is always valid)
	0xF7		End of SysEx
	

	Message: Stored Preset Dump
	--------------------
	0x00		Message ID (0x00 = Stored Preset Dump)
	xxxx		Preset number
	....		Dump data (kMaxPresetParameters bytes)
	
	Message: Edited Preset Dump
	--------------------
	0x01		Message ID (0x01 = Edited Preset Dump)
	....		Dump data (kMaxPresetParameters bytes)
	
	Message: Stored Preset Request
	-----------------------
	0x02		Message ID (0x02 = Stored Preset Request)
	xxxx		Preset number
	
	Message: Edited Preset Request
	-----------------------
	0x03		Message ID (0x03 = Edited Preset Request)
	
	Message: Stored Bank Request
	---------------------
	0x04		Message ID (0x04 = Stored Bank Request)

 ******************************************************************************/

void CMonoSynthEngine::OnMidiSysEx(MidiEvPtr ev)
{
	CParameter *pMidiDeviceID = GetParameterFromID(kGlobalMidiDeviceID);
	ASSERT(pMidiDeviceID);

	// Check for minimum size of SoundArt sysex message
	if (MidiCountFields(ev) < 9)
		return;

	// Check for SoundArt MIDI ID
	if ((MidiGetField(ev, 0) != 0x00) ||
	    (MidiGetField(ev, 1) != 0x20) ||
	    (MidiGetField(ev, 2) != 0x45))
		return;

	// Check for Chameleon Product ID
	if (MidiGetField(ev, 3) != 0x01)
		return;

	// Check for proper Device ID or broadcast
	if ((MidiGetField(ev, 4) != 0x7F) &&
	    (MidiGetField(ev, 4) != pMidiDeviceID->GetValue()))
		return;

	// Check for MonoSynth Application ID
	if ((MidiGetField(ev, 5) != 0x00) ||
	    (MidiGetField(ev, 6) != 0x02))
		return;

	// Check for valid checksum
	if (MidiGetField(ev, MidiCountFields(ev) - 1) != 0x7F)
	{
		int i;
		BYTE byChecksum = 0;

		for (i=0; i<(MidiCountFields(ev)-1) ;i++)
			byChecksum += MidiGetField(ev, i);

		byChecksum &= 0x7F;

		if (byChecksum != MidiGetField(ev, MidiCountFields(ev) - 1))
			return;
	}

	// Message ID
	switch (MidiGetField(ev, 7))
	{
		// Stored Preset dump
		case 0x00:
			if (MidiCountFields(ev) == (9 + 1 + sizeof(CPreset)))
				OnStoredPresetDump(MidiGetField(ev, 8), ev);
			break;

		// Edited Preset dump
		case 0x01:
			if (MidiCountFields(ev) == (8 + 1 + sizeof(CPreset)))
				OnEditedPresetDump(ev);
			break;

		// Stored Preset request
		case 0x02:
			if (MidiCountFields(ev) == (9 + 1))
				OnStoredPresetRequest(MidiGetField(ev, 8));
			break;

		// Edited Preset request
		case 0x03:
			if (MidiCountFields(ev) == (8 + 1))
				OnEditedPresetRequest();
			break;

		// Bank request
		case 0x04:
			if (MidiCountFields(ev) == 9)
				OnStoredBankRequest();
			break;

		default:
			return;
	}
}

void CMonoSynthEngine::OnStoredPresetDump(BYTE byPreset, MidiEvPtr ev)
{
	CPreset preset;
	BYTE *pPreset = (BYTE *) &preset;
	DWORD dwOffset = sizeof(CConfiguration) + byPreset*sizeof(CPreset);	
	DWORD i;

	ASSERT(sizeof(CPreset) == MidiCountFields(ev) - (9 + 1));

	TRACE("Receiving a stored sysex preset dump (preset %d)\n", byPreset+1);

	for (i=0 ;i<sizeof(CPreset) ;i++)
		pPreset[i] = MidiGetField(ev, 9 + i);

	if (preset.byVersion == MONOSYNTH_PRESET_VERSION)
	{
		if (!flash_write_data(
			m_nFlashRef,
			dwOffset,
			pPreset,
			sizeof(CPreset)))
		{
			TRACE("ERROR: cannot write to flash memory.\n");
			return;
		}
	
		if (byPreset == m_byPreset)
			LoadPreset(byPreset, FALSE);
	
		PANEL(m_pPanel)->Refresh();
	}
	else
		TRACE("Invalid version in sysex preset received\n");
}

void CMonoSynthEngine::OnEditedPresetDump(MidiEvPtr ev)
{
	CParameter *pParameter;
	CPreset preset;
	BYTE *pPreset = (BYTE *) &preset;
	DWORD i;

	ASSERT(sizeof(CPreset) == MidiCountFields(ev) - (8 + 1));

	TRACE("Receiving a sysex edited preset dump\n");

	for (i=0 ;i<sizeof(CPreset) ;i++)
		pPreset[i] = MidiGetField(ev, 8 + i);

	if (preset.byVersion == MONOSYNTH_PRESET_VERSION)
	{
		for (int i=0; i<kMaxPresetParameters ;i++)
		{
			if (m_tablePresetParameters[i] != kParameterNone)
			{
				pParameter = GetParameterFromID(m_tablePresetParameters[i]);
				ASSERT(pParameter);
				pParameter->SetValue(preset.byData[i]);
				PANEL(m_pPanel)->SendMsgParameterChanged(m_tablePresetParameters[i]);
			}
		}
	
		UpdatePresetParameters();
		PANEL(m_pPanel)->Refresh();
	}
	else
		TRACE("Invalid version in sysex preset received\n");
}

void CMonoSynthEngine::OnStoredPresetRequest(BYTE byPreset)
{
	CPreset preset;
	BYTE *pPreset = (BYTE *) &preset;
	DWORD dwOffset = sizeof(CConfiguration) + byPreset*sizeof(CPreset);	
	CParameter *pMidiDeviceID = GetParameterFromID(kGlobalMidiDeviceID);
	MidiEvPtr ev;
	DWORD i;
	
	ASSERT(pMidiDeviceID);

	if (!flash_read_data(
		m_nFlashRef,
		dwOffset,
		pPreset,
		sizeof(CPreset)))
	{
		TRACE("ERROR: cannot read from flash memory.\n");
		return;
	}

	ev = MidiNewEv(typeSysEx);
	
	// SoundArt MIDI ID
	MidiAddField(ev, (long) 0x00);
	MidiAddField(ev, (long) 0x20);
	MidiAddField(ev, (long) 0x45);
	
	// Chameleon Product ID
	MidiAddField(ev, (long) 0x01);

	// Device ID
	MidiAddField(ev, (long) pMidiDeviceID->GetValue());

	// MonoSynth Application ID
	MidiAddField(ev, (long) 0x00);
	MidiAddField(ev, (long) 0x02);

	// Message ID: Stored Preset Dump
	MidiAddField(ev, (long) 0x00);
	MidiAddField(ev, (long) byPreset);

	// Preset Data
	for (i=0 ;i<sizeof(CPreset) ;i++)
		MidiAddField(ev, ((long) pPreset[i]) & 0x7F);

	// Checksum
	BYTE byChecksum = 0;
	
	for (int i=0; i<MidiCountFields(ev) ;i++)
		byChecksum += MidiGetField(ev, i);
	
	MidiAddField(ev, (long) byChecksum & 0x7F);

	MidiSendIm(m_nMidiRef, ev);
}

void CMonoSynthEngine::OnEditedPresetRequest(void)
{
	CParameter *pParameter;
	CPreset preset;
	BYTE *pPreset = (BYTE *) &preset;
	CParameter *pMidiDeviceID = GetParameterFromID(kGlobalMidiDeviceID);
	MidiEvPtr ev;
	DWORD i;
	
	ASSERT(pMidiDeviceID);

	ev = MidiNewEv(typeSysEx);
	
	// SoundArt MIDI ID
	MidiAddField(ev, (long) 0x00);
	MidiAddField(ev, (long) 0x20);
	MidiAddField(ev, (long) 0x45);
	
	// Chameleon Product ID
	MidiAddField(ev, (long) 0x01);

	// Device ID
	MidiAddField(ev, (long) pMidiDeviceID->GetValue());

	// MonoSynth Application ID
	MidiAddField(ev, (long) 0x00);
	MidiAddField(ev, (long) 0x02);

	// Message ID: Edited Preset Dump
	MidiAddField(ev, (long) 0x01);

	for (int i=0; i<kMaxPresetParameters ;i++)
	{
		if (m_tablePresetParameters[i] != kParameterNone)
		{
			pParameter = GetParameterFromID(m_tablePresetParameters[i]);
			ASSERT(pParameter);
			preset.byData[i] = pParameter->GetValue();
		}
		else
			preset.byData[i] = 0;
	}
	
	GetPresetName(GetCurrentPreset(), preset.strName);
	preset.byVersion = MONOSYNTH_PRESET_VERSION;

	// Preset Data
	for (i=0 ;i<sizeof(CPreset) ;i++)
		MidiAddField(ev, ((long) pPreset[i]) & 0x7F);

	// Checksum
	BYTE byChecksum = 0;
	
	for (int i=0; i<MidiCountFields(ev) ;i++)
		byChecksum += MidiGetField(ev, i);
	
	MidiAddField(ev, (long) byChecksum & 0x7F);

	MidiSendIm(m_nMidiRef, ev);
}

void CMonoSynthEngine::OnStoredBankRequest(void)
{
	for (int i=0; i<0x80 ;i++)
		OnStoredPresetRequest(i);
}

//////////////////////////////////////////////////////////////////////////////

void CMonoSynthEngine::PlayMidiFile(const BYTE *pData, DWORD dwSize)
{
	MidiFileInfos info;
	PlayerState state;
	MidiSeqPtr seq = MidiNewSeq();
	ASSERT(seq);

	m_bDemoPlaying = TRUE;
	MidiFileLoadFromMemory(pData, dwSize, seq, &info);
	SetAllTrackPlayer(m_nPlayerRef, seq, info.clicks);
	GetEndScorePlayer(m_nPlayerRef, &state);
	SetLoopStartMsPlayer(m_nPlayerRef, 0);
	SetLoopEndMsPlayer(m_nPlayerRef, state.date);
	SetLoopPlayer(m_nPlayerRef, kLoopOn);
	StartPlayer(m_nPlayerRef);
}

void CMonoSynthEngine::StopMidiFile(void)
{
	StopPlayer(m_nPlayerRef);
	m_bDemoPlaying = FALSE;
}
