/*******************************************************************************
 * C H A M E L E O N                                                           *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/MONOSYNTH/MonoSynthEngine.h         $
 *     $Date:: 13/05/02 20:11                                                  $
 * $Revision:: 1                                                               $
 *-----------------------------------------------------------------------------*
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#ifndef __MONOSYNTH_ENGINE_H__
#define __MONOSYNTH_ENGINE_H__

#include "engine.h"
#include "MonoSynthRenderer.h"

////////////////////////////////////////////////////////////////////////////////
#define ENGINE(x)	((CMonoSynthEngine *) x)
//////////////////////////////////////////////////////////////////////////////
#define kMaxGlobalParameters		16
#define kMaxPresetParameters		128
#define kMaxMidiCtrlParameters		128
#define kMaxPot2MidiCtrl		3
#define kMaxPresetNameLen		14
//////////////////////////////////////////////////////////////////////////////

class CMonoSynthEngine : public CEngine
{
private:
	int m_nFlashRef;
	short m_nPlayerRef;
	rtems_id m_idTimerConf;
	
	CMonoSynthRenderer m_renderer;

	BOOL m_bDemoPlaying;
	BOOL m_bPresetModified;
	BYTE m_byPreset;

private:
	WORD m_tableGlobalParameters[kMaxGlobalParameters];
	WORD m_tablePresetParameters[kMaxPresetParameters];
	WORD m_tableMidiCtrlParameters[kMaxMidiCtrlParameters];
	
	struct CPot2MidiCtrl
	{
		WORD	wID;
		BYTE	byMidiCtrl;
	};

	CPot2MidiCtrl m_tablePot2MidiCtrl[kMaxPot2MidiCtrl];

private:
	void InitializeGlobalParameters(void);
	void InitializePresetParameters(void);
	void InitializeMidiCtrlParameters(void);
	void InitializePot2MidiCtrl(void);

	void UpdatePresetParameters(void);

private:
	BOOL LoadConfiguration(void);
	void SaveConfiguration(void);

	void OnMidiKeyOn(MidiEvPtr ev);
	void OnMidiKeyOff(MidiEvPtr ev);
	void OnMidiSysEx(MidiEvPtr ev);

	void OnStoredPresetDump(BYTE byPreset, MidiEvPtr ev);
	void OnEditedPresetDump(MidiEvPtr ev);
	void OnStoredPresetRequest(BYTE byPreset);
	void OnEditedPresetRequest(void);
	void OnStoredBankRequest(void);

protected:
	virtual BOOL Initialize(void);
	virtual void OnMessage(CMessage *pMsg);

public:
	BYTE GetCurrentPreset(void)
	{
		return m_byPreset;
	}
	
	void GetPresetName(BYTE byPreset, char *strPresetName);

	BOOL IsDemoPlaying(void)
	{
		return m_bDemoPlaying;
	}
	
	BOOL IsPresetModified(void)
	{
		return m_bPresetModified;
	}
	
	void SetPresetModified(BOOL bModified = TRUE)
	{
		m_bPresetModified = bModified;
	}
	
	void LoadPreset(BYTE byPreset, BOOL bSendMidiPrgChange = TRUE);
	void SavePreset(BYTE byPresetNumber, char *strPresetName);
	
	void PlayMidiFile(const BYTE *pData, DWORD dwSize);
	void StopMidiFile(void);
	
	void DumpBank(void);
	void DumpPreset(void);
	
public:
	CMonoSynthEngine(void) : CEngine() { }
	virtual ~CMonoSynthEngine(void) { }
	
	void Task(void);
	
public:
	void SendMsgParameterChanged(WORD wID);
};

//////////////////////////////////////////////////////////////////////////////

#endif
