/*******************************************************************************
 * C H A M E L E O N                                                           *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/MONOSYNTH/MonoSynthPanel.cpp        $
 *     $Date:: 13/05/02 20:11                                                  $
 * $Revision:: 1                                                               $
 *-----------------------------------------------------------------------------*
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#include <rtems.h>
#include <chameleon.h>

#include "MonoSynthEngine.h"
#include "MonoSynthPanel.h"
#include "MonoSynthParameters.h"

//////////////////////////////////////////////////////////////////////////////
enum
{
	kPanelMsgChangeMode = kPanelMsgUser,
	kPanelMsgParameterChanged,
	kPanelMsgPresetChanged,
};
//////////////////////////////////////////////////////////////////////////////

struct CPanelMsg_ChangeMode : public CMessage
{
	int nMode;

	CPanelMsg_ChangeMode(void)
	{
		ASSERT(GetSize() <= PANEL_MAX_MESSAGE_SIZE);
		dwMsg = kPanelMsgChangeMode;
	}

	virtual const DWORD GetSize(void) { return sizeof(CPanelMsg_ChangeMode); }
};

struct CPanelMsg_ParameterChanged : public CMessage
{
	WORD wID;

	CPanelMsg_ParameterChanged(void)
	{
		ASSERT(GetSize() <= PANEL_MAX_MESSAGE_SIZE);
		dwMsg = kPanelMsgParameterChanged;
	}

	virtual const DWORD GetSize(void) { return sizeof(CPanelMsg_ParameterChanged); }
};

struct CPanelMsg_PresetChanged : public CMessage
{
	BYTE byPreset;

	CPanelMsg_PresetChanged(void)
	{
		ASSERT(GetSize() <= PANEL_MAX_MESSAGE_SIZE);
		dwMsg = kPanelMsgPresetChanged;
	}

	virtual const DWORD GetSize(void) { return sizeof(CPanelMsg_PresetChanged); }
};

////////////////////////////////////////////////////////////////////////////////
static const BYTE m_pLcdSymbolRightArrow[8] =
{
	0x10,	/* 00010000 */
	0x18,	/* 00011000 */
	0x1C,	/* 00011100 */
	0x1E,	/* 00011110 */
	0x1C,	/* 00011100 */
	0x18,	/* 00011000 */
	0x10,	/* 00010000 */
	0x00	/* 00000000 */
};
static const BYTE m_pLcdSymbolLeftArrow[8] =
{
	0x02,	/* 00000010 */
	0x06,	/* 00000110 */
	0x0E,	/* 00001110 */
	0x1E,	/* 00011110 */
	0x0E,	/* 00001110 */
	0x06,	/* 00000110 */
	0x02,	/* 00000010 */
	0x00	/* 00000000 */
};
//////////////////////////////////////////////////////////////////////////////

BOOL CMonoSynthPanel::Initialize(void)
{
	rtems_unsigned8 model;

	if (!info_get_model(&model))
		return FALSE;

	ASSERT(model == 0x01);

	PanelLcdRedefine(1, m_pLcdSymbolRightArrow);
	PanelLcdRedefine(2, m_pLcdSymbolLeftArrow);

	if (!m_panelModeInit.Initialize(this))
		return FALSE;

	m_pPanelModeHandler = &m_panelModeInit;
	m_pPanelModeHandler->BeginMode(kPanelModeInvalid);
	m_nPanelMode = kPanelModeInit;
	m_pPanelModeHandler->Display();
	
	if (!m_pEngine->Create(this))
		return FALSE;	

	if (!m_panelModePreset.Initialize(this) ||
	    !m_panelModeEdit.Initialize(this) ||
	    !m_panelModeSave.Initialize(this) ||
	    !m_panelModeAssign.Initialize(this) ||
	    !m_panelModeMidi.Initialize(this))
		return FALSE;
	
	return TRUE;
}

void CMonoSynthPanel::OnMessage(CMessage *pMsg)
{
	ASSERT(m_pPanelModeHandler);
	ASSERT(pMsg);
	
	switch (pMsg->dwMsg)
	{
		case kPanelMsgPotentiometer:
		{
			CPanelMsg_Potentiometer *msg = (CPanelMsg_Potentiometer *) pMsg;
			CParameter *pParameter = NULL;
			
			if (msg->byPotentiometer == PANEL01_POT_VOLUME)
			{
				pParameter = GetParameterFromID(kPotMasterVolume);
				ASSERT(pParameter);
				pParameter->SetValueMidi(msg->byValue);
				ENGINE(GetEngine())->SendMsgParameterChanged(kPotMasterVolume);
				break;
			}

			if (msg->byPotentiometer == PANEL01_POT_CTRL1)
				pParameter = GetParameterFromID(kPotController1);
			else if (msg->byPotentiometer == PANEL01_POT_CTRL2)
				pParameter = GetParameterFromID(kPotController2);
			else if (msg->byPotentiometer == PANEL01_POT_CTRL3)
				pParameter = GetParameterFromID(kPotController3);

			if (pParameter)
			{
				ASSERT(pParameter);
				pParameter->SetValueMidi(msg->byValue);
				ENGINE(GetEngine())->SendMsgParameterChanged(pParameter->GetID());
			}

			m_pPanelModeHandler->OnPotentiometer(msg->byPotentiometer, msg->byValue);

			break;
		}
		case kPanelMsgEncoder:
		{
			CPanelMsg_Encoder *msg = (CPanelMsg_Encoder *) pMsg;		
			m_pPanelModeHandler->OnEncoder(msg->byEncoder, msg->nIncrement);
			break;
		}
		case kPanelMsgKeypad:
		{
			CPanelMsg_Keypad *msg = (CPanelMsg_Keypad *) pMsg;
			m_pPanelModeHandler->OnKeypad(msg->dwKeyBits);
			break;
		}
		case kPanelMsgTimer:
		{
			CPanelMsg_Timer *msg = (CPanelMsg_Timer *) pMsg;
			m_pPanelModeHandler->OnTimer(msg->idTimer);
			break;
		}
		case kPanelMsgParameterChanged:
		{
			CPanelMsg_ParameterChanged *msg = (CPanelMsg_ParameterChanged *) pMsg;
			CParameter *pParameter = NULL;
			DWORD dwLed = 0;
			
			if (msg->wID == kPotControllerAssign1)
				dwLed = PANEL01_LED_CTRL1;
			else if (msg->wID == kPotControllerAssign2)
				dwLed = PANEL01_LED_CTRL2;
			else if (msg->wID == kPotControllerAssign3)
				dwLed = PANEL01_LED_CTRL3;

			if (dwLed)
			{
				pParameter = GetParameterFromID(msg->wID);
				
				if (pParameter && (pParameter->GetValue() != kParameterNone))
					PanelSetLeds(PanelGetLeds() | dwLed);
				else
					PanelSetLeds(PanelGetLeds() & (~dwLed));
			}

			m_pPanelModeHandler->OnParameterChanged(msg->wID);
			break;
		}
		case kPanelMsgPresetChanged:
		{
			CPanelMsg_PresetChanged *msg = (CPanelMsg_PresetChanged *) pMsg;
			m_pPanelModeHandler->OnPresetChanged(msg->byPreset);
			break;
		}
		case kPanelMsgChangeMode:
		{
			CPanelMsg_ChangeMode *msg = (CPanelMsg_ChangeMode *) pMsg;

			m_pPanelModeHandler->EndMode();

			switch (msg->nMode)
			{
				case kPanelModeInit:
					m_pPanelModeHandler = &m_panelModeInit;
					break;

				case kPanelModePreset:
					m_pPanelModeHandler = &m_panelModePreset;
					break;
					
				case kPanelModeEdit:
					m_pPanelModeHandler = &m_panelModeEdit;
					break;
					
				case kPanelModeSave:
					m_pPanelModeHandler = &m_panelModeSave;
					break;
					
				case kPanelModeAssign:
					m_pPanelModeHandler = &m_panelModeAssign;
					break;
					
				case kPanelModeMidi:
					m_pPanelModeHandler = &m_panelModeMidi;
					break;
					
				default:
					TRACE("ERROR: Invalid mode to change received in CMonoSynthPanel::OnMessage (%d)\n", msg->nMode);
					ASSERT(FALSE);
			}

			m_pPanelModeHandler->BeginMode(m_nPanelMode);
			m_nPanelMode = msg->nMode;
			m_pPanelModeHandler->Display();
			break;
		}
		default:
			TRACE("ERROR: Invalid Message received in CMonoSynthPanel::OnMessage (%ld)\n", pMsg->dwMsg);
			ASSERT(FALSE);
	}
}

void CMonoSynthPanel::SendMsgChangeMode(int nMode)
{
	CPanelMsg_ChangeMode msg;

	msg.nMode = nMode;
	SendMessage(&msg);
}

void CMonoSynthPanel::SendMsgParameterChanged(WORD wID)
{
	CPanelMsg_ParameterChanged msg;

	msg.wID = wID;
	SendMessage(&msg);
}

void CMonoSynthPanel::SendMsgPresetChanged(BYTE byPreset)
{
	CPanelMsg_PresetChanged msg;

	msg.byPreset = byPreset;
	SendMessage(&msg);
}

void CMonoSynthPanel::Refresh(void)
{
	ASSERT(m_pPanelModeHandler);
	m_pPanelModeHandler->Display();
}
