/*******************************************************************************
 * C H A M E L E O N                                                           *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/MONOSYNTH/MonoSynthPanelModeAssign. $
 *     $Date:: 13/05/02 20:11                                                  $
 * $Revision:: 1                                                               $
 *-----------------------------------------------------------------------------*
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#include <rtems.h>
#include <chameleon.h>

#include "MonoSynthEngine.h"
#include "MonoSynthPanel.h"
#include "MonoSynthPanelModeAssign.h"
#include "MonoSynthParameters.h"

BOOL CMonoSynthPanelModeAssign::Initialize(CPanel *pPanel)
{
	if (!CPanelModeHandler::Initialize(pPanel))
		return FALSE;

	m_idTimer = m_pPanel->CreateTimer(rtems_build_name('T', 'A', 'S', 'S'), 650);

	return TRUE;
}

void CMonoSynthPanelModeAssign::BeginMode(int nModeBefore)
{
	ASSERT(m_pPanel);
	TRACE("Assign Mode\n");

	m_nModeBefore = nModeBefore;
	m_wSelectedID = kParameterNone;

	m_pPanel->ResetTimer(m_idTimer);
}

void CMonoSynthPanelModeAssign::EndMode(void)
{
	ASSERT(m_pPanel);

	m_pPanel->CancelTimer(m_idTimer);
}

void CMonoSynthPanelModeAssign::OnPotentiometer(BYTE byPot, BYTE byValue)
{
	ASSERT(m_pPanel);

	CParameter *pCtrl = NULL;
	CParameter *pParameter = NULL;
	
	m_wSelectedID = kParameterNone;

	switch (byPot)
	{
		case PANEL01_POT_CTRL1:
			pCtrl = GetParameterFromID(kPotControllerAssign1);
			m_wSelectedID = kPotControllerAssign1;
			break;

		case PANEL01_POT_CTRL2:
			pCtrl = GetParameterFromID(kPotControllerAssign2);
			m_wSelectedID = kPotControllerAssign2;
			break;

		case PANEL01_POT_CTRL3:
			pCtrl = GetParameterFromID(kPotControllerAssign3);
			m_wSelectedID = kPotControllerAssign3;
			break;
	}

	if (pCtrl && (pCtrl->GetValue() != kParameterNone))
	{
		pParameter = GetParameterFromID(pCtrl->GetValue());
		if (pParameter)
		{
			m_pPanel->CancelTimer(m_idTimer);
			pParameter->SetValueMidi(byValue);
			ENGINE(m_pPanel->GetEngine())->SendMsgParameterChanged(pParameter->GetID());
			m_pPanel->ResetTimer(m_idTimer);
			Display();
		}
	}
	else
		PANEL(m_pPanel)->SendMsgChangeMode(m_nModeBefore);
}

void CMonoSynthPanelModeAssign::OnEncoder(BYTE byEncoder, char nInc)
{
	ASSERT(m_pPanel);

	PANEL(m_pPanel)->SendMsgChangeMode(m_nModeBefore);
}

void CMonoSynthPanelModeAssign::OnKeypad(DWORD dwKeyBits)
{
	ASSERT(m_pPanel);

	PANEL(m_pPanel)->SendMsgChangeMode(m_nModeBefore);
}

void CMonoSynthPanelModeAssign::OnTimer(rtems_id timer_id)
{
	ASSERT(m_pPanel);

	if (timer_id == m_idTimer)
	{
		PANEL(m_pPanel)->SendMsgChangeMode(m_nModeBefore);
	}
	else
		ASSERT(FALSE);
}

void CMonoSynthPanelModeAssign::OnParameterChanged(WORD wID)
{
	ASSERT(m_pPanel);

	if ((wID == kPotControllerAssign1) ||
	    (wID == kPotControllerAssign2) ||
	    (wID == kPotControllerAssign3))
	{
		m_wSelectedID = wID;
		Display();
	}
}

void CMonoSynthPanelModeAssign::OnPresetChanged(BYTE byPreset)
{
	ASSERT(m_pPanel);

	m_pPanel->PanelSetLeds(m_pPanel->PanelGetLeds() & ~(PANEL01_LED_EDIT));
	PANEL(m_pPanel)->SendMsgChangeMode(CMonoSynthPanel::kPanelModePreset);
}

void CMonoSynthPanelModeAssign::Display(void)
{
	ASSERT(m_pPanel);
	
	if (m_wSelectedID != kParameterNone)
	{
		CParameter *pCtrl = NULL;
		CParameter *pParameter = NULL;
		char strValueText[16+1];
		int i, n;
		
		pCtrl = GetParameterFromID(m_wSelectedID);
		ASSERT(pCtrl);

		if (pCtrl->GetValue() != kParameterNone)
		{
			pParameter = GetParameterFromID(pCtrl->GetValue());
			ASSERT(pParameter);
			
			strValueText[0] = 1;
			pParameter->GetValueText(strValueText+1);
			strValueText[strlen(strValueText)+1] = '\0';
			strValueText[strlen(strValueText)] = 2;

			sprintf(m_strLine0, "%-16s", "");
			sprintf(m_strLine1, "%-16s", "");
	
			n = strlen(pParameter->GetLongMenuName());
			for (i=0 ;i<n ;i++)
				m_strLine0[i+(16-n)/2] = pParameter->GetLongMenuName()[i];
	
			n = strlen(strValueText);
			for (i=0 ;i<n ;i++)
				m_strLine1[i+(16-n)/2] = strValueText[i];
	
			m_pPanel->PanelLcdPrint(0, 0, m_strLine0);
			m_pPanel->PanelLcdPrint(1, 0, m_strLine1);
		}
	}
}
