/*******************************************************************************
 * C H A M E L E O N                                                           *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/MONOSYNTH/MonoSynthPanelModeEdit.cp $
 *     $Date:: 13/05/02 20:11                                                  $
 * $Revision:: 1                                                               $
 *-----------------------------------------------------------------------------*
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#include <rtems.h>
#include <chameleon.h>

#include "MonoSynthEngine.h"
#include "MonoSynthPanel.h"
#include "MonoSynthPanelModeEdit.h"
#include "MonoSynthParameters.h"

void CMonoSynthPanelModeEdit::AddMenuParameter(CMenuItem *pMenu, WORD wID)
{
	ASSERT(m_pPanel);

	CParameter *pParameter = GetParameterFromID(wID);
	ASSERT(pParameter);
	pMenu->AddSubItem(wID, pParameter->GetShortMenuName());
	TRACE("Adding menu '%s' (0x%04x)\n", pParameter->GetShortMenuName(), wID);
}

BOOL CMonoSynthPanelModeEdit::Initialize(CPanel *pPanel)
{
	if (!CPanelModeHandler::Initialize(pPanel))
		return FALSE;

	// Create the menu structure
	CMenuItem *pItemGroup, *pItemPage;

	pItemGroup = m_menu.AddSubItem("OSCILLATOR:");
		pItemPage = pItemGroup->AddSubItem("Osc1");
			AddMenuParameter(pItemPage, kOsc1Waveform);
			AddMenuParameter(pItemPage, kOsc1Transpose);
		pItemPage = pItemGroup->AddSubItem("Osc2");
			AddMenuParameter(pItemPage, kOsc2Waveform);
			AddMenuParameter(pItemPage, kOsc2Transpose);
			AddMenuParameter(pItemPage, kOsc2Detune);
	pItemGroup = m_menu.AddSubItem("MIXER:");
		pItemPage = pItemGroup->AddSubItem("");
			AddMenuParameter(pItemPage, kOsc1Volume);
			AddMenuParameter(pItemPage, kOsc2Volume);
			AddMenuParameter(pItemPage, kMixNoiseVolume);
			AddMenuParameter(pItemPage, kMixInputVolume);
	pItemGroup = m_menu.AddSubItem("FILTER:");
		pItemPage = pItemGroup->AddSubItem("");
			AddMenuParameter(pItemPage, kFilterCutoff);
			AddMenuParameter(pItemPage, kFilterResonance);
			AddMenuParameter(pItemPage, kFilterEnvDepth);
			AddMenuParameter(pItemPage, kFilterKeyDepth);
	pItemGroup = m_menu.AddSubItem("ENV:");
		pItemPage = pItemGroup->AddSubItem("Filter");
			AddMenuParameter(pItemPage, kFilterEnvAttack);
			AddMenuParameter(pItemPage, kFilterEnvDecay);
			AddMenuParameter(pItemPage, kFilterEnvSustain);
			AddMenuParameter(pItemPage, kFilterEnvRelease);
		pItemPage = pItemGroup->AddSubItem("Amp");
			AddMenuParameter(pItemPage, kAmpEnvAttack);
			AddMenuParameter(pItemPage, kAmpEnvDecay);
			AddMenuParameter(pItemPage, kAmpEnvSustain);
			AddMenuParameter(pItemPage, kAmpEnvRelease);
	pItemGroup = m_menu.AddSubItem("AMP:");
		pItemPage = pItemGroup->AddSubItem("");
			AddMenuParameter(pItemPage, kAmpEnvDepth);
			AddMenuParameter(pItemPage, kAmpKeyDepth);
	pItemGroup = m_menu.AddSubItem("EFX:");
		pItemPage = pItemGroup->AddSubItem("Delay Left");
			AddMenuParameter(pItemPage, kDelayLeftLevel);
			AddMenuParameter(pItemPage, kDelayLeftTime);
			AddMenuParameter(pItemPage, kDelayLeftFeedback);
		pItemPage = pItemGroup->AddSubItem("Delay Right");
			AddMenuParameter(pItemPage, kDelayRightLevel);
			AddMenuParameter(pItemPage, kDelayRightTime);
			AddMenuParameter(pItemPage, kDelayRightFeedback);
	pItemGroup = m_menu.AddSubItem("GLOBAL:");
		pItemPage = pItemGroup->AddSubItem("");
			AddMenuParameter(pItemPage, kGlobalInputThru);
			AddMenuParameter(pItemPage, kGlobalMidiChannel);
			AddMenuParameter(pItemPage, kGlobalMidiThru);
			AddMenuParameter(pItemPage, kGlobalMidiDeviceID);

	m_idTimer0 = m_pPanel->CreateTimer(rtems_build_name('T', 'E', 'D', '0'), 500);
	m_idTimer1 = m_pPanel->CreateTimer(rtems_build_name('T', 'E', 'D', '1'), 500);
	m_idTimer2 = m_pPanel->CreateTimer(rtems_build_name('T', 'E', 'D', '1'), 1500);

	return TRUE;
}

void CMonoSynthPanelModeEdit::BeginMode(int nModeBefore)
{
	ASSERT(m_pPanel);

	TRACE("Edit Mode\n");
	
	CMenuItem *pItemGroup, *pItemPage, *pItemParam;

	pItemGroup = m_menu.GetSelectedItem();
	ASSERT(pItemGroup);
	pItemPage = pItemGroup->GetSelectedItem();
	ASSERT(pItemPage);
	pItemParam = pItemPage->GetSelectedItem();
	ASSERT(pItemParam);

	m_wSelectedID = pItemParam->GetID();
	m_byCtrlSelected = 0;
	m_dwLastKeyBits = m_pPanel->PanelGetKeys();	

	SetEncoderFactor();

	m_pPanel->PanelSetLeds(m_pPanel->PanelGetLeds() & ~(PANEL01_LED_SHIFT));
	m_pPanel->PanelSetLeds(m_pPanel->PanelGetLeds() | PANEL01_LED_EDIT);	

	m_bModified = ENGINE(m_pPanel->GetEngine())->IsPresetModified();
	if (m_bModified)
		m_pPanel->ResetTimer(m_idTimer1);
}

void CMonoSynthPanelModeEdit::EndMode(void)
{
	ASSERT(m_pPanel);

	m_pPanel->PanelSetLeds(m_pPanel->PanelGetLeds() & ~(PANEL01_LED_SHIFT));
	m_pPanel->PanelSetLeds(m_pPanel->PanelGetLeds() & ~(PANEL01_LED_EDIT));
	m_pPanel->CancelTimer(m_idTimer0);
	m_pPanel->CancelTimer(m_idTimer1);
	m_pPanel->CancelTimer(m_idTimer2);
}

void CMonoSynthPanelModeEdit::OnPotentiometer(BYTE byPot, BYTE byValue)
{
	ASSERT(m_pPanel);

	CParameter *pCtrl = NULL;
	CParameter *pParameter = NULL;

	if (m_pPanel->PanelGetKeys() & PANEL01_KEY_SHIFT)
	{
		m_byCtrlSelected = 0;
		m_pPanel->CancelTimer(m_idTimer2);

		switch (byPot)
		{
			case PANEL01_POT_CTRL1:
				pCtrl = GetParameterFromID(kPotControllerAssign1);
				m_byCtrlSelected = 1;
				break;

			case PANEL01_POT_CTRL2:
				pCtrl = GetParameterFromID(kPotControllerAssign2);
				m_byCtrlSelected = 2;
				break;

			case PANEL01_POT_CTRL3:
				pCtrl = GetParameterFromID(kPotControllerAssign3);
				m_byCtrlSelected = 3;
				break;

			default:
				ASSERT(FALSE);
		}

		if (pCtrl)
		{
			pCtrl->SetValue(m_wSelectedID);
			ENGINE(m_pPanel->GetEngine())->SendMsgParameterChanged(pCtrl->GetID());
			PANEL(m_pPanel)->SendMsgParameterChanged(pCtrl->GetID());
			Display();
			m_pPanel->ResetTimer(m_idTimer2);
		}
	}
	else
	{
		switch (byPot)
		{
			case PANEL01_POT_CTRL1:
				pCtrl = GetParameterFromID(kPotControllerAssign1);
				break;

			case PANEL01_POT_CTRL2:
				pCtrl = GetParameterFromID(kPotControllerAssign2);
				break;

			case PANEL01_POT_CTRL3:
				pCtrl = GetParameterFromID(kPotControllerAssign3);
				break;
		}

		if (pCtrl && (pCtrl->GetValue() != kParameterNone))
		{
			pParameter = GetParameterFromID(pCtrl->GetValue());

			if (pParameter)
			{
				pParameter->SetValueMidi(byValue);
				ENGINE(m_pPanel->GetEngine())->SendMsgParameterChanged(pParameter->GetID());
				PANEL(m_pPanel)->SendMsgChangeMode(CMonoSynthPanel::kPanelModeAssign);
			}
		}
	}
}

void CMonoSynthPanelModeEdit::OnEncoder(BYTE byEncoder, char nInc)
{
	ASSERT(m_pPanel);

	int nValue;
	CParameter *pParameter = GetParameterFromID(m_wSelectedID);
	ASSERT(pParameter);

	m_nEncoderValue += nInc;
	nValue = m_nEncoderValue/m_nEncoderFactor;

	if (nValue < (int) pParameter->GetMin())
	{
		nValue = (int) pParameter->GetMin();
		m_nEncoderValue = nValue*m_nEncoderFactor;
	}
	else if (nValue > (int) pParameter->GetMax())
	{
		nValue = (int) pParameter->GetMax();
		m_nEncoderValue = nValue*m_nEncoderFactor;
	}

	m_pPanel->CancelTimer(m_idTimer0);
	
	if (nValue != (int) pParameter->GetValue())
	{
		pParameter->SetValue((short) nValue);
		ENGINE(m_pPanel->GetEngine())->SendMsgParameterChanged(m_wSelectedID);
		Display();
		
		if (!m_bModified && ENGINE(m_pPanel->GetEngine())->IsPresetModified())
		{
			m_bModified = TRUE;
			m_pPanel->ResetTimer(m_idTimer1);
		}
	}
	
	if (m_byCtrlSelected)
	{
		m_pPanel->CancelTimer(m_idTimer2);
		m_byCtrlSelected = 0;
		Display();
	}
}

void CMonoSynthPanelModeEdit::OnKeypad(DWORD dwKeyBits)
{
	ASSERT(m_pPanel);

	CMenuItem *pItemGroup, *pItemPage, *pItemParam;
	CParameter *pParameter;
	BOOL bKeyDown = FALSE;

	pItemGroup = m_menu.GetSelectedItem();
	ASSERT(pItemGroup);
	pItemPage = pItemGroup->GetSelectedItem();
	ASSERT(pItemPage);
	pItemParam = pItemPage->GetSelectedItem();
	ASSERT(pItemParam);
	pParameter = GetParameterFromID(pItemParam->GetID());
	ASSERT(pParameter);
	
	if (!(m_dwLastKeyBits & PANEL01_KEY_SHIFT) || (dwKeyBits & PANEL01_KEY_SHIFT))
	{
		m_pPanel->CancelTimer(m_idTimer0);
	}
	
	if (!(m_dwLastKeyBits & PANEL01_KEY_VALUE_UP) && (dwKeyBits & PANEL01_KEY_VALUE_UP))
	{
		bKeyDown = TRUE;
		if (pParameter->IncValue())
		{
			m_nIncrement = 1;
			ENGINE(m_pPanel->GetEngine())->SendMsgParameterChanged(m_wSelectedID);
			Display();
			m_pPanel->ChangeTimerInterval(m_idTimer0, 650);

			if (!m_bModified && ENGINE(m_pPanel->GetEngine())->IsPresetModified())
			{
				m_bModified = TRUE;
				m_pPanel->ResetTimer(m_idTimer1);
			}
		}
	}
	else if (!(m_dwLastKeyBits & PANEL01_KEY_VALUE_DOWN) && (dwKeyBits & PANEL01_KEY_VALUE_DOWN))
	{
		bKeyDown = TRUE;
		if (pParameter->DecValue())
		{
			m_nIncrement = -1;
			ENGINE(m_pPanel->GetEngine())->SendMsgParameterChanged(m_wSelectedID);
			Display();
			m_pPanel->ChangeTimerInterval(m_idTimer0, 650);

			if (!m_bModified && ENGINE(m_pPanel->GetEngine())->IsPresetModified())
			{
				m_bModified = TRUE;
				m_pPanel->ResetTimer(m_idTimer1);
			}
		}
	}
	else if (dwKeyBits & PANEL01_KEY_PARAM_UP)
	{
		bKeyDown = TRUE;
		if (pItemPage->SelectNextItem())
		{
			SetEncoderFactor();
			Display();
		}
	}
	else if (dwKeyBits & PANEL01_KEY_PARAM_DOWN)
	{
		bKeyDown = TRUE;
		if (pItemPage->SelectPrevItem())
		{
			SetEncoderFactor();
			Display();
		}
	}
	else if (dwKeyBits & PANEL01_KEY_PAGE_UP)
	{
		bKeyDown = TRUE;
		if (pItemGroup->SelectNextItem())
		{
			SetEncoderFactor();
			Display();
		}
	}
	else if (dwKeyBits & PANEL01_KEY_PAGE_DOWN)
	{
		bKeyDown = TRUE;
		if (pItemGroup->SelectPrevItem())
		{
			SetEncoderFactor();
			Display();
		}
	}
	else if (dwKeyBits & PANEL01_KEY_GROUP_UP)
	{
		bKeyDown = TRUE;
		if (m_menu.SelectNextItem())
		{
			SetEncoderFactor();
			Display();
		}
	}
	else if (dwKeyBits & PANEL01_KEY_GROUP_DOWN)
	{
		bKeyDown = TRUE;
		if (m_menu.SelectPrevItem())
		{
			SetEncoderFactor();
			Display();
		}
	}
	else if (dwKeyBits & PANEL01_KEY_EDIT)
	{
		if (ENGINE(m_pPanel->GetEngine())->IsPresetModified())
		{
			PANEL(m_pPanel)->SendMsgChangeMode(CMonoSynthPanel::kPanelModeSave);
			return;
		}

		m_pPanel->PanelSetLeds(m_pPanel->PanelGetLeds() & ~(PANEL01_LED_EDIT));
		PANEL(m_pPanel)->SendMsgChangeMode(CMonoSynthPanel::kPanelModePreset);
	}

	pItemGroup = m_menu.GetSelectedItem();
	ASSERT(pItemGroup);
	pItemPage = pItemGroup->GetSelectedItem();
	ASSERT(pItemPage);
	pItemParam = pItemPage->GetSelectedItem();
	ASSERT(pItemParam);

	m_wSelectedID = pItemParam->GetID();
	m_dwLastKeyBits = dwKeyBits;
	
	SetEncoderFactor();

	if (m_byCtrlSelected && bKeyDown)
	{
		m_pPanel->CancelTimer(m_idTimer2);
		m_byCtrlSelected = 0;
		Display();
	}
}

void CMonoSynthPanelModeEdit::OnTimer(rtems_id timer_id)
{
	ASSERT(m_pPanel);

	if (timer_id == m_idTimer0)
	{
		CParameter *pParameter = GetParameterFromID(m_wSelectedID);
		ASSERT(pParameter);
		short nValue = (short) pParameter->GetValue();

		if (m_nIncrement > 0)
			nValue++;
		else if (m_nIncrement < 0)
			nValue--;
		else
			ASSERT(FALSE);

		pParameter->SetValue(nValue);
		ENGINE(m_pPanel->GetEngine())->SendMsgParameterChanged(m_wSelectedID);
		Display();
		if (nValue == (short) pParameter->GetValue())
			m_pPanel->ChangeTimerInterval(m_idTimer0, 100);
	}
	else if (timer_id == m_idTimer1)
	{
		if (m_pPanel->PanelGetLeds() & PANEL01_LED_EDIT)
			m_pPanel->PanelSetLeds(m_pPanel->PanelGetLeds() & ~(PANEL01_LED_EDIT));
		else
			m_pPanel->PanelSetLeds(m_pPanel->PanelGetLeds() | PANEL01_LED_EDIT);

		m_pPanel->ResetTimer(m_idTimer1);
	}
	else if (timer_id == m_idTimer2)
	{
		if (m_byCtrlSelected)
		{
			m_byCtrlSelected = 0;
			Display();
		}
		else
			ASSERT(FALSE);
	}
	else
		ASSERT(FALSE);
}

void CMonoSynthPanelModeEdit::OnParameterChanged(WORD wID)
{
	ASSERT(m_pPanel);

	if (wID == m_wSelectedID)
	{
		Display();
	}
	
	if (!m_bModified && ENGINE(m_pPanel->GetEngine())->IsPresetModified())
	{
		m_bModified = TRUE;
		m_pPanel->ResetTimer(m_idTimer1);
	}
}

void CMonoSynthPanelModeEdit::OnPresetChanged(BYTE byPreset)
{
	ASSERT(m_pPanel);

	m_pPanel->PanelSetLeds(m_pPanel->PanelGetLeds() & ~(PANEL01_LED_EDIT));
	PANEL(m_pPanel)->SendMsgChangeMode(CMonoSynthPanel::kPanelModePreset);
}

void CMonoSynthPanelModeEdit::Display(void)
{
	ASSERT(m_pPanel);

	if (m_byCtrlSelected)
	{
		sprintf(m_strLine0, " Ctrl%d assigned ", m_byCtrlSelected);
		sprintf(m_strLine1, "%16s", "");
	}
	else
	{	
		CParameter *pParameter;
		CMenuItem *pItemGroup, *pItemPage, *pItemParam;
		char strValueText[16+1];
		char strFormat[32];
		
		pItemGroup = m_menu.GetSelectedItem();
		ASSERT(pItemGroup);
		pItemPage = pItemGroup->GetSelectedItem();
		ASSERT(pItemPage);
		pItemParam = pItemPage->GetSelectedItem();
		ASSERT(pItemParam);
		pParameter = GetParameterFromID(pItemParam->GetID());
		ASSERT(pParameter);
		
		strValueText[0] = 1;
		pParameter->GetValueText(strValueText+1);
		strValueText[strlen(strValueText)+1] = '\0';
		strValueText[strlen(strValueText)] = 2;
	
		sprintf(strFormat, "%%s %%-%lds", 16-strlen(pItemGroup->GetMenuName())-1);
		sprintf(m_strLine0, strFormat, pItemGroup->GetMenuName(), pItemPage->GetMenuName());
		sprintf(strFormat, "%%s%%%lds%%s", 16-strlen(pItemParam->GetMenuName())-strlen(strValueText));
		sprintf(m_strLine1, strFormat, pItemParam->GetMenuName(), " ", strValueText);
	}

	m_pPanel->PanelLcdPrint(0, 0, m_strLine0);
	m_pPanel->PanelLcdPrint(1, 0, m_strLine1);
}

void CMonoSynthPanelModeEdit::SetEncoderFactor(void)
{
	CParameter *pParameter = GetParameterFromID(m_wSelectedID);
	
	m_nEncoderFactor = 256 / pParameter->GetRange();
	if (m_nEncoderFactor > 32)
		m_nEncoderFactor /= 6;
		
	m_nEncoderValue = (int) pParameter->GetValue() * m_nEncoderFactor;
}
