/*******************************************************************************
 * C H A M E L E O N                                                           *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/MONOSYNTH/MonoSynthPanelModeInit.cp $
 *     $Date:: 13/05/02 20:11                                                  $
 * $Revision:: 1                                                               $
 *-----------------------------------------------------------------------------*
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#include <rtems.h>
#include <chameleon.h>

#include "MonoSynthEngine.h"
#include "MonoSynthPanel.h"
#include "MonoSynthPanelModeInit.h"

BOOL CMonoSynthPanelModeInit::Initialize(CPanel *pPanel)
{
	if (!CPanelModeHandler::Initialize(pPanel))
		return FALSE;

	// Create the timer to exit the init mode
	m_idTimer = m_pPanel->CreateTimer(rtems_build_name('T', 'I', 'N', 'I'), 2000);

	return TRUE;
}

void CMonoSynthPanelModeInit::BeginMode(int nModeBefore)
{
	ASSERT(m_pPanel);
	ASSERT(nModeBefore == CMonoSynthPanel::kPanelModeInvalid);

	TRACE("Init Mode\n");

	m_pPanel->ResetTimer(m_idTimer);
}

void CMonoSynthPanelModeInit::EndMode(void)
{
	ASSERT(m_pPanel);

	m_pPanel->CancelTimer(m_idTimer);
}

void CMonoSynthPanelModeInit::OnKeypad(DWORD dwKeyBits)
{
	ASSERT(m_pPanel);

	if (dwKeyBits != 0)
		Exit();
}

void CMonoSynthPanelModeInit::OnTimer(rtems_id idTimer)
{
	ASSERT(m_pPanel);

	if (idTimer == m_idTimer)
		Exit();
	else
	{
		TRACE("ERROR: Invalid timer received in CMonoSynthPanelModeInit::OnTimer\n");
		ASSERT(FALSE);
	}
}

void CMonoSynthPanelModeInit::Display(void)
{
	ASSERT(m_pPanel);

	m_pPanel->PanelLcdPrint(0, 0, "   mono synth   ");
	m_pPanel->PanelLcdPrint(1, 0, "      v1.0      ");
}

void CMonoSynthPanelModeInit::Exit(void)
{
	ASSERT(m_pPanel);

	while (!m_pPanel->GetEngine()->IsInitialized())
		rtems_task_wake_after(100);

	ENGINE(m_pPanel->GetEngine())->LoadPreset(0);
	PANEL(m_pPanel)->SendMsgChangeMode(CMonoSynthPanel::kPanelModePreset);
}
