/*******************************************************************************
 * C H A M E L E O N                                                           *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/MONOSYNTH/MonoSynthPanelModeMidi.cp $
 *     $Date:: 13/05/02 20:11                                                  $
 * $Revision:: 1                                                               $
 *-----------------------------------------------------------------------------*
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#include <rtems.h>
#include <chameleon.h>

#include "MonoSynthEngine.h"
#include "MonoSynthPanel.h"
#include "MonoSynthPanelModeMidi.h"

BOOL CMonoSynthPanelModeMidi::Initialize(CPanel *pPanel)
{
	if (!CPanelModeHandler::Initialize(pPanel))
		return FALSE;

	// Create the timer to flash the edit and shift leds
	m_idTimer = m_pPanel->CreateTimer(rtems_build_name('T', 'M', 'I', 'D'), 500);

	m_nCurrentMenu = kMidiDumpFullBank;

	return TRUE;
}

void CMonoSynthPanelModeMidi::BeginMode(int nModeBefore)
{
	ASSERT(m_pPanel);
	ASSERT(nModeBefore == CMonoSynthPanel::kPanelModePreset);

	TRACE("MIDI Mode\n");

	m_pPanel->PanelSetLeds(PANEL01_LED_EDIT);

	m_bStatus = FALSE;	
	m_pPanel->ResetTimer(m_idTimer);
}

void CMonoSynthPanelModeMidi::EndMode(void)
{
	ASSERT(m_pPanel);

	m_pPanel->CancelTimer(m_idTimer);
}

void CMonoSynthPanelModeMidi::OnKeypad(DWORD dwKeyBits)
{
	ASSERT(m_pPanel);

	if (dwKeyBits & PANEL01_KEY_EDIT)
	{
		if (m_nCurrentMenu == kMidiDumpFullBank)
		{
			m_pPanel->PanelLcdPrint(1, 0, "Sending sysex...");
			ENGINE(m_pPanel->GetEngine())->DumpBank();
			Display();
		}
		else if (m_nCurrentMenu == kMidiDumpCurrentPreset)
		{
			m_pPanel->PanelLcdPrint(1, 0, "Sending sysex...");
			ENGINE(m_pPanel->GetEngine())->DumpPreset();
			Display();
		}
	}
	else if ((dwKeyBits & PANEL01_KEY_SHIFT) || (dwKeyBits & PANEL01_KEY_GROUP_UP) || (dwKeyBits & PANEL01_KEY_GROUP_DOWN))
	{
		PANEL(m_pPanel)->SendMsgChangeMode(CMonoSynthPanel::kPanelModePreset);
	}
	else if (dwKeyBits & PANEL01_KEY_VALUE_UP)
	{
		if (m_nCurrentMenu < kMidiMaxEntries-1)
		{
			m_nCurrentMenu++;
			Display();
		}
	}
	else if (dwKeyBits & PANEL01_KEY_VALUE_DOWN)
	{
		if (m_nCurrentMenu > 0)
		{
			m_nCurrentMenu--;
			Display();
		}
	}
}

void CMonoSynthPanelModeMidi::OnTimer(rtems_id idTimer)
{
	ASSERT(m_pPanel);

	if (idTimer == m_idTimer)
	{
		if (m_pPanel->PanelGetLeds() & PANEL01_LED_EDIT)
		{
			m_pPanel->PanelSetLeds(m_pPanel->PanelGetLeds() & ~(PANEL01_LED_EDIT));
			m_pPanel->PanelSetLeds(m_pPanel->PanelGetLeds() | PANEL01_LED_SHIFT);
		}
		else
		{
			m_pPanel->PanelSetLeds(m_pPanel->PanelGetLeds() | PANEL01_LED_EDIT);
			m_pPanel->PanelSetLeds(m_pPanel->PanelGetLeds() & ~(PANEL01_LED_SHIFT));
		}

		m_pPanel->ResetTimer(m_idTimer);
	
	}
	else
		ASSERT(FALSE);
}

void CMonoSynthPanelModeMidi::Display(void)
{
	ASSERT(m_pPanel);
	
	static const char *arrayMenus[kMidiMaxEntries] =
	{
		"\1Full Bank     \2",
		"\1Current Preset\2",
	};

	m_pPanel->PanelLcdPrint(0, 0, "MIDI Dump:      ");
	m_pPanel->PanelLcdPrint(1, 0, arrayMenus[m_nCurrentMenu]);
}
