/*******************************************************************************
 * C H A M E L E O N                                                           *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/monosynth/MonoSynthPanelModePreset. $
 *     $Date:: 15/05/02 20:51                                                  $
 * $Revision:: 2                                                               $
 *-----------------------------------------------------------------------------*
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <rtems.h>
#include <chameleon.h>

#include "MonoSynthEngine.h"
#include "MonoSynthPanel.h"
#include "MonoSynthPanelModePreset.h"
#include "MonoSynthParameters.h"

#include "demos/midifile_demo.h"

////////////////////////////////////////////////////////////////////////////////
#define ENCODER_FACTOR		4
////////////////////////////////////////////////////////////////////////////////

BOOL CMonoSynthPanelModePreset::Initialize(CPanel *pPanel)
{
	if (!CPanelModeHandler::Initialize(pPanel))
		return FALSE;

	// Create the timer for showing the modified state of a preset
	m_idTimer0 = m_pPanel->CreateTimer(rtems_build_name('T', 'P', 'R', '0'), 600);
	// Create the timer for change a preset (keypad and encoder)
	m_idTimer1 = m_pPanel->CreateTimer(rtems_build_name('T', 'P', 'R', '2'), 100);
	if ((m_idTimer0 == 0) || (m_idTimer1 == 0))
		return FALSE;

	return TRUE;
}

void CMonoSynthPanelModePreset::BeginMode(int nModeBefore)
{
	ASSERT(m_pPanel);

	TRACE("Preset Mode\n");

	m_byPreset = ENGINE(m_pPanel->GetEngine())->GetCurrentPreset();
	m_dwLastKeyBits = m_pPanel->PanelGetKeys();
	m_nEncoderValue = 0;
	
	CParameter *pParameter;
	DWORD dwLeds = 0;
	
	pParameter = GetParameterFromID(kPotControllerAssign1);
	if (pParameter && (pParameter->GetValue() != kParameterNone))
		dwLeds |= PANEL01_LED_CTRL1;

	pParameter = GetParameterFromID(kPotControllerAssign2);
	if (pParameter && (pParameter->GetValue() != kParameterNone))
		dwLeds |= PANEL01_LED_CTRL2;

	pParameter = GetParameterFromID(kPotControllerAssign3);
	if (pParameter && (pParameter->GetValue() != kParameterNone))
		dwLeds |= PANEL01_LED_CTRL3;

	m_pPanel->PanelSetLeds(dwLeds);

	OnChangePreset();
}

void CMonoSynthPanelModePreset::EndMode(void)
{
	ASSERT(m_pPanel);

	m_pPanel->CancelTimer(m_idTimer0);
	m_pPanel->CancelTimer(m_idTimer1);
}

void CMonoSynthPanelModePreset::OnPotentiometer(BYTE byPot, BYTE byValue)
{
	ASSERT(m_pPanel);

	if (ENGINE(m_pPanel->GetEngine())->IsDemoPlaying())
		return;

	CParameter *pCtrl = NULL;
	CParameter *pParameter = NULL;

	switch (byPot)
	{
		case PANEL01_POT_CTRL1:
			pCtrl = GetParameterFromID(kPotControllerAssign1);
			break;

		case PANEL01_POT_CTRL2:
			pCtrl = GetParameterFromID(kPotControllerAssign2);
			break;

		case PANEL01_POT_CTRL3:
			pCtrl = GetParameterFromID(kPotControllerAssign3);
			break;
	}

	if ((pCtrl) && (pCtrl->GetValue() != kParameterNone))
	{
		pParameter = GetParameterFromID(pCtrl->GetValue());

		if (pParameter)
		{
			pParameter->SetValueMidi(byValue);
			ENGINE(m_pPanel->GetEngine())->SendMsgParameterChanged(pParameter->GetID());
			PANEL(m_pPanel)->SendMsgChangeMode(CMonoSynthPanel::kPanelModeAssign);
		}
	}
}

void CMonoSynthPanelModePreset::OnEncoder(BYTE byEncoder, char nInc)
{
	ASSERT(m_pPanel);

	if (ENGINE(m_pPanel->GetEngine())->IsDemoPlaying())
		return;

	int nPreset;

	m_nEncoderValue += nInc;
	nPreset = m_nEncoderValue/ENCODER_FACTOR;

	if (nPreset > 0x7F)
	{
		nPreset = 0x7F;
		m_nEncoderValue = nPreset*ENCODER_FACTOR;
	}
	else if (nPreset < 0x00)
	{
		nPreset = 0x00;
		m_nEncoderValue = nPreset*ENCODER_FACTOR;
	}

	if ((BYTE) nPreset != m_byPreset)
	{
		m_pPanel->CancelTimer(m_idTimer0);
		m_pPanel->CancelTimer(m_idTimer1);
		m_bStatus = FALSE;
		m_byPreset = (BYTE) nPreset;
		Display();
		m_nPresetInc = 0;
		m_pPanel->ChangeTimerInterval(m_idTimer1, 100);
	}
}

void CMonoSynthPanelModePreset::OnKeypad(DWORD dwKeyBits)
{
	ASSERT(m_pPanel);

	if (ENGINE(m_pPanel->GetEngine())->IsDemoPlaying())
	{
		if (!(m_dwLastKeyBits & PANEL01_KEY_SHIFT) && (dwKeyBits & PANEL01_KEY_SHIFT))
		{
			m_pPanel->CancelTimer(m_idTimer0);
			ENGINE(m_pPanel->GetEngine())->StopMidiFile();
			OnChangePreset();
		}

		m_dwLastKeyBits = dwKeyBits;
		return;
	}

	if ((m_dwLastKeyBits & PANEL01_KEY_VALUE_UP) && !(dwKeyBits & PANEL01_KEY_VALUE_UP))
	{
		m_pPanel->CancelTimer(m_idTimer1);
		if (m_nPresetInc > 0)
			ENGINE(m_pPanel->GetEngine())->LoadPreset(m_byPreset);
	}
	if ((m_dwLastKeyBits & PANEL01_KEY_VALUE_DOWN) && !(dwKeyBits & PANEL01_KEY_VALUE_DOWN))
	{
		m_pPanel->CancelTimer(m_idTimer1);
		if (m_nPresetInc < 0)
			ENGINE(m_pPanel->GetEngine())->LoadPreset(m_byPreset);
	}
	
	if (dwKeyBits & PANEL01_KEY_VALUE_UP)
	{
		if (m_byPreset < 0x7F)
		{
			m_pPanel->CancelTimer(m_idTimer0);
			m_pPanel->CancelTimer(m_idTimer1);
			m_bStatus = FALSE;
			m_byPreset++;
			Display();
			m_nPresetInc = 1;
			m_pPanel->ChangeTimerInterval(m_idTimer1, 650);
		}
	}
	else if (dwKeyBits & PANEL01_KEY_VALUE_DOWN)
	{
		if (m_byPreset > 0x00)
		{
			m_pPanel->CancelTimer(m_idTimer0);
			m_pPanel->CancelTimer(m_idTimer1);
			m_bStatus = FALSE;
			m_byPreset--;
			Display();
			m_nPresetInc = -1;
			m_pPanel->ChangeTimerInterval(m_idTimer1, 650);
		}
	}
	else if ((dwKeyBits & PANEL01_KEY_SHIFT) && (dwKeyBits & PANEL01_KEY_PART_DOWN))
	{
		m_pPanel->PanelLcdPrint(0, 0, " Preparing DEMO ");
		m_pPanel->PanelLcdPrint(1, 0, "   Loading...   ");
		ENGINE(m_pPanel->GetEngine())->PlayMidiFile(midifile_demo, midifile_demoSize);
		m_bStatus = FALSE;
		m_nCounter = 0;
		m_pPanel->ResetTimer(m_idTimer0);
		Display();
	}
	else if ((dwKeyBits & PANEL01_KEY_GROUP_UP) || (dwKeyBits & PANEL01_KEY_GROUP_DOWN))
		PANEL(m_pPanel)->SendMsgChangeMode(CMonoSynthPanel::kPanelModeMidi);
	else if (dwKeyBits & PANEL01_KEY_EDIT)
		PANEL(m_pPanel)->SendMsgChangeMode(CMonoSynthPanel::kPanelModeEdit);

	m_dwLastKeyBits = dwKeyBits;
}

void CMonoSynthPanelModePreset::OnTimer(rtems_id idTimer)
{
	ASSERT(m_pPanel);

	if (idTimer == m_idTimer0)
	{
		m_bStatus = !m_bStatus;
		m_pPanel->ResetTimer(m_idTimer0);
		Display();
	}
	else if (idTimer == m_idTimer1)
	{
		if (m_nPresetInc > 0)
		{
			if (m_byPreset < 0x7F)
			{
				m_byPreset++;
				Display();
				m_pPanel->ChangeTimerInterval(m_idTimer1, 150);
			}
		}
		else if (m_nPresetInc < 0)
		{
			if (m_byPreset > 0x00)
			{
				m_byPreset--;
				Display();
				m_pPanel->ChangeTimerInterval(m_idTimer1, 150);
			}
		}
		else 
			ENGINE(m_pPanel->GetEngine())->LoadPreset(m_byPreset);
	}
	else
		ASSERT(FALSE);
}

void CMonoSynthPanelModePreset::OnPresetChanged(BYTE byPreset)
{
	if (ENGINE(m_pPanel->GetEngine())->IsDemoPlaying())
		return;

	m_byPreset = byPreset;
	OnChangePreset();
}

void CMonoSynthPanelModePreset::Display(void)
{
	ASSERT(m_pPanel);

	if (ENGINE(m_pPanel->GetEngine())->IsDemoPlaying())
	{
		if (!m_bStatus)
			sprintf(m_strLine0, "  Playing DEMO  ");
		else	sprintf(m_strLine0, "%16s", "");

		sprintf(m_strLine1, "%16s", "");
		m_strLine1[m_nCounter++] = 1;
		if (m_nCounter > 15)
			m_nCounter = 0;
	}
	else
	{
		char strPresetName[kMaxPresetNameLen + 1];
		char chModified;
	
		memset(strPresetName, ' ', kMaxPresetNameLen);
		ENGINE(m_pPanel->GetEngine())->GetPresetName(m_byPreset, strPresetName);
		chModified = m_bStatus ? '*' : ' ';
	
		sprintf(m_strLine0, "Preset %03d %c     ", m_byPreset+1, chModified);
		sprintf(m_strLine1, "%-16s", strPresetName);
	}

	m_pPanel->PanelLcdPrint(0, 0, m_strLine0);
	m_pPanel->PanelLcdPrint(1, 0, m_strLine1);
}

void CMonoSynthPanelModePreset::OnChangePreset(void)
{
	m_bModified = ENGINE(m_pPanel->GetEngine())->IsPresetModified();

	if (m_bModified)
		m_pPanel->ResetTimer(m_idTimer0);
	else
		m_pPanel->CancelTimer(m_idTimer0);

	m_nPresetInc = 0;
	m_bStatus = FALSE;
	Display();
}
