/*******************************************************************************
 * C H A M E L E O N                                                           *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/monosynth/MonoSynthPanelModeSave.cp $
 *     $Date:: 15/05/02 20:51                                                  $
 * $Revision:: 2                                                               $
 *-----------------------------------------------------------------------------*
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

//#include <stdlib.h>
//#include <string.h>
#include <rtems.h>
#include <chameleon.h>

#include "MonoSynthEngine.h"
#include "MonoSynthPanel.h"
#include "MonoSynthPanelModeSave.h"

////////////////////////////////////////////////////////////////////////////////

BOOL CMonoSynthPanelModeSave::Initialize(CPanel *pPanel)
{
	if (!CPanelModeHandler::Initialize(pPanel))
		return FALSE;

	// Create the timer for showing the modified state of a preset
	m_idTimer0 = m_pPanel->CreateTimer(rtems_build_name('T', 'S', 'A', '0'), 500);
	// Create the timer to change a preset (keypad)
	m_idTimer1 = m_pPanel->CreateTimer(rtems_build_name('T', 'S', 'A', '1'), 500);
	// Create the timer for preset name editing
	m_idTimer2 = m_pPanel->CreateTimer(rtems_build_name('T', 'S', 'A', '2'), 350);

	return TRUE;
}

void CMonoSynthPanelModeSave::BeginMode(int nModeBefore)
{
	ASSERT(m_pPanel);
	ASSERT(nModeBefore == CMonoSynthPanel::kPanelModeEdit);

	TRACE("Save Mode\n");

	m_bySavePreset = ENGINE(m_pPanel->GetEngine())->GetCurrentPreset();
	sprintf(m_strPresetName, "%14s", "");
	ENGINE(m_pPanel->GetEngine())->GetPresetName(m_bySavePreset, m_strPresetName);
	m_dwLastKeyBits = m_pPanel->PanelGetKeys();
	m_nStage = kStageAsking;
	m_bResponse = TRUE;
	m_bStatus = TRUE;
	
	m_pPanel->PanelSetLeds(PANEL01_LED_EDIT);
	m_pPanel->ResetTimer(m_idTimer0);
}

void CMonoSynthPanelModeSave::EndMode(void)
{
	ASSERT(m_pPanel);

	m_pPanel->CancelTimer(m_idTimer0);
	m_pPanel->CancelTimer(m_idTimer1);
	m_pPanel->CancelTimer(m_idTimer2);
}

void CMonoSynthPanelModeSave::OnEncoder(BYTE byEncoder, char nInc)
{
	ASSERT(m_pPanel);

	if (m_nStage == kStagePresetNumber)
	{
		int nSavePreset = (int) m_bySavePreset;

		m_nEncoderValue += nInc;
		nInc = m_nEncoderValue >> 2;
		m_nEncoderValue -= (nInc << 2);

		nSavePreset += nInc;

		if (nSavePreset > 0x7F)
			nSavePreset = 0x7F;
		else if (nSavePreset < 0x00)
			nSavePreset = 0x00;

		if ((BYTE) nSavePreset != m_bySavePreset)
		{
			m_bySavePreset = (BYTE) nSavePreset;
			Display();
		}
	}
	else if (m_nStage == kStagePresetName)
	{
		int ch = (int) m_strPresetName[m_nCol];

		m_nEncoderValue += nInc;
		nInc = m_nEncoderValue >> 2;
		m_nEncoderValue -= (nInc << 2);

		ch += nInc;

		if (ch > (int) ('z' + 5))
			ch = (int) ('z' + 5);
		else if (ch < (int) ' ')
			ch = (int) ' ';

		if ((char) ch != m_strPresetName[m_nCol])
		{
			m_pPanel->CancelTimer(m_idTimer2);
			m_strPresetName[m_nCol] = (char) ch;
			m_bStatus = TRUE;
			Display();
			m_pPanel->ResetTimer(m_idTimer2);
		}
		
	}
}

void CMonoSynthPanelModeSave::OnKeypad(DWORD dwKeyBits)
{
	ASSERT(m_pPanel);

	if ((m_dwLastKeyBits & PANEL01_KEY_VALUE_UP) && !(dwKeyBits & PANEL01_KEY_VALUE_UP))
	{
		m_pPanel->CancelTimer(m_idTimer1);
		if (m_nStage == kStagePresetName)
			m_pPanel->ResetTimer(m_idTimer2);
	}
	if ((m_dwLastKeyBits & PANEL01_KEY_VALUE_DOWN) && !(dwKeyBits & PANEL01_KEY_VALUE_DOWN))
	{
		m_pPanel->CancelTimer(m_idTimer1);
		if (m_nStage == kStagePresetName)
			m_pPanel->ResetTimer(m_idTimer2);
	}
	else if (dwKeyBits & PANEL01_KEY_VALUE_UP)
	{
		if (m_nStage == kStageAsking)
		{
			m_bResponse = TRUE;
			Display();
		}
		else if (m_nStage == kStagePresetNumber)
		{
			if (m_bySavePreset < 0x7F)
			{
				m_pPanel->CancelTimer(m_idTimer1);
				m_bySavePreset++;
				Display();
				m_nIncrement = 1;
				m_pPanel->ChangeTimerInterval(m_idTimer1, 650);
			}
		}
		else if (m_nStage == kStagePresetName)
		{
			if (m_strPresetName[m_nCol] < ('z' + 5))
			{
				m_pPanel->CancelTimer(m_idTimer2);
				m_pPanel->CancelTimer(m_idTimer1);
				m_strPresetName[m_nCol]++;
				m_bStatus = TRUE;
				Display();
				m_nIncrement = 1;
				m_pPanel->ChangeTimerInterval(m_idTimer1, 650);
			}
		}
	}
	else if (dwKeyBits & PANEL01_KEY_VALUE_DOWN)
	{
		if (m_nStage == kStageAsking)
		{
			m_bResponse = FALSE;
			Display();
		}
		else if (m_nStage == kStagePresetNumber)
		{
			if (m_bySavePreset > 0x00)
			{
				m_pPanel->CancelTimer(m_idTimer1);
				m_bySavePreset--;
				Display();
				m_nIncrement = -1;
				m_pPanel->ChangeTimerInterval(m_idTimer1, 650);
			}
		}
		else if (m_nStage == kStagePresetName)
		{
			if (m_strPresetName[m_nCol] > ' ')
			{
				m_pPanel->CancelTimer(m_idTimer2);
				m_pPanel->CancelTimer(m_idTimer1);
				m_strPresetName[m_nCol]--;
				m_bStatus = TRUE;
				Display();
				m_nIncrement = -1;
				m_pPanel->ChangeTimerInterval(m_idTimer1, 650);
			}
		}
	}
	else if (dwKeyBits & PANEL01_KEY_EDIT)
	{
		m_pPanel->CancelTimer(m_idTimer1);

		if (m_nStage == kStageAsking)
		{
			if (m_bResponse)
			{
				m_nStage = kStagePresetNumber;
				m_nEncoderValue = 0;
				m_nIncrement = 0;
				Display();
			}
			else
				PANEL(m_pPanel)->SendMsgChangeMode(CMonoSynthPanel::kPanelModePreset);
		}
		else if (m_nStage == kStagePresetNumber)
		{
			m_nStage = kStagePresetName;
			m_bStatus = TRUE;
			m_nEncoderValue = 0;
			m_nCol = 0;
			Display();
			m_pPanel->ResetTimer(m_idTimer2);
		}
		else if (m_nStage == kStagePresetName)
		{
			m_pPanel->CancelTimer(m_idTimer0);
			m_pPanel->CancelTimer(m_idTimer1);
			m_pPanel->CancelTimer(m_idTimer2);
			m_pPanel->PanelSetLeds(m_pPanel->PanelGetLeds() | PANEL01_LED_SHIFT | PANEL01_LED_EDIT);
			m_nStage = kStageWaitingSave;
			Display();
			m_strPresetName[kMaxPresetNameLen] = '\0';
			ENGINE(m_pPanel->GetEngine())->SavePreset(m_bySavePreset, m_strPresetName);
			ENGINE(m_pPanel->GetEngine())->LoadPreset(m_bySavePreset);
			PANEL(m_pPanel)->SendMsgChangeMode(CMonoSynthPanel::kPanelModePreset);
		}
	}
	else if (dwKeyBits & PANEL01_KEY_SHIFT)
	{
		PANEL(m_pPanel)->SendMsgChangeMode(CMonoSynthPanel::kPanelModePreset);
	}
	else if (dwKeyBits & PANEL01_KEY_PARAM_UP)
	{
		if (m_nStage == kStagePresetName)
		{
			if (m_nCol < kMaxPresetNameLen-1)
				m_nCol++;
		}
	}
	else if (dwKeyBits & PANEL01_KEY_PARAM_DOWN)
	{
		if (m_nStage == kStagePresetName)
		{
			if (m_nCol > 0)
				m_nCol--;
		}
	}

	m_dwLastKeyBits = dwKeyBits;
}

void CMonoSynthPanelModeSave::OnTimer(rtems_id timer_id)
{
	ASSERT(m_pPanel);

	if (timer_id == m_idTimer0)
	{
		if (m_pPanel->PanelGetLeds() & PANEL01_LED_EDIT)
			m_pPanel->PanelSetLeds((m_pPanel->PanelGetLeds() | PANEL01_LED_SHIFT) & ~(PANEL01_LED_EDIT));
		else
			m_pPanel->PanelSetLeds((m_pPanel->PanelGetLeds() | PANEL01_LED_EDIT) & ~(PANEL01_LED_SHIFT));

		if (m_nStage == kStageAsking)
		{
			m_bStatus = !m_bStatus;
			Display();
		}

		m_pPanel->ResetTimer(m_idTimer0);
	}
	else if (timer_id == m_idTimer1)
	{
		if (m_nStage == kStagePresetNumber)
		{
			if (m_nIncrement > 0)
			{
				if (m_bySavePreset < 0x7F)
				{
					m_bySavePreset++;
					Display();
					m_pPanel->ChangeTimerInterval(m_idTimer1, 150);
				}
			}
			else if (m_nIncrement < 0)
			{
				if (m_bySavePreset > 0x00)
				{
					m_bySavePreset--;
					Display();
					m_pPanel->ChangeTimerInterval(m_idTimer1, 150);
				}
			}
		}
		else if (m_nStage == kStagePresetName)
		{
			if (m_nIncrement > 0)
			{
				if (m_strPresetName[m_nCol] < ('z' + 5))
				{
					m_strPresetName[m_nCol]++;
					Display();
					m_pPanel->ChangeTimerInterval(m_idTimer1, 150);
				}
			}
			else if (m_nIncrement < 0)
			{
				if (m_strPresetName[m_nCol] > ' ')
				{
					m_strPresetName[m_nCol]--;
					Display();
					m_pPanel->ChangeTimerInterval(m_idTimer1, 150);
				}
			}
		}

	}
	else if (timer_id == m_idTimer2)
	{
		if (m_nStage == kStagePresetName)
		{
			m_bStatus = !m_bStatus;
			Display();
			m_pPanel->ResetTimer(m_idTimer2);
		}
	}
	else
		ASSERT(FALSE);
}

void CMonoSynthPanelModeSave::OnPresetChanged(BYTE byPreset)
{
	ASSERT(m_pPanel);

	m_pPanel->PanelSetLeds(m_pPanel->PanelGetLeds() & ~(PANEL01_LED_EDIT));
	PANEL(m_pPanel)->SendMsgChangeMode(CMonoSynthPanel::kPanelModePreset);
}

void CMonoSynthPanelModeSave::Display(void)
{
	ASSERT(m_pPanel);

	if (m_nStage == kStageAsking)
	{
		char ch = m_bStatus ? (m_bResponse ? 'Y' : 'N') : ' ';

		sprintf(m_strLine0, "Preset modified ");
		sprintf(m_strLine1, "Store it? [%c]   ", ch);
	}
	else if (m_nStage == kStagePresetNumber)
	{
		sprintf(m_strLine0, "Preset number?  ");
		sprintf(m_strLine1, "[%03d]           ", m_bySavePreset+1);
	}
	else if (m_nStage == kStagePresetName)
	{
		sprintf(m_strLine0, "Preset name?    ");
		sprintf(m_strLine1, "[%-14s]  ", m_strPresetName);
		if (!m_bStatus)
			m_strLine1[1 + m_nCol] = '_';
	}
	else
	{
		sprintf(m_strLine0, "Saving Preset%3d", m_bySavePreset+1);
		sprintf(m_strLine1, "%-16s", m_strPresetName);
	}

	m_pPanel->PanelLcdPrint(0, 0, m_strLine0);
	m_pPanel->PanelLcdPrint(1, 0, m_strLine1);
}
