/*******************************************************************************
 * C H A M E L E O N                                                           *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/monosynth/MonoSynthRenderer.cpp     $
 *     $Date:: 15/05/02 20:51                                                  $
 * $Revision:: 3                                                               $
 *-----------------------------------------------------------------------------*
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#include <math.h>
#include <rtems.h>
#include <chameleon.h>

#include "MonoSynthRenderer.h"

#include "dsp/dsp_code.h"

//////////////////////////////////////////////////////////////////////////////
#define LOG_SLOPE	80.0f
//////////////////////////////////////////////////////////////////////////////

float CMonoSynthRenderer::m_arrayLin_0_Max[128];
float CMonoSynthRenderer::m_arrayLin_Min_Max[128];
float CMonoSynthRenderer::m_arrayLog_0_Max[128];
float CMonoSynthRenderer::m_arrayLog_Min_Max[128];
float CMonoSynthRenderer::m_arrayNote[128];

static rtems_signed32 GetAttenuation_dB(BYTE byValue)
{
	float dB;

	if (byValue < 27)
		dB = -90.0f + (float) 40.0f*byValue/27.0f;
	else
		dB = -50.0f + (float) 50.0f*(byValue-27)/100.0f;

	return float_to_fix(powf(10.0f, dB/20.0f));
}

//////////////////////////////////////////////////////////////////////////////

BOOL CMonoSynthRenderer::Initialize(void)
{
	if (!m_dsp.Create(dspCode))
		return FALSE;

	const float fBase = expf(log(LOG_SLOPE + 1.0f) / 127.0f);
	int i;

	for (i=0; i<128 ;i++)
	{
		m_arrayLin_0_Max[i] = i / 127.0f;
		m_arrayLin_Min_Max[i] = (m_arrayLin_0_Max[i] - 0.5f) * 2.0f;
		m_arrayLog_0_Max[i] = (powf(fBase, i) - 1.0f) / LOG_SLOPE;
		m_arrayNote[i] = 220.0f*powf(2.0f, (double) (i-57)/12.0f);
	}

	for (i=0; i<64 ;i++)
	{
		m_arrayLog_Min_Max[63-i] = -m_arrayLog_0_Max[2*i+1];
		m_arrayLog_Min_Max[64+i] = m_arrayLog_0_Max[2*i+1];
	}

	m_byNoteActive = 64 + 0x80;
	m_nPitchBend = 0;
	
	m_paramPotMasterVolume.Initialize(kPotMasterVolume);
	m_paramPotController1.Initialize(kPotController1);
	m_paramPotController2.Initialize(kPotController2);
	m_paramPotController3.Initialize(kPotController3);

	m_paramGlobalInputThru.Initialize(kGlobalInputThru);
	m_paramGlobalMidiChannel.Initialize(kGlobalMidiChannel);
	m_paramGlobalMidiThru.Initialize(kGlobalMidiThru);
	m_paramGlobalMidiDeviceID.Initialize(kGlobalMidiDeviceID);

	m_paramOsc1Waveform.Initialize(kOsc1Waveform);
	m_paramOsc1Transpose.Initialize(kOsc1Transpose);
	m_paramOsc1Volume.Initialize(kOsc1Volume);

	m_paramOsc2Waveform.Initialize(kOsc2Waveform);
	m_paramOsc2Transpose.Initialize(kOsc2Transpose);
	m_paramOsc2Detune.Initialize(kOsc2Detune);
	m_paramOsc2Volume.Initialize(kOsc2Volume);

	m_paramMixNoiseVolume.Initialize(kMixNoiseVolume);
	m_paramMixInputVolume.Initialize(kMixInputVolume);

	m_paramFilterCutoff.Initialize(kFilterCutoff);
	m_paramFilterResonance.Initialize(kFilterResonance);
	m_paramFilterEnvDepth.Initialize(kFilterEnvDepth);
	m_paramFilterKeyDepth.Initialize(kFilterKeyDepth);

	m_paramFilterEnvAttack.Initialize(kFilterEnvAttack);
	m_paramFilterEnvDecay.Initialize(kFilterEnvDecay);
	m_paramFilterEnvSustain.Initialize(kFilterEnvSustain);
	m_paramFilterEnvRelease.Initialize(kFilterEnvRelease);

	m_paramAmpEnvDepth.Initialize(kAmpEnvDepth);
	m_paramAmpKeyDepth.Initialize(kAmpKeyDepth);

	m_paramAmpEnvAttack.Initialize(kAmpEnvAttack);
	m_paramAmpEnvDecay.Initialize(kAmpEnvDecay);
	m_paramAmpEnvSustain.Initialize(kAmpEnvSustain);
	m_paramAmpEnvRelease.Initialize(kAmpEnvRelease);

	m_paramDelayLeftLevel.Initialize(kDelayLeftLevel, 7);
	m_paramDelayLeftTime.Initialize(kDelayLeftTime, 68);
	m_paramDelayLeftFeedback.Initialize(kDelayLeftFeedback, 34);
	m_paramDelayRightLevel.Initialize(kDelayRightLevel, 15);
	m_paramDelayRightTime.Initialize(kDelayRightTime, 51);
	m_paramDelayRightFeedback.Initialize(kDelayRightFeedback, 32);

	m_paramPotControllerAssign1.Initialize(kPotControllerAssign1);
	m_paramPotControllerAssign2.Initialize(kPotControllerAssign2);
	m_paramPotControllerAssign3.Initialize(kPotControllerAssign3);

	m_dsp.SendMsgFillBlockX(DSPX_Delay1_Table, DSPK_DELAY_TABLE_LEN, 0);
	m_dsp.SendMsgFillBlockX(DSPX_Delay2_Table, DSPK_DELAY_TABLE_LEN, 0);
	m_dsp.SendMsgWriteWordX(DSPX_Flags, 1);

	return TRUE;
}

void CMonoSynthRenderer::EnableProcessing(BOOL bEnable)
{
	m_dsp.SendMsgFillBlockX(DSPX_Delay1_Table, DSPK_DELAY_TABLE_LEN, 0);
	m_dsp.SendMsgFillBlockX(DSPX_Delay2_Table, DSPK_DELAY_TABLE_LEN, 0);
	m_dsp.SendMsgWriteWordX(DSPX_Flags, bEnable ? 1 : 0);
}

//////////////////////////////////////////////////////////////////////////////

void CMonoSynthRenderer::ChangeOscWaveform(DWORD dwLabelAddr1, DWORD dwLabelAddr2, BYTE byWaveform)
{
	static DWORD array[] =
	{
		DSPX_SineWave,
		DSPX_TriangleWave,
		DSPX_SawtoothWave,
		DSPX_SquareWave,
	};
	ASSERT(byWaveform < sizeof(array)/sizeof(DWORD *));
	
	m_dsp.SendMsgWriteWordX(dwLabelAddr1, array[byWaveform]);
	m_dsp.SendMsgWriteWordX(dwLabelAddr2, array[byWaveform]);
}

void CMonoSynthRenderer::ChangeOscFrequency(DWORD dwLabelAddr, BYTE byNote, BYTE byTranspose, BYTE byDetune, short nPitchBend)
{
	int nNote = byNote + byTranspose - 64;
	
	if (nNote < 0)
		nNote = 0;
	else if (nNote > 127)
		nNote = 127;

	float fInc = m_arrayNote[nNote] * DSPK_WAVE_TABLE_LEN / kSampleRate;
	static const float fMaxBend = powf(2.0f, 8.0f/12.0f);

	if (nPitchBend > 0)
		fInc *= 1.0f + (fMaxBend - 1.0f)*nPitchBend/8191.0f;
	else if (nPitchBend < 0)
		fInc *= 1.0f + (1.0f - 1.0f/fMaxBend)*nPitchBend/8192.0f;
		
	static const float fMaxDetune = powf(2.0f, 1.0f/12.0f);

	fInc *= 1.0f + (fMaxDetune - 1.0f) * byDetune / 127.0f;

	int iInc = (int) floor(fInc);
	rtems_signed32 data[2];

	fInc -= iInc;
	
	data[0] = (rtems_signed32) iInc;
	data[1] = float_to_fix(fInc) << 1;

	m_dsp.SendMsgWriteBlockX(dwLabelAddr, 2, data);
}

void CMonoSynthRenderer::ChangeOscVolume(DWORD dwLabelAddr, BYTE byVolume)
{
	m_dsp.SendMsgWriteWordX(dwLabelAddr, GetAttenuation_dB(byVolume));
}

void CMonoSynthRenderer::ChangeFilterCutoff(DWORD dwLabelAddr, BYTE byCutoff)
{
	// Cutoff values close to 0.0 make the filter unstable
	// and generates clicks to the output signal
	float fValue = 0.0217f+m_arrayLin_0_Max[byCutoff];
	
	m_dsp.SendMsgWriteWordX(dwLabelAddr, float_to_fix(fValue));
}

void CMonoSynthRenderer::ChangeFilterResonance(DWORD dwLabelAddr, BYTE byResonance)
{
	float fValue = m_arrayLin_0_Max[byResonance];

	m_dsp.SendMsgWriteWordX(dwLabelAddr, float_to_fix(fValue));
}

void CMonoSynthRenderer::ChangeEnvStateAttack(DWORD dwLabelAddr, DWORD dwEnvTableAddr)
{
	rtems_signed32 data[4];
	
	data[0] = 0;
	data[1] = 0;
	data[2] = 0;
	data[3] = dwEnvTableAddr;

	m_dsp.SendMsgWriteBlockX(dwLabelAddr, 4, data);
}

void CMonoSynthRenderer::ChangeEnvStateRelease(DWORD dwLabelAddr, DWORD dwEnvTableAddr)
{
	rtems_signed32 data[4];

	data[0] = 0;
	data[1] = 0;
	data[2] = 0;
	data[3] = dwEnvTableAddr+9;
	
	m_dsp.SendMsgWriteBlockX(dwLabelAddr, 4, data);
}

void CMonoSynthRenderer::ChangeEnvScale(DWORD dwLabelAddr, BYTE byVelocity, BYTE byEnvDepth, BYTE byKeyDepth)
{
	float fMax = 1.0f;
	float fMin = 1.0f - m_arrayLin_0_Max[byKeyDepth];
	float fValue = m_arrayLog_0_Max[byEnvDepth];

	fValue *= fMin + m_arrayLog_0_Max[byVelocity]*(fMax - fMin);

	m_dsp.SendMsgWriteWordX(dwLabelAddr, float_to_fix(fValue));
}

void CMonoSynthRenderer::ChangeEnvAttack(DWORD dwLabelAddr, BYTE byAttack)
{
	float fAttack = 100.0f * kSampleRate * m_arrayLog_0_Max[byAttack];
	rtems_signed32 data[15];

	if (fAttack < (8.0f * DSPK_CHUNK_SIZE))
		fAttack = (8.0f * DSPK_CHUNK_SIZE);

	data[0] = (rtems_signed32) (fAttack / (float) DSPK_CHUNK_SIZE);
	data[1] = 0x7FFFFF;
	data[2] = float_to_fix(7.9f / fAttack);
	
	m_dsp.SendMsgWriteBlockY(dwLabelAddr, 3, data);
}

void CMonoSynthRenderer::ChangeEnvDecay(DWORD dwLabelAddr, BYTE byDecay, BYTE bySustain)
{
	float fDecay = 5.0f * kSampleRate * m_arrayLog_0_Max[byDecay];
	float fSustain = m_arrayLog_0_Max[bySustain];
	rtems_signed32 data[15];

	if (fDecay < (8.0f * DSPK_CHUNK_SIZE))
		fDecay = (8.0f * DSPK_CHUNK_SIZE);

	data[3] = (rtems_signed32) (fDecay / (float) DSPK_CHUNK_SIZE);
	data[4] = float_to_fix(fSustain);
	data[5] = float_to_fix(7.9f / fDecay);
	data[6] = 0x800000;
	data[7] = 0;
	data[8] = 0;

	m_dsp.SendMsgWriteBlockY(dwLabelAddr+3, 9-3, data+3);
}

void CMonoSynthRenderer::ChangeEnvRelease(DWORD dwLabelAddr, BYTE byRelease)
{
	float fRelease = 15.0f * kSampleRate * m_arrayLog_0_Max[byRelease];
	rtems_signed32 data[15];

	if (fRelease < (8.0f * DSPK_CHUNK_SIZE))
		fRelease = (8.0f * DSPK_CHUNK_SIZE);

	data[9] = 0x7FFFFF;
	data[10] = 0;
	data[11] = float_to_fix(7.9f / fRelease);
	data[12] = 0x800000;
	data[13] = 0;
	data[14] = 0;

	m_dsp.SendMsgWriteBlockY(dwLabelAddr+9, 15-9, data+9);
}

void CMonoSynthRenderer::ChangeDelayLevel(DWORD dwLabelAddr, BYTE byTime)
{
	float fValue = m_arrayLin_0_Max[byTime];
	
	m_dsp.SendMsgWriteWordX(dwLabelAddr, float_to_fix(fValue));
}

void CMonoSynthRenderer::ChangeDelayTime(DWORD dwLabelAddr, BYTE byTime)
{
	float fValue = m_arrayLin_0_Max[byTime];
	
	m_dsp.SendMsgWriteWordX(dwLabelAddr, float_to_fix(fValue));
}

void CMonoSynthRenderer::ChangeDelayFeedback(DWORD dwLabelAddr, BYTE byFeedback)
{
	float fValue = m_arrayLin_0_Max[byFeedback];
	
	m_dsp.SendMsgWriteWordX(dwLabelAddr, float_to_fix(fValue));
}

//////////////////////////////////////////////////////////////////////////////

void CMonoSynthRenderer::DoMidiKeyOn(BYTE byNote, BYTE byVel)
{
	if (byVel == 0)
	{
		DoMidiKeyOff(byNote, byVel);
		return;
	}
	
	m_byNoteActive = byNote;

	ChangeOscFrequency(DSPX_Osc1_IncInt, m_byNoteActive, m_paramOsc1Transpose.GetValue(), 0, m_nPitchBend);
	ChangeOscFrequency(DSPX_Osc2_IncInt, m_byNoteActive, m_paramOsc2Transpose.GetValue(), m_paramOsc2Detune.GetValue(), m_nPitchBend);
	ChangeEnvScale(DSPX_Scale1_ScaleValue, byVel, m_paramFilterEnvDepth.GetValue(), m_paramFilterKeyDepth.GetValue());
	ChangeEnvScale(DSPX_Scale2_ScaleValue, byVel, m_paramAmpEnvDepth.GetValue(), m_paramAmpKeyDepth.GetValue());

	ChangeEnvStateAttack(DSPX_Env1_Dur, DSPY_Env1_Table);
	ChangeEnvStateAttack(DSPX_Env2_Dur, DSPY_Env2_Table);

	m_dsp.SendMsgUpdate();
}

void CMonoSynthRenderer::DoMidiKeyOff(BYTE byNote, BYTE byVel)
{
	if (byNote == m_byNoteActive)
	{
		ChangeEnvStateRelease(DSPX_Env1_Dur, DSPY_Env1_Table);
		ChangeEnvStateRelease(DSPX_Env2_Dur, DSPY_Env2_Table);
		m_dsp.SendMsgUpdate();

		m_byNoteActive += 0x80;
	}
}

void CMonoSynthRenderer::DoMidiPitchWheel(short value)
{
	m_nPitchBend = value;

	ChangeOscFrequency(DSPX_Osc1_IncInt, m_byNoteActive & 0x7F, m_paramOsc1Transpose.GetValue(), 0, m_nPitchBend);
	ChangeOscFrequency(DSPX_Osc2_IncInt, m_byNoteActive & 0x7F, m_paramOsc2Transpose.GetValue(), m_paramOsc2Detune.GetValue(), m_nPitchBend);
	m_dsp.SendMsgUpdate();
}

//////////////////////////////////////////////////////////////////////////////

void CMonoSynthRenderer::Update(WORD wID)
{
	switch (wID)
	{
		case kPotMasterVolume:
			m_dsp.SendMsgWriteWordX(DSPX_MasterVolume, GetAttenuation_dB(m_paramPotMasterVolume.GetValue()));
			m_dsp.SendMsgUpdate();
			break;

		case kGlobalInputThru:
			m_dsp.SendMsgWriteWordX(DSPX_InputThru, GetAttenuation_dB(m_paramGlobalInputThru.GetValue()));
			m_dsp.SendMsgUpdate();
			break;

		case kGlobalMidiChannel:
		case kGlobalMidiThru:
		case kGlobalMidiDeviceID:
			break;

		case kOsc1Waveform:
			ChangeOscWaveform(DSPX_Osc1_TableAddr , DSPX_Osc1_PhaseInt, m_paramOsc1Waveform.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kOsc1Transpose:
			ChangeOscFrequency(DSPX_Osc1_IncInt, m_byNoteActive & 0x7F, m_paramOsc1Transpose.GetValue(), 0, m_nPitchBend);
			m_dsp.SendMsgUpdate();
			break;

		case kOsc1Volume:
			ChangeOscVolume(DSPX_Osc1_Amplitude, m_paramOsc1Volume.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kOsc2Waveform:
			ChangeOscWaveform(DSPX_Osc2_TableAddr , DSPX_Osc2_PhaseInt, m_paramOsc2Waveform.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kOsc2Transpose:
		case kOsc2Detune:
			ChangeOscFrequency(DSPX_Osc2_IncInt, m_byNoteActive & 0x7F, m_paramOsc2Transpose.GetValue(), m_paramOsc2Detune.GetValue(), m_nPitchBend);
			m_dsp.SendMsgUpdate();
			break;

		case kOsc2Volume:
			ChangeOscVolume(DSPX_Osc2_Amplitude, m_paramOsc2Volume.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kMixNoiseVolume:
			ChangeOscVolume(DSPX_Mixer_ScaleValue2, m_paramMixNoiseVolume.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kMixInputVolume:
			ChangeOscVolume(DSPX_Mixer_ScaleValue1, m_paramMixInputVolume.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kFilterCutoff:
			ChangeFilterCutoff(DSPX_Addk_Constant, m_paramFilterCutoff.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kFilterResonance:
			ChangeFilterResonance(DSPX_Filter_Res, m_paramFilterResonance.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kFilterEnvDepth:
		case kFilterKeyDepth:
			break;

		case kFilterEnvAttack:
			ChangeEnvAttack(DSPY_Env1_Table, m_paramFilterEnvAttack.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kFilterEnvDecay:
		case kFilterEnvSustain:
			ChangeEnvDecay(DSPY_Env1_Table, m_paramFilterEnvDecay.GetValue(), m_paramFilterEnvSustain.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kFilterEnvRelease:
			ChangeEnvRelease(DSPY_Env1_Table, m_paramFilterEnvRelease.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kAmpEnvDepth:
		case kAmpKeyDepth:
			break;

		case kAmpEnvAttack:
			ChangeEnvAttack(DSPY_Env2_Table, m_paramAmpEnvAttack.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kAmpEnvDecay:
		case kAmpEnvSustain:
			ChangeEnvDecay(DSPY_Env2_Table, m_paramAmpEnvDecay.GetValue(), m_paramAmpEnvSustain.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kAmpEnvRelease:
			ChangeEnvRelease(DSPY_Env2_Table, m_paramAmpEnvRelease.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kDelayLeftLevel:
			ChangeDelayLevel(DSPX_Delay1_Level, m_paramDelayLeftLevel.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kDelayLeftTime:
			ChangeDelayTime(DSPX_Delay1_Size, m_paramDelayLeftTime.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kDelayLeftFeedback:
			ChangeDelayFeedback(DSPX_Delay1_Feedback, m_paramDelayLeftFeedback.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kDelayRightLevel:
			ChangeDelayLevel(DSPX_Delay2_Level, m_paramDelayRightLevel.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kDelayRightTime:
			ChangeDelayTime(DSPX_Delay2_Size, m_paramDelayRightTime.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kDelayRightFeedback:
			ChangeDelayFeedback(DSPX_Delay2_Feedback, m_paramDelayRightFeedback.GetValue());
			m_dsp.SendMsgUpdate();
			break;

		case kPotController1:
		case kPotController2:
		case kPotController3:
		case kPotControllerAssign1:
		case kPotControllerAssign2:
		case kPotControllerAssign3:
			break;

		default:
			TRACE("ERROR: Invalid parameter to update received in CMonoSynthRenderer::Update (%d)\n", wID);
			ASSERT(FALSE);
	}
}

