/*******************************************************************************
 * C H A M E L E O N                                                           *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/monosynth/MonoSynthRenderer.h       $
 *     $Date:: 15/05/02 20:51                                                  $
 * $Revision:: 2                                                               $
 *-----------------------------------------------------------------------------*
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#ifndef __MONOSYNTH_RENDERER_H__
#define __MONOSYNTH_RENDERER_H__

#include "dsp.h"
#include "MonoSynthParameters.h"

//////////////////////////////////////////////////////////////////////////////

class CParameter_PotMasterVolume : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 0)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Master Volume"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_PotController : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 0)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Realtime Controller"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_InputThru : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 0)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Input Thru"; }
	virtual const char *GetShortMenuName(void) { return "InputThru"; }
	virtual const char *GetLongMenuName(void) { return "Input Thru"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_MidiChannel : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 0)
	{
		CParameter::Initialize(wID, byDefValue, 0, 15);
	}
	
	virtual const char *GetName(void) { return "MIDI Channel"; }
	virtual const char *GetShortMenuName(void) { return "MidiChannel"; }
	virtual const char *GetLongMenuName(void) { return "MIDI Channel"; }

	virtual void GetValueText(char *strValueText)
	{
		ASSERT(strValueText);

		sprintf(strValueText, "%02d", GetValue() + 1);
	}
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_MidiThru : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 1)
	{
		CParameter::Initialize(wID, byDefValue, 0, 1);
	}
	
	virtual const char *GetName(void) { return "MIDI Thru"; }
	virtual const char *GetShortMenuName(void) { return "MidiThru"; }
	virtual const char *GetLongMenuName(void) { return "MIDI Thru"; }

	virtual void GetValueText(char *strValueText)
	{
		ASSERT(strValueText);
		static char *arrayText[] = { "Off", " On" };

		sprintf(strValueText, "%3s", arrayText[GetValue()]);
	}
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_MidiDeviceID : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 127)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "MIDI Device Identifier"; }
	virtual const char *GetShortMenuName(void) { return "Midi ID"; }
	virtual const char *GetLongMenuName(void) { return "MIDI Device ID"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_OscWaveform : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 2)
	{
		CParameter::Initialize(wID, byDefValue, 0, 3);
	}
	
	virtual const char *GetName(void) { return "Oscillator Waveform"; }
	virtual const char *GetShortMenuName(void) { return "Waveform"; }
	virtual const char *GetLongMenuName(void) { return "Osc. Waveform"; }

	virtual void GetValueText(char *strValueText)
	{
		ASSERT(strValueText);
		static char *arrayText[] = { "Sine", " Tri", " Saw", " Sqr" };

		sprintf(strValueText, "%4s", arrayText[GetValue()]);
	}
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_OscTranspose : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 64-12)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Oscillator Transpose"; }
	virtual const char *GetShortMenuName(void) { return "Transpose"; }
	virtual const char *GetLongMenuName(void) { return "Osc. Transpose"; }

	virtual void GetValueText(char *strValueText)
	{
		ASSERT(strValueText);
		sprintf(strValueText, "%+3d", (int) GetValue() - 0x40);
	}
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_OscDetune : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 10)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Oscillator Detune"; }
	virtual const char *GetShortMenuName(void) { return "Detune"; }
	virtual const char *GetLongMenuName(void) { return "Osc. Detune"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_OscVolume : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 115)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void)
	{
		if (GetID() == kOsc1Volume)
			return "Oscillator1 Volume";
		else
			return "Oscillator2 Volume";
	}

	virtual const char *GetShortMenuName(void)
	{
		if (GetID() == kOsc1Volume)
			return "Osc1Vol";
		else
			return "Osc2Vol";
	}

	virtual const char *GetLongMenuName(void)
	{
		if (GetID() == kOsc1Volume)
			return "Osc1 Volume";
		else
			return "Osc2 Volume";
	}
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_MixNoiseVolume : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 0)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Noise Volume"; }
	virtual const char *GetShortMenuName(void) { return "NoiseVol"; }
	virtual const char *GetLongMenuName(void) { return "Noise Volume"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_MixInputVolume : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 0)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Input Volume"; }
	virtual const char *GetShortMenuName(void) { return "InputVol"; }
	virtual const char *GetLongMenuName(void) { return "Input Volume"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_FilterCutoff : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 127)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Filter Cutoff"; }
	virtual const char *GetShortMenuName(void) { return "Cutoff"; }
	virtual const char *GetLongMenuName(void) { return "Filter Cutoff"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_FilterResonance : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 0)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Filter Resonance"; }
	virtual const char *GetShortMenuName(void) { return "Resonance"; }
	virtual const char *GetLongMenuName(void) { return "Resonance"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_FilterEnvDepth : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 0)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Filter Envelope Depth"; }
	virtual const char *GetShortMenuName(void) { return "EnvDepth"; }
	virtual const char *GetLongMenuName(void) { return "Filt. Env. Depth"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_FilterKeyDepth : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 0)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Filter Key Depth"; }
	virtual const char *GetShortMenuName(void) { return "KeyDepth"; }
	virtual const char *GetLongMenuName(void) { return "Filt. Key. Depth"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_FilterEnvAttack : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 0)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Filter Envelope Attack"; }
	virtual const char *GetShortMenuName(void) { return "Attack"; }
	virtual const char *GetLongMenuName(void) { return "Filt.Env.Attack"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_FilterEnvDecay : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 0)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Filter Envelope Decay"; }
	virtual const char *GetShortMenuName(void) { return "Decay"; }
	virtual const char *GetLongMenuName(void) { return "Filt.Env.Decay"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_FilterEnvSustain : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 127)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Filter Envelope Sustain"; }
	virtual const char *GetShortMenuName(void) { return "Sustain"; }
	virtual const char *GetLongMenuName(void) { return "Filt.Env.Sustain"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_FilterEnvRelease : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 20)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Filter Envelope Release"; }
	virtual const char *GetShortMenuName(void) { return "Release"; }
	virtual const char *GetLongMenuName(void) { return "Filt.Env.Release"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_AmpEnvDepth : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 127)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Amplifier Envelope Depth"; }
	virtual const char *GetShortMenuName(void) { return "EnvDepth"; }
	virtual const char *GetLongMenuName(void) { return "Amp. Env. Depth"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_AmpKeyDepth : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 90)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Amplifier Key Depth"; }
	virtual const char *GetShortMenuName(void) { return "KeyDepth"; }
	virtual const char *GetLongMenuName(void) { return "Amp. Key. Depth"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_AmpEnvAttack : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 0)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Amplifier Envelope Attack"; }
	virtual const char *GetShortMenuName(void) { return "Attack"; }
	virtual const char *GetLongMenuName(void) { return "Amp.Env.Attack"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_AmpEnvDecay : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 0)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Amplifier Envelope Decay"; }
	virtual const char *GetShortMenuName(void) { return "Decay"; }
	virtual const char *GetLongMenuName(void) { return "Amp.Env.Decay"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_AmpEnvSustain : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 127)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Amplifier Envelope Sustain"; }
	virtual const char *GetShortMenuName(void) { return "Sustain"; }
	virtual const char *GetLongMenuName(void) { return "Amp.Env.Sustain"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_AmpEnvRelease : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 40)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Amplifier Envelope Release"; }
	virtual const char *GetShortMenuName(void) { return "Release"; }
	virtual const char *GetLongMenuName(void) { return "Amp.Env.Release"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_DelayLevel : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 32)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Delay Level Send"; }
	virtual const char *GetShortMenuName(void) { return "DelayLev"; }
	virtual const char *GetLongMenuName(void) { return "Delay Level"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_DelayTime : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 48)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Delay Time"; }
	virtual const char *GetShortMenuName(void) { return "DelayTime"; }
	virtual const char *GetLongMenuName(void) { return "Delay Time"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_DelayFeedback : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = 24)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Delay Feedback"; }
	virtual const char *GetShortMenuName(void) { return "DelayFb"; }
	virtual const char *GetLongMenuName(void) { return "Delay Feedback"; }
};

//////////////////////////////////////////////////////////////////////////////

class CParameter_PotControllerAssign : public CParameter
{
public:
	void Initialize(WORD wID, BYTE byDefValue = kParameterNone)
	{
		CParameter::Initialize(wID, byDefValue, 0, 127);
	}
	
	virtual const char *GetName(void) { return "Realtime Controller Assign"; }
};

//////////////////////////////////////////////////////////////////////////////
class CMonoSynthRenderer
{
private:
	CDsp m_dsp;

	static float m_arrayLin_0_Max[128];
	static float m_arrayLin_Min_Max[128];
	static float m_arrayLog_0_Max[128];
	static float m_arrayLog_Min_Max[128];
	static float m_arrayNote[128];

private:
	BYTE m_byNoteActive;
	short m_nPitchBend;
	
private:
	void ChangeOscWaveform(DWORD dwLabelAddr1, DWORD dwLabelAddr2, BYTE byWaveform);
	void ChangeOscFrequency(DWORD dwLabelAddr, BYTE byNote, BYTE byTranspose, BYTE byDetune, short nPitchBend);
	void ChangeOscVolume(DWORD dwLabelAddr, BYTE byVolume);
	void ChangeFilterCutoff(DWORD dwLabelAddr, BYTE byCutoff);
	void ChangeFilterResonance(DWORD dwLabelAddr, BYTE byResonance);
	void ChangeEnvScale(DWORD dwLabelAddr, BYTE byVelocity, BYTE byEnvDepth, BYTE byKeyDepth);
	void ChangeEnvAttack(DWORD dwLabelAddr, BYTE byAttack);
	void ChangeEnvDecay(DWORD dwLabelAddr, BYTE byDecay, BYTE bySustain);
	void ChangeEnvRelease(DWORD dwLabelAddr, BYTE byRelease);
	void ChangeDelayLevel(DWORD dwLabelAddr, BYTE byTime);
	void ChangeDelayTime(DWORD dwLabelAddr, BYTE byTime);
	void ChangeDelayFeedback(DWORD dwLabelAddr, BYTE byFeedback);
	void ChangeEnvStateAttack(DWORD dwLabelAddr, DWORD dwEnvTableAddr);
	void ChangeEnvStateRelease(DWORD dwLabelAddr, DWORD dwEnvTableAddr);

private:
	CParameter_PotMasterVolume 	m_paramPotMasterVolume;
	CParameter_PotController	m_paramPotController1;
	CParameter_PotController	m_paramPotController2;
	CParameter_PotController	m_paramPotController3;

	CParameter_InputThru		m_paramGlobalInputThru;
	CParameter_MidiChannel		m_paramGlobalMidiChannel;
	CParameter_MidiThru		m_paramGlobalMidiThru;
	CParameter_MidiDeviceID		m_paramGlobalMidiDeviceID;

	CParameter_OscWaveform		m_paramOsc1Waveform;
	CParameter_OscTranspose		m_paramOsc1Transpose;
	CParameter_OscVolume		m_paramOsc1Volume;

	CParameter_OscWaveform		m_paramOsc2Waveform;
	CParameter_OscTranspose		m_paramOsc2Transpose;
	CParameter_OscDetune		m_paramOsc2Detune;
	CParameter_OscVolume		m_paramOsc2Volume;

	CParameter_MixNoiseVolume	m_paramMixNoiseVolume;
	CParameter_MixInputVolume	m_paramMixInputVolume;

	CParameter_FilterCutoff		m_paramFilterCutoff;
	CParameter_FilterResonance	m_paramFilterResonance;
	CParameter_FilterEnvDepth	m_paramFilterEnvDepth;
	CParameter_FilterKeyDepth	m_paramFilterKeyDepth;

	CParameter_FilterEnvAttack	m_paramFilterEnvAttack;
	CParameter_FilterEnvDecay	m_paramFilterEnvDecay;
	CParameter_FilterEnvSustain	m_paramFilterEnvSustain;
	CParameter_FilterEnvRelease	m_paramFilterEnvRelease;

	CParameter_AmpEnvDepth		m_paramAmpEnvDepth;
	CParameter_AmpKeyDepth		m_paramAmpKeyDepth;

	CParameter_AmpEnvAttack		m_paramAmpEnvAttack;
	CParameter_AmpEnvDecay		m_paramAmpEnvDecay;
	CParameter_AmpEnvSustain	m_paramAmpEnvSustain;
	CParameter_AmpEnvRelease	m_paramAmpEnvRelease;
	
	CParameter_DelayLevel		m_paramDelayLeftLevel;
	CParameter_DelayTime		m_paramDelayLeftTime;
	CParameter_DelayFeedback	m_paramDelayLeftFeedback;
	CParameter_DelayLevel		m_paramDelayRightLevel;
	CParameter_DelayTime		m_paramDelayRightTime;
	CParameter_DelayFeedback	m_paramDelayRightFeedback;

	CParameter_PotControllerAssign	m_paramPotControllerAssign1;
	CParameter_PotControllerAssign	m_paramPotControllerAssign2;
	CParameter_PotControllerAssign	m_paramPotControllerAssign3;

public:
	BOOL Initialize(void);
	void EnableProcessing(BOOL bEnable);
	void Update(WORD wID);

	void DoMidiKeyOn(BYTE byNote, BYTE byVel);
	void DoMidiKeyOff(BYTE byNote, BYTE byVel);
	void DoMidiPitchWheel(short value);
};

//////////////////////////////////////////////////////////////////////////////

#endif
