/*******************************************************************************
 * C H A M E L E O N                                                           *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/MONOSYNTH/dsp.cpp                   $
 *     $Date:: 13/05/02 20:11                                                  $
 * $Revision:: 1                                                               $
 *-----------------------------------------------------------------------------*
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#include <string.h>
#include <rtems.h>
#include <chameleon.h>

#include "dsp.h"
#include "dsp/dsp_code.h"

//////////////////////////////////////////////////////////////////////////////
#define DSP_MAX_MESSAGE_LEN	100
#define DSP_MAX_MESSAGE_SIZE	(DSP_MAX_MESSAGE_LEN*sizeof(rtems_signed32))
#define DSP_MAX_MESSAGE_COUNT	250
//////////////////////////////////////////////////////////////////////////////
enum
{
	kDspMsgCommand = 0,
	kDspMaxMessages
};
//////////////////////////////////////////////////////////////////////////////

struct CDspMsg : public CMessage
{
	CDspMsg(void)
	{
		dwMsg = kDspMsgCommand;
	}
	
	virtual rtems_signed32 *GetDataWritePtr(void) = 0;
	virtual const DWORD GetDataWriteLen(void) = 0;

	virtual rtems_signed32 *GetDataReadPtr(void) { return NULL; }
	virtual const DWORD GetDataReadLen(void) { return 0; }
};

struct CDspMsg_Update : public CDspMsg
{
private:
	rtems_signed32 data;

public:
	CDspMsg_Update(void)
	{
		data = 0;	// CommandAddr = 0 is for update
	}
	
	virtual rtems_signed32 *GetDataWritePtr(void) { return &data; }
	virtual const DWORD GetDataWriteLen(void) { return 1; }
	virtual const DWORD GetSize(void) { return sizeof(CDspMsg_Update); }
};

class CDspMsg_WriteBlock : public CDspMsg
{
private:
	rtems_signed32 data[DSP_MAX_MESSAGE_LEN-5];

public:
	CDspMsg_WriteBlock(DWORD dwDspCommand, DWORD dwAddr, DWORD dwLen, rtems_signed32 *pData) : CDspMsg()
	{
		ASSERT(GetSize() <= DSP_MAX_MESSAGE_SIZE);
		ASSERT(dwLen <= DSP_MAX_MESSAGE_LEN-5);
		data[0] = dwDspCommand;
		data[1] = dwAddr;
		data[2] = dwLen;
		memcpy(data+3, pData, dwLen*sizeof(rtems_signed32));
	}
	
	virtual rtems_signed32 *GetDataWritePtr(void) { return data; }
	virtual const DWORD GetDataWriteLen(void) { return 3 + data[2]; }
	virtual const DWORD GetSize(void) { return sizeof(CDspMsg_WriteBlock); }
};

struct CDspMsg_WriteBlockP : public CDspMsg_WriteBlock
{
	CDspMsg_WriteBlockP(DWORD dwAddr, DWORD dwLen, rtems_signed32 *pData) : CDspMsg_WriteBlock(DSPP_CommandWriteBlockP, dwAddr, dwLen, pData) { }
};

struct CDspMsg_WriteBlockX : public CDspMsg_WriteBlock
{
	CDspMsg_WriteBlockX(DWORD dwAddr, DWORD dwLen, rtems_signed32 *pData) : CDspMsg_WriteBlock(DSPP_CommandWriteBlockX, dwAddr, dwLen, pData) { }
};

struct CDspMsg_WriteBlockY : public CDspMsg_WriteBlock
{
	CDspMsg_WriteBlockY(DWORD dwAddr, DWORD dwLen, rtems_signed32 *pData) : CDspMsg_WriteBlock(DSPP_CommandWriteBlockY, dwAddr, dwLen, pData) { }
};

class CDspMsg_WriteWord : public CDspMsg
{
private:
	rtems_signed32 data[3];

public:
	CDspMsg_WriteWord(DWORD dwDspCommand, DWORD dwAddr, DWORD dwWord) : CDspMsg()
	{
		ASSERT(GetSize() <= DSP_MAX_MESSAGE_SIZE);
		data[0] = dwDspCommand;
		data[1] = dwAddr;
		data[2] = dwWord;
	}
	
	virtual rtems_signed32 *GetDataWritePtr(void) { return data; }
	virtual const DWORD GetDataWriteLen(void) { return 3; }
	virtual const DWORD GetSize(void) { return sizeof(CDspMsg_WriteWord); }
};

struct CDspMsg_WriteWordP : public CDspMsg_WriteWord
{
	CDspMsg_WriteWordP(DWORD dwAddr, rtems_signed32 dwWord) : CDspMsg_WriteWord(DSPP_CommandWriteWordP, dwAddr, dwWord) { }
};

struct CDspMsg_WriteWordX : public CDspMsg_WriteWord
{
	CDspMsg_WriteWordX(DWORD dwAddr, rtems_signed32 dwWord) : CDspMsg_WriteWord(DSPP_CommandWriteWordX, dwAddr, dwWord) { }
};

struct CDspMsg_WriteWordY : public CDspMsg_WriteWord
{
	CDspMsg_WriteWordY(DWORD dwAddr, rtems_signed32 dwWord) : CDspMsg_WriteWord(DSPP_CommandWriteWordY, dwAddr, dwWord) { }
};

class CDspMsg_CopyBlock : public CDspMsg
{
private:
	rtems_signed32 data[4];

public:
	CDspMsg_CopyBlock(DWORD dwDspCommand, DWORD dwAddrSrc, DWORD dwAddrDst, DWORD dwLen) : CDspMsg()
	{
		ASSERT(GetSize() <= DSP_MAX_MESSAGE_SIZE);
		data[0] = dwDspCommand;
		data[1] = dwAddrSrc;
		data[2] = dwAddrDst;
		data[3] = dwLen;
	}
	
	virtual rtems_signed32 *GetDataWritePtr(void) { return data; }
	virtual const DWORD GetDataWriteLen(void) { return 4; }
	virtual const DWORD GetSize(void) { return sizeof(CDspMsg_CopyBlock); }
};

struct CDspMsg_CopyBlockP : public CDspMsg_CopyBlock
{
	CDspMsg_CopyBlockP(DWORD dwAddrSrc, DWORD dwAddrDst, DWORD dwLen) : CDspMsg_CopyBlock(DSPP_CommandCopyBlockP, dwAddrSrc, dwAddrDst, dwLen) { }
};

class CDspMsg_FillBlock : public CDspMsg
{
private:
	rtems_signed32 data[4];

public:
	CDspMsg_FillBlock(DWORD dwDspCommand, DWORD dwAddr, DWORD dwLen, rtems_signed32 dwWord) : CDspMsg()
	{
		ASSERT(GetSize() <= DSP_MAX_MESSAGE_SIZE);
		data[0] = dwDspCommand;
		data[1] = dwAddr;
		data[2] = dwLen;
		data[3] = dwWord;
	}
	
	virtual rtems_signed32 *GetDataWritePtr(void) { return data; }
	virtual const DWORD GetDataWriteLen(void) { return 4; }
	virtual const DWORD GetSize(void) { return sizeof(CDspMsg_FillBlock); }
};

struct CDspMsg_FillBlockX : public CDspMsg_FillBlock
{
	CDspMsg_FillBlockX(DWORD dwAddr, DWORD dwLen, rtems_signed32 dwWord) : CDspMsg_FillBlock(DSPP_CommandFillBlockX, dwAddr, dwLen, dwWord) { }
};

struct CDspMsg_FillBlockY : public CDspMsg_FillBlock
{
	CDspMsg_FillBlockY(DWORD dwAddr, DWORD dwLen, rtems_signed32 dwWord) : CDspMsg_FillBlock(DSPP_CommandFillBlockY, dwAddr, dwLen, dwWord) { }
};

//////////////////////////////////////////////////////////////////////////////

void CDsp::SendMsgUpdate(void)
{
	CDspMsg_Update msg;
	if (!SendMessage(&msg))
		ASSERT(FALSE);
}

void CDsp::SendMsgWriteBlockP(DWORD dwAddr, DWORD dwLen, rtems_signed32 *pData)
{
	CDspMsg_WriteBlockP msg(dwAddr, dwLen, pData);
	if (!SendMessage(&msg))
		ASSERT(FALSE);
}

void CDsp::SendMsgWriteBlockX(DWORD dwAddr, DWORD dwLen, rtems_signed32 *pData)
{
	CDspMsg_WriteBlockX msg(dwAddr, dwLen, pData);
	if (!SendMessage(&msg))
		ASSERT(FALSE);
}

void CDsp::SendMsgWriteBlockY(DWORD dwAddr, DWORD dwLen, rtems_signed32 *pData)
{
	CDspMsg_WriteBlockY msg(dwAddr, dwLen, pData);
	if (!SendMessage(&msg))
		ASSERT(FALSE);
}

void CDsp::SendMsgWriteWordP(DWORD dwAddr, rtems_signed32 dwWord)
{
	CDspMsg_WriteWordP msg(dwAddr, dwWord);
	if (!SendMessage(&msg))
		ASSERT(FALSE);
}

void CDsp::SendMsgWriteWordX(DWORD dwAddr, rtems_signed32 dwWord)
{
	CDspMsg_WriteWordX msg(dwAddr, dwWord);
	if (!SendMessage(&msg))
		ASSERT(FALSE);
}

void CDsp::SendMsgWriteWordY(DWORD dwAddr, rtems_signed32 dwWord)
{
	CDspMsg_WriteWordY msg(dwAddr, dwWord);
	if (!SendMessage(&msg))
		ASSERT(FALSE);
}

void CDsp::SendMsgCopyBlockP(DWORD dwAddrSrc, DWORD dwAddrDst, DWORD dwLen)
{
	CDspMsg_CopyBlockP msg(dwAddrSrc, dwAddrDst, dwLen);
	if (!SendMessage(&msg))
		ASSERT(FALSE);
}

void CDsp::SendMsgFillBlockX(DWORD dwAddr, DWORD dwLen, rtems_signed32 dwWord)
{
	CDspMsg_FillBlockX msg(dwAddr, dwLen, dwWord);
	if (!SendMessage(&msg))
		ASSERT(FALSE);
}

void CDsp::SendMsgFillBlockY(DWORD dwAddr, DWORD dwLen, rtems_signed32 dwWord)
{
	CDspMsg_FillBlockY msg(dwAddr, dwLen, dwWord);
	if (!SendMessage(&msg))
		ASSERT(FALSE);
}

//////////////////////////////////////////////////////////////////////////////

BOOL CDsp::Create(const BYTE dspCode[])
{
	ASSERT(m_nDspRef == 0);

	TRACE("Booting the DSP...\n");
	m_nDspRef = dsp_init(1, dspCode);
	if (m_nDspRef < 0)
	{
		TRACE("ERROR: cannot init the DSP in CDsp::Create\n");
		return FALSE;
	}

	m_dwDataSentLen = 0;

	TRACE("Creating task for handling DSP messages...\n");

	if (!CMessageHandler::Create(
		rtems_build_name('D', 'S', 'P', 'T'), 10,
		rtems_build_name('D', 'S', 'P', 'Q'), DSP_MAX_MESSAGE_COUNT, DSP_MAX_MESSAGE_SIZE))
	{
		TRACE("ERROR: cannot create the message handler in CDsp::Create\n");
		return FALSE;
	}

	return TRUE;
}

void CDsp::OnMessage(CMessage *pMsg)
{
	ASSERT(m_nDspRef != 0);
	ASSERT(pMsg);
	ASSERT(pMsg->dwMsg == kDspMsgCommand);
	
	CDspMsg *pDspMsg = (CDspMsg *) pMsg;

#ifdef DEBUG
#if 0	// put #if 1 here if you want to see what we are sending to the DSP
	TRACE("Writting %ld words to the DSP\n", pDspMsg->GetDataWriteLen());
	rtems_signed32 *pData = pDspMsg->GetDataWritePtr();
	for (unsigned int i=0; i<pDspMsg->GetDataWriteLen() ;i++)
		TRACE("\t0x%06X\n", (*pData++) & 0xFFFFFF);
#endif
	m_dwDataSentLen += pDspMsg->GetDataWriteLen();
	if (m_dwDataSentLen >= DSPK_HOST_BUFFER_LEN)
	{
		TRACE("ERROR: HOST_BUFFER_LEN reached in CDsp::OnMessage\n");
		ASSERT(FALSE);
	}
#endif
	if (*pDspMsg->GetDataWritePtr() == 0)	// if command is update
	{
		m_dwDataSentLen = 0;
		if (!dsp_write_command(m_nDspRef, DSPP_VecHostCommandUpdate/2, FALSE))
		{
			TRACE("ERROR: Cannot write command to the DSP in CDsp::OnMessage\n");
			ASSERT(FALSE);
		}
	}
	else if (!dsp_write_data(m_nDspRef, pDspMsg->GetDataWritePtr(), pDspMsg->GetDataWriteLen()))
	{
		TRACE("ERROR: Cannot write data to the DSP in CDsp::OnMessage\n");
		ASSERT(FALSE);
	}
}

