;*******************************************************************************
; C H A M E L E O N   DSP Assembler file                                       *
;*******************************************************************************
; Copyright (C) 2001-2002 Soundart                                             *
; www.soundart-hot.com                                                         *
; support@soundart-hot.com                                                     *
;*******************************************************************************

	nolist
	page	255,0
	opt	MU,S,CC,CEX,MEX,MD
	list

	nolist
	include	"SDK\include\dsp\dsp_equ.asm"
	include "SDK\include\dsp\dma.asm"
	list

;*******************************************************************************
;		define	SIMULATOR		''
;		define	TEST_PERFORMANCE	''
;*******************************************************************************

CHUNK_SIZE	equ	16
HOST_BUFFER_LEN	equ	64

PARAM		macro LABEL,VALUE
LABEL:
		dc	VALUE
		endm

XMEM		macro LABEL,ADDR
LABEL:
		dc	ADDR
		endm

YMEM		macro LABEL,ADDR
LABEL:
		dc	ADDR
		endm

		define	CHUNK		'ds	CHUNK_SIZE'
		define	TABLE		'ds'

		define	R_X		'R0'
		define	N_X		'N0'
		define	R_Y		'R4'
		define	N_Y		'N4'

		define	BEGIN_LOOP	 'DOR	#CHUNK_SIZE,_Loop'
		define	BEGIN_LOOP_MINUS 'DOR	#CHUNK_SIZE-1,_Loop'
		define	BEGIN_LOOP_PLUS  'DOR	#CHUNK_SIZE+1,_Loop'
		define	END_LOOP	 '_Loop:'


	;**********************************************************************
	; Interrupts
	;**********************************************************************

	org	P:VecHwReset
	BRA	>Start

	org	P:VecHostRxDataFull
	BSR	>IntHostRxDataFull

	org	P:VecHostCommandDefault	; IntVecBase+$64
VecHostCommandUpdate:
	BSR	>IntHostCommandUpdate


	;**********************************************************************
	; System Data
	;**********************************************************************

	org	Xl:$000000

IN_L_CHUNK:	CHUNK
OUT_L_CHUNK:	CHUNK

Flags:
 if @def('SIMULATOR')
		dc	$000001			; Bit0=EnableProcessing=1
 else
		dc	$000000			; Bit0=EnableProcessing=0
 endif

MasterVolume:	dc	$000000
MasterVolValue:	dc	$000000

InputThru:	dc	$000000
InputThruValue:	dc	$000000


	;**********************************************************************
	; NOTE: for DMA buffer sizes < 128, external memory have to be used
	;**********************************************************************

	org	Xh:$400000

BufferDAC1	dsm	2*CHUNK_SIZE			; Buffer1 for stereo audio to be transmitted
BufferDAC2	dsm	2*CHUNK_SIZE			; Buffer2 for stereo audio to be transmitted
BufferADC1	dsm	2*CHUNK_SIZE			; Buffer1 for stereo audio received
BufferADC2	dsm	2*CHUNK_SIZE			; Buffer2 for stereo audio received


	org	Yl:$000000

HostBuffer:	ds	HOST_BUFFER_LEN		; Buffer for host data received

IN_R_CHUNK:	ds	CHUNK_SIZE
OUT_R_CHUNK:	ds	CHUNK_SIZE

INT_TMP_R0:	ds	1
INT_TMP_M0:	ds	1

ActiveBufferDAC dc	BufferDAC2		; Current output buffer to be used
ActiveBufferADC	dc	BufferADC2		; Current input buffer to be used

HostBufferPtr:	dc	HostBuffer


;*******************************************************************************
	org	P:$000100

Start:
	ORI	#$03,MR				; Disable interrupts

	;**********************************************************************
	; Configure correctly the external memory access to allow the 
	; simulator to keep the correct timing (this programation is made
	; already in a real Chameleon, thus it is not necessary)
	;**********************************************************************
 if @def('SIMULATOR')
	MOVEP	#>$FFB905,X:<<DCR		; External DRAM timing at 100 MHz:
						;  2WS in page, 8WS off page
						;  1 page = 1K words
	MOVEP	#>$40023A,X:<<AAR1		; Mapped from $400000 to $7FFFFF
 endif
	
	;**********************************************************************
	; Empty All ADC and DAC buffers
	;**********************************************************************
	MOVE	#BufferADC1,R0
	MOVE	#BufferDAC1,R1
	CLR	A

	REP	#(2*CHUNK_SIZE)
	  MOVE	A,X:(R0)+

	REP	#(2*CHUNK_SIZE)
	  MOVE	A,X:(R1)+

	;**********************************************************************
	; Configure DMA for using audio double buffering
	;**********************************************************************
	
	ConfigDMA 2*CHUNK_SIZE,BufferDAC1,BufferADC1

	;**********************************************************************
	; Enable host receive and host command interrupts
	;**********************************************************************
	BSET	#HCR_HRIE,X:<<HCR
	BSET	#HCR_HCIE,X:<<HCR

	;**********************************************************************
	; Enable ESSI0 transmiter, receiver, and receive interrupt
	;**********************************************************************
	BSET	#CRB_TE0,X:<<CRB0
	BSET	#CRB_RE,X:<<CRB0

 if !@def('SIMULATOR')
	BRSET	#SSISR_RFS,X:<<SSISR0,*		; Wait while receiving left frame
	BRCLR	#SSISR_RFS,X:<<SSISR0,*		; Wait while receiving right frame
 endif

	ANDI	#<$FC,MR			; Enable interrupts

	;**********************************************************************
	; Main Loop
	;**********************************************************************

MainLoop:
	BSR	ProcessHostData

;------------------------------------------------------------------------------
; DoOutput
; This code fills the ActiveBufferDAC with contents of OUT_L_CHUNK and OUT_R_CHUNK 
; scaled with a ramping volume
;------------------------------------------------------------------------------
DoOutput:
	MOVE	X:MasterVolValue,X0
	MOVE	X:MasterVolume,A
	SUB	X0,A		#OUT_L_CHUNK,R1
	MOVE	#OUT_R_CHUNK,R5
	ASR	#7,A,A
	MOVE	Y:ActiveBufferDAC,R0
	ADD	X0,A		A,X0

	BEGIN_LOOP
	  MOVE	A,Y0		X:(R1)+,X1
	  MPYR	X1,Y0,A		Y:(R5)+,Y1
	  MPYR	Y1,Y0,B
	  MOVE	A,X:(R0)+
	  MOVE	Y0,A
	  ADD	X0,A		
	  MOVE	B,X:(R0)+
	END_LOOP
	
	MOVE	A,X:MasterVolValue

;------------------------------------------------------------------------------
; DoInput
; This code fills IN_L_CHUNK and IN_R_CHUNK with contents of the ActiveBufferADC
;  and copy to OUT_L_CHUNK and OUT_R_CHUNK scaled with a ramping volume.
;------------------------------------------------------------------------------
DoInput:
	MOVE	X:InputThruValue,X0
	MOVE	X:InputThru,A
	SUB	X0,A		#OUT_L_CHUNK,R1
	MOVE	#OUT_R_CHUNK,R5
	ASR	#7,A,A
	MOVE	Y:ActiveBufferADC,R0
	MOVE	#IN_L_CHUNK,R2
	MOVE	#IN_R_CHUNK,R4
	ADD	X0,A		A,X0

	BEGIN_LOOP
	  MOVE	X:(R0)+,X1
	  MOVE	A,Y0		X1,X:(R2)+
	  MPYR	Y0,X1,B		X:(R0)+,Y1
	  MPYR	Y0,Y1,A		Y1,Y:(R4)+
	  MOVE	B,X:(R1)+
	  MOVE	A,Y:(R5)+	Y0,A
	  ADD	X0,A
	END_LOOP

	NOP
	MOVE	A,X:InputThruValue

	BSR	DoProcessing
	
 if @def('SIMULATOR')
 	DEBUG
 else
 	BSR	ChangeActiveBufferDMA
 endif

	BRA	MainLoop


;*******************************************************************************
	org	P:

;------------------------------------------------------------------------------
; DoProcessing
; This routine run all processing code involved (the code in 'prg.asm').
; You can define the symbol TEST_PERFORMANCE to try and obtain the number of cycles
; that the processing code could spend before the active output buffer is sent
; (to test it, change the value loaded in X1, execute, put an input signal to
; the Chameleon and pump up the input thru and master volume parameters.
; If all works right, you will not hear audio artifacts at output, but if you
; specify too much cycles to spend, you will hear noise at output).
;------------------------------------------------------------------------------
DoProcessing:
 if @def('TEST_PERFORMANCE')
	MOVE	#32000,X1			; 32000 cycles work here!!!
	DO	X1,_IdleLoop
	  NOP
_IdleLoop:
	BRA	SkipProcessing
 else
	MOVE	X:Flags,X0
	BRCLR	#0,X0,SkipProcessing
 endif
	;======================================================================
	MOVE	#ModulesDataX,R_X
	MOVE	#ModulesDataY,R_Y

	include	'prg.asm'
	;======================================================================
SkipProcessing:
	RTS

;------------------------------------------------------------------------------
; ChangeActiveBufferDMA
; This routine changes the active DMA ADC and DAC buffers waiting if CODEC is 
; still working with it.
;------------------------------------------------------------------------------
ChangeActiveBufferDMA:
	MOVE	Y:ActiveBufferDAC,A
	MOVE	#BufferDAC2,Y0
	CMP	Y0,A
	BEQ	_ChangeBuffer1

_ChangeBuffer2:
	MOVEP	X:<<DSR0,A
	CMP	Y0,A		#BufferDAC2,X0
	BGE	_ChangeBuffer2
	MOVE	X0,Y:ActiveBufferDAC
	MOVE	#BufferADC2,X0
	MOVE	X0,Y:ActiveBufferADC
	RTS

_ChangeBuffer1:
	MOVEP	X:<<DSR0,A	
	CMP	Y0,A		#BufferDAC1,X0
	BLT	_ChangeBuffer1
	MOVE	X0,Y:ActiveBufferDAC
	MOVE	#BufferADC1,X0
	MOVE	X0,Y:ActiveBufferADC
	RTS

;------------------------------------------------------------------------------
; IntHostCommandUpdate
; This routine is used to signal the update command from the host
;------------------------------------------------------------------------------
IntHostCommandUpdate:
	BCLR	#HCR_HRIE,X:<<HCR		; Disable Host Port receive interrupt until update is completed
	RTI

;------------------------------------------------------------------------------
; IntHostRxDataFull
; This routine is used to fill HostBuffer using the host port
; Warning: The host must ensure not to reach the end of this buffer
;------------------------------------------------------------------------------
IntHostRxDataFull:
	MOVE	R0,Y:INT_TMP_R0
	MOVE	M0,Y:INT_TMP_M0

	MOVE	Y:HostBufferPtr,R0
	MOVE	#-1,M0
	NOP
	NOP
	MOVEP	X:<<HRX,Y:(R0)+
	MOVE	R0,Y:HostBufferPtr

	MOVE	Y:INT_TMP_R0,R0
	MOVE	Y:INT_TMP_M0,M0
	RTI


ProcessHostData:
	BRSET	#HCR_HRIE,X:<<HCR,_ProcessHostDataContinue

	MOVE	Y:HostBufferPtr,R2
	CLR	A		#HostBuffer,R0
	MOVE	R0,Y:HostBufferPtr
	MOVE	A,Y:(R2)
_ProcessHostDataLoop:
	MOVE	Y:(R0),A			
	TST	A		Y:(R0)+,R2	; command is update?
	BEQ	_ProcessHostDataEnd
	MOVE	Y:(R0)+,R4			; R4 = addr
	MOVE	Y:(R0)+,X0			; X0 = len or value
	JSR	R2
	BRA	_ProcessHostDataLoop
	
_ProcessHostDataEnd:
	BSET	#HCR_HRIE,X:<<HCR		; Reenable Host Port receive interrupt
_ProcessHostDataContinue:
	RTS


CommandWriteBlockP:
	; R0 = buffer
	; R4 = addr
	; X0 = len
	DOR	X0,CommandWriteBlockP_Loop
	  MOVE	Y:(R0)+,X1
	  MOVEM	X1,P:(R4)+
	  NOP
CommandWriteBlockP_Loop:
	RTS

CommandWriteBlockX:
	; R0 = buffer
	; R4 = addr
	; X0 = len
	DOR	X0,CommandWriteBlockX_Loop
	  MOVE	Y:(R0)+,X1
	  MOVE	X1,X:(R4)+
CommandWriteBlockX_Loop:
	RTS

CommandWriteBlockY:
	; R0 = buffer
	; R4 = addr
	; X0 = len
	DOR	X0,CommandWriteBlockY_Loop
	  MOVE	Y:(R0)+,X1
	  MOVE	X1,Y:(R4)+
CommandWriteBlockY_Loop:
	RTS

CommandWriteWordP:
	; R4 = addr
	; X0 = value
	MOVEM	X0,P:(R4)
	RTS

CommandWriteWordX:
	; R4 = addr
	; X0 = value
	MOVE	X0,X:(R4)
	RTS

CommandWriteWordY:
	; R4 = addr
	; X0 = value
	MOVE	X0,Y:(R4)
	RTS

CommandCopyBlockP:
	MOVE	X0,R5
	MOVE	Y:(R0)+,X0
	; R4 = source addr
	; R5 = target addr
	; X0 = len
	DOR	X0,CommandCopyBlockP_Loop
	  MOVEM P:(R4)+,X1
	  MOVEM	X1,P:(R5)+
	  NOP
CommandCopyBlockP_Loop:
	RTS

CommandFillBlockX:
	MOVE	Y:(R0)+,A
	; R4 = addr
	; X0 = len
	;  A = value
	DOR	X0,CommandFillBlockX_Loop
	  MOVE	A,X:(R4)+
CommandFillBlockX_Loop:
	RTS

CommandFillBlockY:
	MOVE	Y:(R0)+,A
	; R4 = addr
	; X0 = len
	;  A = value
	DOR	X0,CommandFillBlockY_Loop
	  MOVE	A,Y:(R4)+
CommandFillBlockY_Loop:
	RTS

	end	Start
