;*******************************************************************************
; C H A M E L E O N   DSP Assembler file                                       *
;*******************************************************************************
; Copyright (C) 2001-2002 Soundart                                             *
; www.soundart-hot.com                                                         *
; support@soundart-hot.com                                                     *
;*******************************************************************************
;
; MODULE: Delay
;	This module implements an interpolating delay with feedback. It uses 
;	fractional delay size from 0 to 1.0
;
;	The code is based in the file units.asm from the 56000 assembly code
;	once part of HMSL, the Forth based Hierarchical Music Specification Language.
;	HMSL was developed by Phil Burk, Larry Polansky and David Rosenboom
;
;
; ARGUMENTS:
;	(X)   = InAddr		- Input buffer (Y)
;	(X+1) = OutAddr		- Output buffer (X)
;	(X+2) = DelayPtr	- Delay buffer pointer (Y)
;	(X+3) = Mask		- Size of the delay buffer - 1
;	(X+4) = Level		- Send level of the input signal to the module
;	(X+5) = DelaySize	- Fractional delay size (1.0 = size of the delay buffer)
;	(X+6) = Feedback	- Level of the signal to be sent to the input again
;
;******************************************************************************


	define	R_In			'R5'
	define	R_Out			'R1'
	define	R_Table			'R6'
	define	M_Table			'M6'
	define	N_Table			'N6'

;===============================CODE=====================================

	MOVE	X:(R_X)+,R_In
	MOVE	X:(R_X)+,R_Out
	MOVE	X:(R_X)+,R_Table		; R_Table = next position in sample table
	MOVE	X:(R_X)+,M_Table		; M_Table = size of sample table - 1

	BEGIN_LOOP
	;----------------------------------------------------------------
	  MOVE	X:(R_X)+,X0	Y:(R_In),Y1	; X0 = input level
	  					; Y1 = input sample
	  MPY	X0,Y1,A		X:(R_X)+,X1	; A = input_level * input_sample
	  					; X1 = FRACTIONAL tap delay
	  MOVE	M_Table,Y0			; Y0 = number of taps-1
	  MPY	X1,Y0,B		A,Y1		; B = FP tap offset
	  NOP
	  MOVE	B1,N_Table			; N_Table = Int tap offset
	  MOVE	B0,B1				; B1 = save fraction, clip off higher bits
	  LSR	B				; convert to 0-1 range,
	  MOVE	R_Table,N7
	  MOVE	B1,X1				; X1 = interpolating fraction
	  MOVE	(R_Table)+N_Table
	  MOVE	Y:(R_Table)+,X0			; X0 = delayed sample from table
	  MPY	-X1,X0,A		 	; -frac*s[n]
	  MOVE	Y:(R_Table),Y0			; get next sample
	  MAC	X1,Y0,A		X:(R_X)-,X1 	; A= frac*s[n+1] - frac*s[n], X1 = feedback level
	  ADD	X0,A		N7,R_Table	; samp = s[n] + frac*s[n+1] - frac*s[n]
	  MOVE	Y:(R_In)+,Y0			; Y0 = input
	  ADD	Y0,A		A,X0		; A = input + delayed
	  					; X0 = delayed
	  MPY	X0,X1,B				; B = delayed*feedbackLevel
	  ADD	Y1,B		(R_X)-		; B = scaled_input + delayed*feedback
	  MOVE	A,X:(R_Out)+			; save output
	  MOVE	B,Y:(R_Table)-			; save sample + feedback into delay line
	;----------------------------------------------------------------
	END_LOOP

	MOVE	R_Table,X:(R_X-2)		; save updated sample position
	LUA	(R_X+3),R_X
	MOVE	#-1,M_Table			; fix modifier registers

;===============================CODE=====================================

	undef	R_In
	undef	R_Out
	undef	R_Table
	undef	M_Table
	undef	N_Table
