;*******************************************************************************
; C H A M E L E O N   DSP Assembler file                                       *
;*******************************************************************************
; Copyright (C) 2001-2002 Soundart                                             *
; www.soundart-hot.com                                                         *
; support@soundart-hot.com                                                     *
;*******************************************************************************
;
; MODULE: Env
;     	This module computes piecewise-exponential envelope segments specified 
;	by duration/target/rate tables in dsp memory.
;
;	Duration can be either positive or negative: a negative duration means that
;	that segment envelope will hold its value forever.
;
;	The generated segments are asymptotes wich trend towards the target value.
;	The algorithm makes the following calculus:
;
;	Value[i] = (1 - Rate)*Value[i-1] + Rate*Target
;
;	This code is based in the file asympenv.asm from the musickit source code.
;	MusicKit is a set of classes and object templates used in NeXT computers
;	(which include a DSP56002 chip) to build applications for music composition,
;	synthesis and performance (author: David Jaffe, Copyright 1996 Stanford 
;	University, CCRMA, all rights reserved).
;
; ARGUMENTS:
;	(X)   = OutAddr		- Output buffer (X)
;	(X+1) = HI(Value)	- Current envelope value
;	(X+2) = LO(Value)	-
;	(X+3) = Dur		- Current segment duration (in CHUNK_SIZE units)
;	(X+4) = Target		- Current segment value to reach
;	(X+5) = Rate		- Current segment rate
;	(X+6) = NextSegAddr	- Address of the next segment data (Y)
;
;******************************************************************************

	define	R_NextSeg		'R5'
	define	R_Out			'R1'
	define	R_Tmp			'R2'

;===============================CODE=====================================

	MOVE	X:(R_X)+,R_Out		; (X)   = OutAddr (X)
	CLR	A		R_X,R_Tmp
	MOVE	X:(R_X)+,A1		; (X+1) = HI(Value)
	MOVE	X:(R_X)+,A0		; (X+2) = LO(Value)
	MOVE	X:(R_X)+,B		; (X+3) = Dur
	MOVE	X:(R_X)+,X0		; (X+4) = Target
	MOVE	X:(R_X)+,Y1		; (X+5) = Rate
	MOVE	X:(R_X)+,R_NextSeg	; (X+6) = NextSegAddr (Y)

_BEGIN:
	TST	B		#>1,Y0
	BEQ	_NEXT
	BGT	_DECAY

_CONSTANT:
	DOR	#CHUNK_SIZE,_LOOP1
	  MOVE	A,X:(R_Out)+
_LOOP1:
	SUB	Y0,B
	BRA	_CONT

_NEXT:	
	MOVE	Y:(R_NextSeg)+,B
	MOVE	Y:(R_NextSeg)+,X0
	MOVE	Y:(R_NextSeg)+,Y1
	BRA	_BEGIN

_DECAY:
	MAC	X0,Y1,A		A,X1
	MAC	-X1,Y1,A
	DOR	#CHUNK_SIZE-1,_LOOP2
	  MAC	X0,Y1,A		A,X1
	  MAC	-X1,Y1,A	X1,X:(R_Out)+
	  NOP
_LOOP2:
	SUB	Y0,B		A,X:(R_Out)+
_CONT:
	MOVE	A1,X:(R_Tmp)+
	MOVE	A0,X:(R_Tmp)+
	MOVE	B,X:(R_Tmp)+
	MOVE	X0,X:(R_Tmp)+
	MOVE	Y1,X:(R_Tmp)+
	MOVE	R_NextSeg,X:(R_Tmp)+

;===============================CODE=====================================

	undef	R_NextSeg
	undef	R_Out
	undef	R_Tmp
