;*******************************************************************************
; C H A M E L E O N   DSP Assembler file                                       *
;*******************************************************************************
; Copyright (C) 2001-2002 Soundart                                             *
; www.soundart-hot.com                                                         *
; support@soundart-hot.com                                                     *
;*******************************************************************************
;
; Basic Virtual Analog Synthesizer with Delay
; -------------------------------------------
; This patch emulates a basic analog synthesizer configuration composed by
; two oscillators, a noise generator, a mixer, a low pass filter with cutoff
; envelope, a VCA and an output (stereo) delay effect.
;
;
;
;	Osc1  --->			          ---> Delay1 --->
;	Osc2  ---> Mixer ---> Filter  ---> Amp ---|                Out (stereo)
;       Noise --->	        ^           ^     ---> Delay2 --->
;       Input --->              |           |
;			       Addk         |
;                               |           |
;			      Scale1      Scale2
;               	        |           |
;			       Env1        Env2
;
;*******************************************************************************

;==========================================
; External Memory Data
;==========================================

	org	Xh:

DELAY_TABLE_LEN		equ	$008000
WAVE_TABLE_LEN		equ	$000100

			buffer M,WAVE_TABLE_LEN
SineWave:
			include	'.\wavetables\sine.asm'
			endbuf

			buffer M,WAVE_TABLE_LEN
TriangleWave:
			include	'.\wavetables\tri.asm'
			endbuf

			buffer M,WAVE_TABLE_LEN
SawtoothWave:
			include	'.\wavetables\saw.asm'
			endbuf

			buffer M,WAVE_TABLE_LEN
SquareWave:
			include	'.\wavetables\square.asm'
			endbuf


Delay1_Table:		dsm 	DELAY_TABLE_LEN
Delay2_Table:		dsm 	DELAY_TABLE_LEN


;==========================================
; Internal Memory Data
;==========================================
	org	Xl:

Noise_Out:		CHUNK
Env1_Out:		CHUNK
Addk_Out:		CHUNK
Filter_Out:		CHUNK
Env2_Out:		CHUNK
Delay1_Out:		CHUNK
Delay2_Out:		CHUNK

ModulesDataX:


	org	Yl:

Osc1_Out:		CHUNK
Osc2_Out:		CHUNK
Mixer_Out:		CHUNK
Scale1_Out:		CHUNK
Scale2_Out:		CHUNK
Amp_Out:		CHUNK
Env1_Table:		TABLE 3*5
Env2_Table:		TABLE 3*5

ModulesDataY:


;==========================================
; Module Osc1
;==========================================
	org	Xl:
	XMEM	Osc1_TableAddr,SawtoothWave		;(X)   = TableAddr (X)
	PARAM	Osc1_TableMask,WAVE_TABLE_LEN-1		;(X+1) = TableMask
	YMEM	Osc1_OutAddr,Osc1_Out			;(X+2) = OutAddr (Y)
	PARAM	Osc1_Amplitude,0.25			;(X+3) = Amplitude
	PARAM	Osc1_IncInt,1				;(X+4) = INT(Inc)
	PARAM	Osc1_IncFract,0.0			;(X+5) = FRACT(Inc)
	PARAM	Osc1_PhaseInt,SawtoothWave		;(X+6) = INT(Phase)
	PARAM	Osc1_PhaseFract,0.0			;(X+7) = FRACT(Phase)

	org	P:
	include	'modules\osc.asm'

;==========================================
; Module Osc2
;==========================================
	org	Xl:
	XMEM	Osc2_TableAddr,SawtoothWave		;(X)   = TableAddr (X)
	PARAM	Osc2_TableMask,WAVE_TABLE_LEN-1		;(X+1) = TableMask
	YMEM	Osc2_OutAddr,Osc2_Out			;(X+2) = OutAddr (Y)
	PARAM	Osc2_Amplitude,0.25			;(X+3) = Amplitude
	PARAM	Osc2_IncInt,1				;(X+4) = INT(Inc)
	PARAM	Osc2_IncFract,0.1			;(X+5) = FRACT(Inc)
	PARAM	Osc2_PhaseInt,SawtoothWave		;(X+6) = INT(Phase)
	PARAM	Osc2_PhaseFract,0.0			;(X+7) = FRACT(Phase)

	org	P:
	include	'modules\osc.asm'

;==========================================
; Module Noise
;==========================================
	org	Xl:
	XMEM	Noise_OutAddr,Noise_Out			;(X)   = OutAddr (X)
	PARAM	Noise_Seed,$123456			;(X+1) = Seed

	org	P:
	include	'modules\noise.asm'

;==========================================
; Module Mixer
;==========================================
	org	Xl:
	XMEM	Mixer_In1Addr,IN_L_CHUNK			;(X)   = In1Addr (X)
	XMEM	Mixer_In2Addr,Noise_Out			;(X+1) = In2Addr (X)
	YMEM	Mixer_In3Addr,Osc1_Out			;(X+1) = In3Addr (Y)
	YMEM	Mixer_In4Addr,Osc2_Out			;(X+1) = In4Addr (Y)
	YMEM	Mixer_OutAddr,Mixer_Out			;(X+2) = OutAddr (Y)
	PARAM	Mixer_ScaleValue1,0			;(X+3) = ScaleValue1
	PARAM	Mixer_ScaleValue2,0			;(X+4) = ScaleValue2

	org	P:
	include	'.\modules\mixer.asm'

;==========================================
; Module Env1
;==========================================
	org	Xl:
	XMEM	Env1_OutAddr,Env1_Out			;(X)   = OutAddr (X)
	PARAM	Env1_HiValue,0				;(X+1) = HI(Value)
	PARAM	Env1_LoValue,0				;(X+2) = LO(Value)
	PARAM	Env1_Dur,-1				;(X+3) = Dur
	PARAM	Env1_Target,0				;(X+4) = Target
	PARAM	Env1_Rate,0				;(X+5) = Rate
	YMEM	Env1_NextSegAddr,Env1_Table		;(X+6) = NextSegAddr (Y)

	org	P:
	include	'modules\env.asm'

;==========================================
; Module Scale1
;==========================================
	org	Xl:
	XMEM	Scale1_InAddr,Env1_Out			;(X)   = InAddr (X)
	YMEM	Scale1_OutAddr,Scale1_Out		;(X+1) = OutAddr (Y)
	PARAM	Scale1_ScaleValue,0			;(X+2) = ScaleValue

	org	P:
	include	'modules\scale.asm'

;==========================================
; Module Addk
;==========================================
	org	Xl:
	YMEM	Addk_InAddr,Scale1_Out			;(X)   = InAddr (Y)
	XMEM	Addk_OutAddr,Addk_Out			;(X+1) = OutAddr (X)
	PARAM	Addk_Constant,$7FFFFF			;(X+2) = Constant

	org	P:
	include	'modules\addk.asm'

;==========================================
; Module Filter
;==========================================
	org	Xl:
	YMEM	Filter_InAddr,Mixer_Out			;(X)   = InAddr (Y)
	XMEM	Filter_OutAddr,Filter_Out		;(X+1) = OutAddr (X)
	XMEM	Filter_ControlAddr,Addk_Out		;(X+2) = ControlAddr (X)
	PARAM	Filter_Res,0				;(X+3) = Resonance
	PARAM	Filter_In1,0				;(X+4) = In1
	PARAM	Filter_Out1,0				;(X+5) = Out1
	PARAM	Filter_In2,0				;(X+6) = In2
	PARAM	Filter_Out2,0				;(X+7) = Out2
	PARAM	Filter_In3,0				;(X+8) = In3
	PARAM	Filter_Out3,0				;(X+9) = Out3
	PARAM	Filter_In4,0				;(X+10)= In4
	PARAM	Filter_Out4,0				;(X+11)= Out4

	org	P:
	include	'modules\filter.asm'

;==========================================
; Module Env2
;==========================================
	org	Xl:
	XMEM	Env2_OutAddr,Env2_Out			;(X)   = OutAddr (X)
	PARAM	Env2_HiValue,0				;(X+1) = HI(Value)
	PARAM	Env2_LoValue,0				;(X+2) = LO(Value)
	PARAM	Env2_Dur,-1				;(X+3) = Dur
	PARAM	Env2_Target,0				;(X+4) = Target
	PARAM	Env2_Rate,0				;(X+5) = Rate
	YMEM	Env2_NextSegAddr,Env2_Table		;(X+6) = NextSegAddr (Y)

	org	P:
	include	'modules\env.asm'

;==========================================
; Module Scale2
;==========================================
	org	Xl:
	XMEM	Scale2_InAddr,Env2_Out			;(X)   = InAddr (X)
	YMEM	Scale2_OutAddr,Scale2_Out		;(X+1) = OutAddr (Y)
	PARAM	Scale2_ScaleValue,0			;(X+2) = ScaleValue

	org	P:
	include	'modules\scale.asm'

;==========================================
; Module Amp
;==========================================
	org	Xl:
	XMEM	Amp_In1Addr,Filter_Out			;(X)   = In1Addr (X)
	YMEM	Amp_In2Addr,Scale2_Out			;(X+1) = In2Addr (Y)
	YMEM	Amp_OutAddr,Amp_Out 			;(X+2) = OutAddr (Y)

	org	P:
	include	'modules\amp.asm'

;==========================================
; Module Delay1
;==========================================
	org	Xl:
	YMEM	Delay1_InAddr,Amp_Out			;(X)   = InAddr (Y)
	XMEM	Delay1_OutAddr,Delay1_Out		;(X+1) = OutAddr (X)
	YMEM	Delay1_Pointer,Delay1_Table		;(X+2) = Pointer (Y)
	PARAM	Delay1_Mask,DELAY_TABLE_LEN-1		;(X+3) = Mask
	PARAM	Delay1_Level,0.9			;(X+4) = Level
	PARAM	Delay1_Size,0.55			;(X+5) = Size (fract)
	PARAM	Delay1_Feedback,0.25			;(X+6) = Feedback

	org	P:
	include	'.\modules\delay.asm'

;==========================================
; Module Delay2
;==========================================
	org	Xl:
	YMEM	Delay2_InAddr,Amp_Out			;(X)   = InAddr (Y)
	XMEM	Delay2_OutAddr,Delay2_Out		;(X+1) = OutAddr (X)
	YMEM	Delay2_Pointer,Delay2_Table		;(X+2) = Pointer (Y)
	PARAM	Delay2_Mask,DELAY_TABLE_LEN-1		;(X+3) = Mask
	PARAM	Delay2_Level,0.9			;(X+4) = Level
	PARAM	Delay2_Size,0.75			;(X+5) = Size (fract)
	PARAM	Delay2_Feedback,0.3			;(X+6) = Feedback

	org	P:
	include	'.\modules\delay.asm'

;==========================================
; Module Out
;==========================================
	org	Xl:
	XMEM	Out_In1Addr,Delay1_Out			;(X)   = In1Addr (X)
	XMEM	Out_In2Addr,Delay2_Out			;(X+1) = In2Addr (X)

	org	P:
	include	'modules\out.asm'

