/*******************************************************************************
 * C H A M E L E O N                                                           *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/MONOSYNTH/engine.cpp                $
 *     $Date:: 13/05/02 20:11                                                  $
 * $Revision:: 1                                                               $
 *-----------------------------------------------------------------------------*
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#include <rtems.h>
#include <chameleon.h>

#include "Engine.h"

//////////////////////////////////////////////////////////////////////////////

static void receive_alarm(short nMidiRef)
{
	CEngine *pEngine = (CEngine *) MidiGetInfo(nMidiRef);
	MidiEvPtr ev;
	
	ASSERT(pEngine);
	
	while ((ev = MidiGetEv(nMidiRef)))
		pEngine->SendMsgMidi(ev);
}

BOOL CEngine::Create(CPanel *pPanel)
{
	ASSERT(m_pPanel == NULL);
	ASSERT(pPanel);
	
	m_pPanel = pPanel;
	
	m_nMidiRef = MidiOpen("Midi");
	if (m_nMidiRef < 0)
	{
		TRACE("ERROR: cannot open MidiShare in CEngine::Create\n");
		return FALSE;
	}

	TRACE("Creating task for handling engine messages...\n");
	
	if (!CMessageHandler::Create(
		rtems_build_name('E', 'N', 'G', 'T'), 100,
		rtems_build_name('E', 'N', 'G', 'Q'), ENGINE_MAX_MESSAGE_COUNT, ENGINE_MAX_MESSAGE_SIZE))
	{
		TRACE("ERROR: cannot create the message handler in CEngine::Create\n");
		return FALSE;
	}

	MidiStreamInitMthTbl(m_lTbl);
	MidiStreamInit(&m_xmt, m_lTbl);

	if (!Initialize())
	{
		TRACE("ERROR: cannot initialize engine\n");
		return FALSE;
	}

	MidiSetInfo(m_nMidiRef, (void *) this);
	MidiSetRcvAlarm(m_nMidiRef, receive_alarm);
	
	MidiConnect(0, m_nMidiRef, TRUE);
	MidiConnect(m_nMidiRef, 0, TRUE);

	m_bInitialized = TRUE;

	return TRUE;
}

void CEngine::SendMsgMidi(MidiEvPtr ev)
{
	ASSERT(ev);
	CEngineMsg_Midi msg;
	
	msg.ev = ev;
	SendMessage(&msg);
}

void CEngine::SendMsgTimer(rtems_id idTimer)
{
	CEngineMsg_Timer msg;

	msg.idTimer = idTimer;
	SendMessage(&msg);
}

//////////////////////////////////////////////////////////////////////////////

static rtems_timer_service_routine TimerProc(rtems_id idTimer, void *data)
{
	((CEngine *) data)->SendMsgTimer(idTimer);
}

rtems_id CEngine::CreateTimer(rtems_name timer_name, rtems_interval interval)
{
	rtems_status_code status;
	rtems_id idTimer;

	status = rtems_timer_create(timer_name, &idTimer);
	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: cannot create timer.\n");
		return 0;
	}

	status = rtems_timer_fire_after(idTimer, interval, TimerProc, this);
	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: cannot start timer.\n");
		return 0;
	}
	rtems_timer_cancel(idTimer);

	return idTimer;
}

void CEngine::ChangeTimerInterval(rtems_id idTimer, rtems_interval interval)
{
	rtems_timer_fire_after(idTimer, interval, TimerProc, this);
}

void CEngine::ResetTimer(rtems_id idTimer)
{	
	rtems_timer_reset(idTimer);
}

void CEngine::CancelTimer(rtems_id idTimer)
{	
	rtems_timer_cancel(idTimer);
}

