/*******************************************************************************
 * C H A M E L E O N                                                           *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/MONOSYNTH/engine.h                  $
 *     $Date:: 13/05/02 20:11                                                  $
 * $Revision:: 1                                                               $
 *-----------------------------------------------------------------------------*
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#ifndef __ENGINE_H__
#define __ENGINE_H__

#include <string.h>
#include <rtems.h>
#include <MidiShare.h>
#include <EventToMidiStream.h>
#include <MidiStreamToEvent.h>

#include "message.h"

//////////////////////////////////////////////////////////////////////////////
class CPanel;
//////////////////////////////////////////////////////////////////////////////

class CEngine : public CMessageHandler
{
protected:
	CPanel *m_pPanel;

protected:
	short m_nMidiRef;
	BOOL  m_bInitialized;

protected:
	Ev2StreamRec	m_xmt;
	LinearizeMthTbl	m_lTbl;

protected:
	virtual BOOL Initialize(void) = 0;

public:
	CEngine(void) { m_pPanel = NULL; m_bInitialized = FALSE; }
	
	void Task(void);

	BOOL Create(CPanel *pPanel);

public:
	BOOL IsInitialized(void) { return m_bInitialized; }

public:
	void SendMsgMidi(MidiEvPtr ev);
	void SendMsgTimer(rtems_id idTimer);

public:
	rtems_id CreateTimer(rtems_name timer_name, rtems_interval interval);
	void ChangeTimerInterval(rtems_id idTimer, rtems_interval interval);
	void ResetTimer(rtems_id idTimer);
	void CancelTimer(rtems_id idTimer);
};

//////////////////////////////////////////////////////////////////////////////
#define ENGINE_MAX_MESSAGE_LEN		25
#define ENGINE_MAX_MESSAGE_SIZE		(ENGINE_MAX_MESSAGE_LEN*sizeof(BYTE))
#define ENGINE_MAX_MESSAGE_COUNT	100
//////////////////////////////////////////////////////////////////////////////
enum
{
	kEngineMsgMidi = 0,
	kEngineMsgTimer,
	kEngineMsgUser,
};
//////////////////////////////////////////////////////////////////////////////

struct CEngineMsg_Midi : public CMessage
{
	MidiEvPtr ev;

	CEngineMsg_Midi(void)
	{
		ASSERT(GetSize() <= ENGINE_MAX_MESSAGE_SIZE);
		dwMsg = kEngineMsgMidi;
	}
	
	virtual const DWORD GetSize(void) { return sizeof(CEngineMsg_Midi); }
};

struct CEngineMsg_Timer : public CMessage
{
	rtems_id idTimer;

	CEngineMsg_Timer(void)
	{
		ASSERT(GetSize() <= ENGINE_MAX_MESSAGE_SIZE);
		dwMsg = kEngineMsgTimer;
	}

	virtual const DWORD GetSize(void) { return sizeof(CEngineMsg_Timer); }
};

//////////////////////////////////////////////////////////////////////////////

#endif
