/*******************************************************************************
 * C H A M E L E O N                                                           *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/MONOSYNTH/message.cpp               $
 *     $Date:: 13/05/02 20:11                                                  $
 * $Revision:: 1                                                               $
 *-----------------------------------------------------------------------------*
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#include "message.h"

CMessageHandler::~CMessageHandler(void)
{
	if (m_idQueue)
		rtems_message_queue_delete(m_idQueue);
}

static rtems_task task(rtems_task_argument argument)
{
	((CMessageHandler *) argument)->Task();
}

void CMessageHandler::Task(void)
{
	rtems_unsigned8	  *msg_buffer;
	rtems_unsigned32   msg_size;
	rtems_status_code  status;

	ASSERT(m_idQueue);

	TRACE("Starting CMessageHandler::Task\n");
	
	msg_buffer = new BYTE[m_nMaxMessageSize];

	while (TRUE)
	{
		status = rtems_message_queue_receive(
			m_idQueue, 
			msg_buffer, 
			&msg_size, 
			RTEMS_WAIT, 
			RTEMS_NO_TIMEOUT
		);
		if (status != RTEMS_SUCCESSFUL)
			break;

		OnMessage((CMessage *) msg_buffer);
	}

	delete[] msg_buffer;

	TRACE("ERROR: unexpected exit from CMessageHandler::Task\n");
	rtems_task_delete(RTEMS_SELF);
}

BOOL CMessageHandler::Create(rtems_name task_name, rtems_task_priority priority, rtems_name queue_name, int nMaxMsgCount, int nMaxMsgSize)
{
	rtems_id	  task_id;
	rtems_status_code status;
	
	ASSERT(m_idQueue == 0);
	
	status = rtems_message_queue_create(
		queue_name, 
		nMaxMsgCount, 
		nMaxMsgSize, 
		RTEMS_FIFO | RTEMS_LOCAL, 
		&m_idQueue);
	if (status != RTEMS_SUCCESSFUL)
		return FALSE;
		
	m_nMaxMessageSize = nMaxMsgSize;

	status = rtems_task_create(
		task_name,
		priority,
		RTEMS_MINIMUM_STACK_SIZE,
		RTEMS_DEFAULT_MODES,
		RTEMS_DEFAULT_ATTRIBUTES,
		&task_id
	);
	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: cannot create CMessageHandler::Task\n");
		return FALSE;
	}

	status = rtems_task_start(task_id, task, (rtems_task_argument) this);
	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: cannot start CMessageHandler::Task\n");
		return FALSE;
	}
	
	TRACE("Task created successfully: 0x%08lX\n", (DWORD) task_id);
	
	return TRUE;
}
