/*******************************************************************************
 * C H A M E L E O N                                                           *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/MONOSYNTH/panel.cpp                 $
 *     $Date:: 13/05/02 20:11                                                  $
 * $Revision:: 1                                                               $
 *-----------------------------------------------------------------------------*
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#include <string.h>
#include <rtems.h>
#include <chameleon.h>

#include "panel.h"

//////////////////////////////////////////////////////////////////////////////

static rtems_task task(rtems_task_argument argument)
{
	((CPanel *) argument)->Task();
}

void CPanel::Task(void)
{
	rtems_unsigned8	 potentiometer;
	rtems_unsigned8	 value;
	rtems_unsigned32 key_bits;
	rtems_unsigned8	 encoder;
	rtems_signed8	 increment;

	TRACE("Starting CPanel::Task\n");

	while (TRUE)
	{
		if (!panel_in_new_event(m_nPanelRef, TRUE))
			break;

		if (panel_in_potentiometer(m_nPanelRef, &potentiometer, &value))
			SendMsgPotentiometer(potentiometer, value);
		else if (panel_in_encoder(m_nPanelRef, &encoder, &increment))
			SendMsgEncoder(encoder, increment);
		else if (panel_in_keypad(m_nPanelRef, &key_bits))
			SendMsgKeypad(key_bits);
		else
			continue;
	}

	TRACE("ERROR: unexpected exit from CPanel::Task\n");
	rtems_task_delete(RTEMS_SELF);
}

BOOL CPanel::Create(CEngine *pEngine)
{
	rtems_id	  task_id = 0;
	rtems_status_code status;

	ASSERT(m_pEngine == NULL);
	ASSERT(pEngine);
	
	m_pEngine = pEngine;

	m_nPanelRef = panel_init();
	if (m_nPanelRef < 0)
	{
		TRACE("ERROR: cannot init the panel in CPanel::Create\n");
		return FALSE;
	}

	/*
	 * Create the message queue used to receive messages from other tasks
	 */
	 
	TRACE("Creating task for handling panel messages...\n");
	 
	if (!CMessageHandler::Create(
		rtems_build_name('P', 'A', 'N', 'T'), 50,
		rtems_build_name('P', 'A', 'N', 'Q'), PANEL_MAX_MESSAGE_COUNT, PANEL_MAX_MESSAGE_SIZE))
	{
		TRACE("ERROR: cannot create the message handler in CPanel::Create\n");
		return FALSE;
	}

	m_dwKeyBits = 0;
	m_dwLedBits = 0;

	if (!Initialize())
	{
		TRACE("ERROR: cannot initialize panel\n");
		return FALSE;
	}

	/*
	 * Create the task used to receive from panel
	 */
	
	TRACE("Creating task for handling panel events...\n");
	
	status = rtems_task_create(
		rtems_build_name('P', 'A', 'N', 'W'),
		20,
		RTEMS_MINIMUM_STACK_SIZE,
		RTEMS_DEFAULT_MODES,
		RTEMS_DEFAULT_ATTRIBUTES,
		&task_id
	);
	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: cannot create CPanel::Task\n");
		return FALSE;
	}

	status = rtems_task_start(task_id, task, (rtems_task_argument) this);
	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: cannot start CPanel::Task\n");
		return FALSE;
	}

	return TRUE;
}

void CPanel::SendMsgPotentiometer(BYTE byPotentiometer, BYTE byValue)
{
	CPanelMsg_Potentiometer msg;
	
	msg.byPotentiometer = byPotentiometer;
	msg.byValue = byValue;
	SendMessage(&msg);
	
	m_byPotValue[byPotentiometer] = byValue;
}

void CPanel::SendMsgEncoder(BYTE byEncoder, char nIncrement)
{
	CPanelMsg_Encoder msg;
	
	msg.byEncoder = byEncoder;
	msg.nIncrement = nIncrement;
	SendMessage(&msg);
}

void CPanel::SendMsgKeypad(DWORD dwKeyBits)
{
	CPanelMsg_Keypad msg;
	
	msg.dwKeyBits = dwKeyBits;
	SendMessage(&msg);
	
	m_dwKeyBits = dwKeyBits;
}

void CPanel::SendMsgTimer(rtems_id idTimer)
{
	CPanelMsg_Timer msg;

	msg.idTimer = idTimer;
	SendMessage(&msg);
}

//////////////////////////////////////////////////////////////////////////////

static rtems_timer_service_routine TimerProc(rtems_id idTimer, void *data)
{
	((CPanel *) data)->SendMsgTimer(idTimer);
}

rtems_id CPanel::CreateTimer(rtems_name timer_name, rtems_interval interval)
{
	rtems_status_code status;
	rtems_id idTimer;

	status = rtems_timer_create(timer_name, &idTimer);
	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: cannot create timer.\n");
		return 0;
	}

	status = rtems_timer_fire_after(idTimer, interval, TimerProc, this);
	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: cannot start timer.\n");
		return 0;
	}
	rtems_timer_cancel(idTimer);

	return idTimer;
}

void CPanel::ChangeTimerInterval(rtems_id idTimer, rtems_interval interval)
{
	rtems_timer_fire_after(idTimer, interval, TimerProc, this);
}

void CPanel::ResetTimer(rtems_id idTimer)
{	
	rtems_timer_reset(idTimer);
}

void CPanel::CancelTimer(rtems_id idTimer)
{	
	rtems_timer_cancel(idTimer);
}

