/*******************************************************************************
 * C H A M E L E O N                                                           *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/MONOSYNTH/panel.h                   $
 *     $Date:: 13/05/02 20:11                                                  $
 * $Revision:: 1                                                               $
 *-----------------------------------------------------------------------------*
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#ifndef __PANEL_H__
#define __PANEL_H__

#include "message.h"

//////////////////////////////////////////////////////////////////////////////
class CEngine;
//////////////////////////////////////////////////////////////////////////////

class CPanel : public CMessageHandler
{
protected:
	CEngine *m_pEngine;

private:
	int m_nPanelRef;
	
private:
	DWORD m_dwKeyBits;
	DWORD m_dwLedBits;
	BYTE  m_byPotValue[256];

protected:
	virtual BOOL Initialize(void) = 0;

public:
	CEngine *GetEngine(void)
	{
		return m_pEngine;
	}

public:
	DWORD PanelGetPotentiometer(BYTE byPot);
	DWORD PanelGetKeys(void);
	DWORD PanelGetLeds(void);
	void  PanelSetLeds(DWORD dwLedBits);
	void  PanelLcdPrint(BYTE byRow, BYTE byCol, const char *strText);
	void  PanelLcdRedefine(BYTE byChar, const BYTE *data);

public:
	CPanel(void) { m_pEngine = NULL; }
	
	void Task(void);

	BOOL Create(CEngine *pEngine);
	
public:
	void SendMsgPotentiometer(BYTE byPotentiometer, BYTE byValue);
	void SendMsgEncoder(BYTE byEncoder, char nIncrement);
	void SendMsgKeypad(DWORD dwKeyBits);
	void SendMsgTimer(rtems_id idTimer);

public:
	rtems_id CreateTimer(rtems_name timer_name, rtems_interval interval);
	void ChangeTimerInterval(rtems_id idTimer, rtems_interval interval);
	void ResetTimer(rtems_id idTimer);
	void CancelTimer(rtems_id idTimer);
};

//////////////////////////////////////////////////////////////////////////////
#define PANEL_MAX_MESSAGE_LEN		25
#define PANEL_MAX_MESSAGE_SIZE		(PANEL_MAX_MESSAGE_LEN*sizeof(BYTE))
#define PANEL_MAX_MESSAGE_COUNT		100
//////////////////////////////////////////////////////////////////////////////
enum
{
	kPanelMsgPotentiometer = 0,
	kPanelMsgEncoder,
	kPanelMsgKeypad,
	kPanelMsgTimer,
	kPanelMsgUser,
};
//////////////////////////////////////////////////////////////////////////////

struct CPanelMsg_Potentiometer : public CMessage
{
	BYTE byPotentiometer;
	BYTE byValue;

	CPanelMsg_Potentiometer(void)
	{
		ASSERT(GetSize() <= PANEL_MAX_MESSAGE_SIZE);
		dwMsg = kPanelMsgPotentiometer;
	}
	
	virtual const DWORD GetSize(void) { return sizeof(CPanelMsg_Potentiometer); }
};

struct CPanelMsg_Encoder : public CMessage
{
	BYTE byEncoder;
	char nIncrement;

	CPanelMsg_Encoder(void)
	{
		ASSERT(GetSize() <= PANEL_MAX_MESSAGE_SIZE);
		dwMsg = kPanelMsgEncoder;
	}

	virtual const DWORD GetSize(void) { return sizeof(CPanelMsg_Encoder); }
};

struct CPanelMsg_Keypad : public CMessage
{
	DWORD dwKeyBits;

	CPanelMsg_Keypad(void)
	{
		ASSERT(GetSize() <= PANEL_MAX_MESSAGE_SIZE);
		dwMsg = kPanelMsgKeypad;
	}

	virtual const DWORD GetSize(void) { return sizeof(CPanelMsg_Keypad); }
};

struct CPanelMsg_Timer : public CMessage
{
	rtems_id idTimer;

	CPanelMsg_Timer(void)
	{
		ASSERT(GetSize() <= PANEL_MAX_MESSAGE_SIZE);
		dwMsg = kPanelMsgTimer;
	}

	virtual const DWORD GetSize(void) { return sizeof(CPanelMsg_Timer); }
};

//////////////////////////////////////////////////////////////////////////////

inline DWORD CPanel::PanelGetPotentiometer(BYTE byPot)
{
	return m_byPotValue[byPot];
}

inline DWORD CPanel::PanelGetKeys(void)
{
	return m_dwKeyBits;
}

inline DWORD CPanel::PanelGetLeds(void)
{
	return m_dwLedBits;
}

inline void CPanel::PanelSetLeds(DWORD dwLedBits)
{
	ASSERT(m_nPanelRef >= 0);
	panel_out_led(m_nPanelRef, dwLedBits);
	m_dwLedBits = dwLedBits;
}

inline void CPanel::PanelLcdPrint(BYTE byRow, BYTE byCol, const char *strText)
{
	ASSERT(m_nPanelRef >= 0);
	panel_out_lcd_print(m_nPanelRef, byRow, byCol, (char *) strText);
}

inline void CPanel::PanelLcdRedefine(BYTE byChar, const BYTE *data)
{
	ASSERT(m_nPanelRef >= 0);
	panel_out_lcd_redefine(m_nPanelRef, byChar, data);
}

//////////////////////////////////////////////////////////////////////////////

#endif
