/*******************************************************************************
 * C H A M E L E O N                                                           *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/MONOSYNTH/parameters.h              $
 *     $Date:: 13/05/02 20:11                                                  $
 * $Revision:: 1                                                               $
 *-----------------------------------------------------------------------------*
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#ifndef __PARAMETERS_H__
#define __PARAMETERS_H__

#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
#define kSampleRate	48000
//////////////////////////////////////////////////////////////////////////////
class CParameter;
//////////////////////////////////////////////////////////////////////////////
CParameter *GetParameterFromID(WORD wID);
void RegisterParameter(WORD wID, CParameter *pParameter);
void UnregisterParameter(WORD wID);
//////////////////////////////////////////////////////////////////////////////

class CParameter
{
protected:
	WORD  m_wID;
	
	BYTE  m_byValue;
	BYTE  m_byMinValue;
	BYTE  m_byMaxValue;
	
public:
	void Initialize(WORD wID, BYTE byDefValue, BYTE byMinValue, BYTE byMaxValue)
	{
		ASSERT(byMinValue <= byMaxValue);
		ASSERT(byDefValue >= byMinValue);
		ASSERT(byDefValue <= byMaxValue);

		m_wID = wID;
		
		m_byMinValue = byMinValue;
		m_byMaxValue = byMaxValue;
		
		RegisterParameter(wID, this);
		
		SetValue(byDefValue);
	}

	virtual const char *GetName(void) = 0;
	virtual const char *GetShortMenuName(void) { return GetName(); }
	virtual const char *GetLongMenuName(void) { return GetName(); }
	
	WORD GetID(void)
	{
		return m_wID;
	}
	
	BYTE GetValue(void) 	{ return m_byValue; }	
	BYTE GetMin(void)   	{ return m_byMinValue; }
	BYTE GetMax(void)   	{ return m_byMaxValue; }
	BYTE GetRange(void) 	{ return m_byMaxValue - m_byMinValue + 1; }
	
	BOOL IncValue(void)
	{
		if (m_byValue < m_byMaxValue)
		{
			SetValue(m_byValue+1);
			return TRUE;
		}
		
		return FALSE;
	}

	BOOL DecValue(void)
	{
		if (m_byValue > m_byMinValue)
		{
			SetValue(m_byValue-1);
			return TRUE;
		}
		
		return FALSE;
	}

	virtual void SetValue(short byValue)
	{
		if (byValue < (short) m_byMinValue)
			m_byValue = m_byMinValue;
		else if (byValue > m_byMaxValue)
			m_byValue = m_byMaxValue;
		else
			m_byValue = (BYTE) byValue;
	}

	void SetValueMidi(BYTE byValueMidi) 
	{
		DWORD dwValue = (DWORD) byValueMidi*(m_byMaxValue - m_byMinValue);
	
		if (dwValue > 0x40)
			SetValue((BYTE) m_byMinValue + 1 + (dwValue>>7));
		else
			SetValue(m_byValue = (BYTE) m_byMinValue + (dwValue>>7));
	}

	virtual void GetValueText(char *strValueText)
	{
		ASSERT(strValueText);
		sprintf(strValueText, "%3d", (int) m_byValue);
	}
};

//////////////////////////////////////////////////////////////////////////////

#endif

