/*******************************************************************************
 * C H A M E L E O N    S. D. K.                                               *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/showpanel/main.c                    $
 *     $Date:: 14/05/02 17:12                                                  $
 * $Revision:: 8                                                               $
 *-----------------------------------------------------------------------------*
 * This file is part of the Chameleon Software Development Kit                 *
 *                                                                             *
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <rtems.h>
#include <chameleon.h>

/******************************************************************************/
#define WORKSPACE_SIZE	128*1024
rtems_unsigned32 rtems_workspace_size = WORKSPACE_SIZE;
rtems_unsigned32 rtems_workspace_start[WORKSPACE_SIZE];
/******************************************************************************/

void Error(char *error)
{
	TRACE(error);
	exit(1);
}

void show_info(char serial_number[11])
{
	rtems_unsigned8 model;

	if (!info_get_serial_number(serial_number) ||
	    !info_get_model(&model))
	{
		rtems_task_delete(RTEMS_SELF);
	}

	TRACE("Serial Number: %s\n", serial_number);
	TRACE("Model:         model#%02X\n", model);
	TRACE("\n");
}

static rtems_task led_task(rtems_task_argument argument)
{
	rtems_name		name;
	rtems_id		period_id;
	rtems_status_code	status;
	rtems_unsigned32	led_bits = 0;
	int			panel = argument;

	name = rtems_build_name('P', 'E', 'R', '0');
	status = rtems_rate_monotonic_create(name, &period_id);
	if (status == RTEMS_SUCCESSFUL)
	{
		while (rtems_rate_monotonic_period(period_id, 200) != RTEMS_TIMEOUT)
		{
			switch (led_bits)
			{
				case 0:
					led_bits = PANEL01_LED_SHIFT;
					break;
				case PANEL01_LED_SHIFT:
					led_bits = PANEL01_LED_EDIT;
					break;
				case PANEL01_LED_EDIT:
					led_bits = PANEL01_LED_CTRL1;
					break;
				case PANEL01_LED_CTRL1:
					led_bits = PANEL01_LED_CTRL2;
					break;
				case PANEL01_LED_CTRL2:
					led_bits = PANEL01_LED_CTRL3;
					break;
				case PANEL01_LED_CTRL3:
					led_bits = PANEL01_LED_SHIFT;
					break;
			}

			if (!panel_out_led(panel, led_bits))
				break;
		}
	}

	TRACE("ERROR: unexpected exit from led_task.\n");
	rtems_task_delete(RTEMS_SELF);
}

rtems_id create_led_task(rtems_task_argument argument)
{
	rtems_id		task_id;
	rtems_status_code	status;
	rtems_name		name;

	name = rtems_build_name('T', 'L', 'E', 'D');

	status = rtems_task_create(
		name,
		100,
		RTEMS_MINIMUM_STACK_SIZE,
		RTEMS_DEFAULT_MODES,
		RTEMS_DEFAULT_ATTRIBUTES,
		&task_id
	);
	if (status != RTEMS_SUCCESSFUL)
		return 0;

	status = rtems_task_start(task_id, led_task, argument);
	if (status != RTEMS_SUCCESSFUL)
		return 0;

	return task_id;
}

static rtems_task clear_task(rtems_task_argument argument)
{
	int panel = argument;

	if (panel)
	{
		rtems_task_wake_after(500);
		panel_out_lcd_print(panel, 1, 0, "                ");
	}
	rtems_task_suspend(RTEMS_SELF);
}

rtems_id create_clear_task(rtems_task_argument argument)
{
	rtems_id		task_id;
	rtems_status_code	status;
	rtems_name		name;

	name = rtems_build_name('T', 'C', 'L', 'R');

	status = rtems_task_create(
		name,
		100,
		RTEMS_MINIMUM_STACK_SIZE,
		RTEMS_DEFAULT_MODES,
		RTEMS_DEFAULT_ATTRIBUTES,
		&task_id
	);
	if (status != RTEMS_SUCCESSFUL)
		return 0;

	status = rtems_task_start(task_id, clear_task, argument);
	if (status != RTEMS_SUCCESSFUL)
		return 0;

	return task_id;
}

static rtems_unsigned8 lcd_symbol1[8] =
{
	0x01,	/* 00000001 */
	0x03,	/* 00000011 */
	0x07,	/* 00000111 */
	0x0f,	/* 00001111 */
	0x07,	/* 00000111 */
	0x03,	/* 00000011 */
	0x01,	/* 00000001 */
	0x00	/* 00000000 */
};
static rtems_unsigned8 lcd_symbol2[8] =
{
	0x10,	/* 00010000 */
	0x18,	/* 00011000 */
	0x1C,	/* 00011100 */
	0x1E,	/* 00011110 */
	0x1C,	/* 00011100 */
	0x18,	/* 00011000 */
	0x10,	/* 00010000 */
	0x00	/* 00000000 */
};

static rtems_task panel_task(rtems_task_argument argument)
{
	rtems_unsigned8		potentiometer;
	rtems_unsigned8		value;
	rtems_unsigned32	key_bits;
	rtems_unsigned8		encoder;
	rtems_signed8		increment;
	rtems_id		led_task_id;
	rtems_id		clear_task_id;
	char			text[17];
	char			serial_number[11];
	int			panel;

	TRACE("******************************************************************************\n");
	TRACE("Panel Example\n");
	TRACE("******************************************************************************\n");

	show_info(serial_number);

	panel = panel_init();
	if (!panel)
		Error("ERROR: cannot access the panel.\n");

	if (!panel_out_lcd_redefine(panel, 1, lcd_symbol1))
		Error("ERROR: cannot redefine LCD symbol 0.\n");

	if (!panel_out_lcd_redefine(panel, 2, lcd_symbol2))
		Error("ERROR: cannot redefine LCD symbol 1.\n");

	led_task_id = create_led_task(panel);
	if (!led_task_id)
		Error("ERROR: cannot create led_task.\n");

	clear_task_id = create_clear_task(0);
	if (!clear_task_id)
		Error("ERROR: cannot create clear_task.\n");

	panel_out_lcd_print(panel, 0, 0, " Panel Example  ");

	while (TRUE)
	{
		if (!panel_in_new_event(panel, TRUE))
		{
			TRACE("ERROR: unexpected exit waiting new panel event.\n");
			break;
		}

		if (panel_in_potentiometer(panel, &potentiometer, &value))
		{
			sprintf(text, "Pot %02X     %c%3d%c", potentiometer, 1, value, 2);
			panel_out_lcd_print(panel, 1, 0, text);
			TRACE("POTENTIOMETER 0x%02X = %03d (0x%02X)\n", potentiometer, value, value);
		}
		else if (panel_in_keypad(panel, &key_bits))
		{
			sprintf(text, "Keypad  %08X", key_bits);
			panel_out_lcd_print(panel, 1, 0, text);
			TRACE("KEYPAD        0x%08X\n", key_bits);
		}
		else if (panel_in_encoder(panel, &encoder, &increment))
		{
			sprintf(text, "Encoder %02X %c%+3d%c", encoder, 1, increment, 2);
			panel_out_lcd_print(panel, 1, 0, text);
			TRACE("ENCODER       0x%02X = %+d\n", encoder, increment);
		}

		rtems_task_restart(clear_task_id, panel);
	}

	panel_exit(panel);

	TRACE("ERROR: unexpected exit from panel_task.\n");
	rtems_task_delete(RTEMS_SELF);
}

rtems_boolean create_panel_task(void)
{
	rtems_id		task_id;
	rtems_status_code	status;

	status = rtems_task_create(
		rtems_build_name('S', 'H', 'O', 'W'),
		50,
		RTEMS_MINIMUM_STACK_SIZE,
		RTEMS_DEFAULT_MODES,
		RTEMS_DEFAULT_ATTRIBUTES,
		&task_id
	);

	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: cannot create panel_task.\n");
		return FALSE;
	}

	status = rtems_task_start(task_id, panel_task, 0);
	if (status != RTEMS_SUCCESSFUL)
	{
		TRACE("ERROR: cannot start panel_task.\n");
		return FALSE;
	}

	return TRUE;
}

rtems_task rtems_main(rtems_task_argument ignored)
{
	create_panel_task();
	rtems_task_delete(RTEMS_SELF);
}
