/*******************************************************************************
 * C H A M E L E O N    S. D. K.                                               *
 *******************************************************************************
 *  $Archive:: /Chameleon.sdk/src/examples/welcome/main.c                      $
 *     $Date:: 18/02/02 11:54                                                  $
 * $Revision:: 8                                                               $
 *-----------------------------------------------------------------------------*
 * This file is part of the Chameleon Software Development Kit                 *
 *                                                                             *
 * Copyright (C) 2001-2002 Soundart                                            *
 * www.soundart-hot.com                                                        *
 * support@soundart-hot.com                                                    *
 ******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <rtems.h>
#include <chameleon.h>

/******************************************************************************/
#define WORKSPACE_SIZE	128*1024
rtems_unsigned32 rtems_workspace_size = WORKSPACE_SIZE;
rtems_unsigned32 rtems_workspace_start[WORKSPACE_SIZE];
/******************************************************************************/

rtems_boolean show_info(char serial_number[12+1])
{
	rtems_unsigned8 model;

	if (!info_get_serial_number(serial_number) ||
	    !info_get_model(&model))
	{
		return FALSE;
	}

	TRACE("Serial Number: %s\n", serial_number);
	TRACE("Model:         model#%02X\n", model);
	TRACE("\n");

	return TRUE;
}

rtems_task rtems_main(rtems_task_argument ignored)
{
	rtems_unsigned8  potentiometer;
	rtems_unsigned8  value;
	rtems_unsigned32 key_bits;
	rtems_unsigned8  encoder;
	rtems_signed8	 increment;
	rtems_unsigned32 led_bits = 0;
	char		 serial_number[12+1];
	int		 panel;

	if (!show_info(serial_number))
	{
		TRACE("ERROR: can't access info driver.\n");
		rtems_task_delete(RTEMS_SELF);
	}

	panel = panel_init();

	if (panel)
	{
		while (TRUE)
		{
			if (!panel_in_new_event(panel, TRUE))
				break;

			if (panel_in_potentiometer(panel, &potentiometer, &value))
			{
			}
			else if (panel_in_keypad(panel, &key_bits))
			{
				if (key_bits & PANEL01_KEY_SHIFT)
				{
					char text[17];
					sprintf(text, "  %s  ", serial_number);
					panel_out_led(panel, led_bits | PANEL01_LED_SHIFT);
					panel_out_lcd_print(panel, 0, 0, " SERIAL NUMBER  ");
					panel_out_lcd_print(panel, 1, 0, text);
				}
				else
				{
					panel_out_lcd_print(panel, 0, 0, "  WELCOME TO    ");
					panel_out_lcd_print(panel, 1, 0, "     CHAMELEON  ");
					panel_out_led(panel, led_bits & ~PANEL01_LED_SHIFT);
				}
			}
			else if (panel_in_encoder(panel, &encoder, &increment))
			{
			}
		}
	}

	panel_exit(panel);

	rtems_task_delete(RTEMS_SELF);
}
